
LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;

ENTITY adc_spi IS
    PORT (
        CLK   : IN  STD_LOGIC;
        RST   : IN  STD_LOGIC;
        -- internal interface
        data  : IN  STD_LOGIC_VECTOR (7 DOWNTO 0);
        send  : IN  STD_LOGIC;
        busy  : OUT STD_LOGIC;
        -- ADS527x
        nCS   : OUT STD_LOGIC;
        SCLK  : OUT STD_LOGIC;
        SDATA : OUT STD_LOGIC;
        -- test connector
        TC    : OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
        );
END adc_spi;


ARCHITECTURE adc_spi_arch OF adc_spi IS

    CONSTANT last_bit : INTEGER := 8;
    SIGNAL   cnt      : INTEGER RANGE 0 TO 15;

    SIGNAL srg   : STD_LOGIC_VECTOR (7 DOWNTO 0);
    TYPE   state_type IS (IDLE, CLK_LOW, WAIT_LOW, CLK_HIGH, WAIT_HIGH);
    SIGNAL state : state_type;
    
BEGIN  -- adc_spi_arch

    PROCESS (CLK, RST)
    BEGIN  -- PROCESS
        IF RST = '0' THEN                   -- asynchronous reset (active low)
            state <= IDLE;
        ELSIF CLK'EVENT AND CLK = '1' THEN  -- rising clock edge
            --SDATA <= srg(7);
            CASE state IS
                WHEN IDLE =>
                    nCS  <= '1';
                    SCLK <= '1';
                    --SDATA <= '1';
                    srg  <= data;
                    cnt <= 0;
                    IF send = '1' THEN
                        state <= CLK_LOW;
                    END IF;
                WHEN CLK_LOW =>
                    nCS              <= '0';
                    SCLK             <= '0';
                    --SDATA <= '1';
                    SDATA <= srg(7);
                    cnt <= cnt + 1;
                    srg (7 DOWNTO 1) <= srg (6 DOWNTO 0);
                    state            <= WAIT_LOW;
                WHEN WAIT_LOW =>
                    nCS   <= '0';
                    SCLK  <= '0';
                    --SDATA <= '1';
                    state <= CLK_HIGH;
                WHEN CLK_HIGH =>
                    nCS   <= '0';
                    SCLK  <= '1';
                    --SDATA <= '1';
                    state <= WAIT_HIGH;
                WHEN WAIT_HIGH =>
                    nCS  <= '0';
                    SCLK <= '1';
                    --SDATA <= '1';
                    IF cnt = last_bit THEN
                        state <= IDLE;
                    ELSE
                        state <= CLK_LOW;
                    END IF;
                WHEN OTHERS =>
                    NULL;
            END CASE;
        END IF;
    END PROCESS;

    busy <= '0' WHEN state = IDLE ELSE '1';
    
END adc_spi_arch;
