LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_arith.ALL;
USE IEEE.std_logic_unsigned.ALL;

ENTITY clusterfinder IS
    PORT (
        CLK          : IN  STD_LOGIC;
        RST          : IN  STD_LOGIC;
        -- setup
        ring_thres   : IN  STD_LOGIC_VECTOR (8 DOWNTO 0);
        center_thres : IN  STD_LOGIC_VECTOR (8 DOWNTO 0);
        -- data in
        pixel_addr   : IN  STD_LOGIC_VECTOR (16 DOWNTO 0);
		  line0_in		: IN  STD_LOGIC_VECTOR (8 DOWNTO 0);
		  line1_in		: IN  STD_LOGIC_VECTOR (8 DOWNTO 0);
		  line2_in		: IN  STD_LOGIC_VECTOR (8 DOWNTO 0);
        -- clusters OUT
        cluster_addr : OUT STD_LOGIC_VECTOR (16 DOWNTO 0);
        cluster_we   : OUT STD_LOGIC;
        -- test connector
        TC           : OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
        );
END clusterfinder;

ARCHITECTURE clusterfinder_arch OF clusterfinder IS

    SIGNAL got_cluster : STD_LOGIC;

    SIGNAL center  : STD_LOGIC_VECTOR (8 DOWNTO 0);

	 --SIGNAL ring	: STD_LOGIC_VECTOR (8 DOWNTO 0);
	 
	 type cluster is array (integer range <>) of std_logic_vector(8 downto 0);
	 type addr is array (integer range <>) of std_logic_vector(16 downto 0);
 	 signal cluster9: cluster(8 downto 0);
	 signal clusterAddr: addr(1 downto 0);
	 
	 signal prev_line_addr: std_logic_vector(16 downto 0);
    
BEGIN  -- clusterfinder_arch
	

	
	
	
    PROCESS (CLK, RST)
        VARIABLE got_ring   : STD_LOGIC;
        VARIABLE got_center : STD_LOGIC;
		
    BEGIN  -- PROCESS CLK
        IF RST = '1' THEN               -- asynchronous reset (active high)

        ELSIF CLK'EVENT AND CLK = '1' THEN  -- rising clock edge
			
				-- propagate 3 pixels from 3 lines through a buffer 3 events deep
				cluster9(8 downto 0) <= cluster9(5 downto 0)& line2_in & line1_in & line0_in;
				
				-- the address of the pixel comes from the output of the line line sorter
				-- since that is the third line for a cluster structure
				-- it is necessary to retrieve the address of a previous line  : (-1)
				prev_line_addr <= (pixel_addr(16 DOWNTO 10)-1) & pixel_addr(9 DOWNTO 0);
				
				-- propagate the address of the central pixel to stay synchronized with data
				clusterAddr(1 downto 0) <= clusterAddr(0) & prev_line_addr;
				
				got_ring   := '0';
            	got_center := '0';
				
			-- BEGIN check the threshold levels
            FOR i IN 0 TO 3 LOOP
                IF cluster9(i) > ring_thres THEN
                    got_ring := '1';
                END IF;
            END LOOP;  -- i
			
			IF cluster9(4) > center_thres THEN	-- check the central pixel
                got_center := '1';
            END IF;
			
			FOR i IN 5 TO 8 LOOP
                IF cluster9(i) > ring_thres THEN
                    got_ring := '1';
                END IF;
            END LOOP;  -- i
			-- END check the threshold levels
				
			got_cluster <= got_center AND got_ring;
			
        END IF;
    END PROCESS;
cluster_addr <= clusterAddr(1);
cluster_we<= got_cluster;
END clusterfinder_arch;
