LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;


ENTITY daq IS
    PORT (
        CLK50           : IN    STD_LOGIC;
        CLK100          : IN    STD_LOGIC;
        CLK100_RAM_RD   : IN    STD_LOGIC;
        RST             : IN    STD_LOGIC;
        RST_DCM         : IN    STD_LOGIC;
        dcm_adc_locked  : OUT   STD_LOGIC;
        -- setup
        idle            : OUT   STD_LOGIC;
        fake_data_mode  : IN    STD_LOGIC;
        do_resync       : IN    STD_LOGIC;  -- when doing jtag
        resync_status   : OUT   STD_LOGIC;
        do_daq          : IN    STD_LOGIC;
        daq_busy        : OUT   STD_LOGIC;
        do_cds_daq      : IN    STD_LOGIC;
        cds_daq_busy    : OUT   STD_LOGIC;
        get_nios_io     : IN    STD_LOGIC;
        got_nios_io     : OUT   STD_LOGIC;
        do_clr_mem      : IN    STD_LOGIC;
        clr_mem_busy    : OUT   STD_LOGIC;
        do_full_frame   : IN    STD_LOGIC;
        full_frame_busy : OUT STD_LOGIC;
        daq_error       : OUT   STD_LOGIC;
        ack_daq_error   : IN    STD_LOGIC;
        do_mem_test     : IN    STD_LOGIC;
        mem_test_busy   : OUT   STD_LOGIC;
        mem_test_done   : OUT   STD_LOGIC;
        -- random read write
        nios_addr       : IN    STD_LOGIC_VECTOR (17 DOWNTO 0);
        nios_wr         : IN    STD_LOGIC;
        nios_data_wr    : IN    STD_LOGIC_VECTOR (143 DOWNTO 0);
        nios_data_rd    : OUT   STD_LOGIC_VECTOR (143 DOWNTO 0);
        -- full frame read
        adc_select      : IN    STD_LOGIC_VECTOR (2 DOWNTO 0);
        sort_select      : IN    STD_LOGIC_VECTOR (1 DOWNTO 0);
        full_frame_addr : OUT   STD_LOGIC_VECTOR (17 DOWNTO 0);
        full_frame_data : OUT   STD_LOGIC_VECTOR (11 DOWNTO 0);
        full_frame_wr   : OUT   STD_LOGIC;
        full_frame_done : OUT   STD_LOGIC;
        -- mem test
        mem_test_fail   : OUT   STD_LOGIC;
        -- mimostar2
        CkRdC           : OUT   STD_LOGIC;
        SYNC            : OUT   STD_LOGIC;
        MxFirst         : IN    STD_LOGIC;
        -- ADS5271
        ADCLK           : OUT   STD_LOGIC;
        Ads5273Rst      : OUT   STD_LOGIC;
        LclkP           : IN    STD_LOGIC;  -- Six times ADCLK
        LclkN           : IN    STD_LOGIC;
        AdClkP          : IN    STD_LOGIC;  -- Skewed and Data aligned version of the ADC sample clock. Used as strobe.
        AdClkN          : IN    STD_LOGIC;
        DatInCh0P       : IN    STD_LOGIC;
        DatInCh0N       : IN    STD_LOGIC;
        DatInCh1P       : IN    STD_LOGIC;
        DatInCh1N       : IN    STD_LOGIC;
        DatInCh2P       : IN    STD_LOGIC;
        DatInCh2N       : IN    STD_LOGIC;
        DatInCh3P       : IN    STD_LOGIC;
        DatInCh3N       : IN    STD_LOGIC;
        DatInCh4P       : IN    STD_LOGIC;
        DatInCh4N       : IN    STD_LOGIC;
        DatInCh5P       : IN    STD_LOGIC;
        DatInCh5N       : IN    STD_LOGIC;
        DatInCh6P       : IN    STD_LOGIC;
        DatInCh6N       : IN    STD_LOGIC;
        DatInCh7P       : IN    STD_LOGIC;
        DatInCh7N       : IN    STD_LOGIC;
        -- ADC5271 SPI
        -- sram
        RAM_A           : OUT   STD_LOGIC_VECTOR (17 DOWNTO 0);
        RAM_DQ          : INOUT STD_LOGIC_VECTOR (143 DOWNTO 0);
        RAM_ADV         : OUT   STD_LOGIC;
        RAM_CK_A        : OUT   STD_LOGIC;
        RAM_CK_B        : OUT   STD_LOGIC;
        RAM_E2          : OUT   STD_LOGIC;
        RAM_FT          : OUT   STD_LOGIC;
        RAM_G           : OUT   STD_LOGIC;
        RAM_LBO         : OUT   STD_LOGIC;
        RAM_RnW         : OUT   STD_LOGIC;
        RAM_PE          : OUT   STD_LOGIC;
        RAM_ZQ          : OUT   STD_LOGIC;
        -- to cluster finder
        pixel_addr : OUT STD_LOGIC_VECTOR (17 DOWNTO 0);
        cluster_we : OUT STD_LOGIC_VECTOR ( 7 DOWNTO 0);
        -- test connector
        TC              : OUT   STD_LOGIC_VECTOR (7 DOWNTO 0)
        );
END daq;

ARCHITECTURE daq_arch OF daq IS

    -- mimosstar2
    COMPONENT minostar2
        PORT (
            CLK50       : IN  STD_LOGIC;
            CLK100      : IN  STD_LOGIC;
            RST         : IN  STD_LOGIC;
            -- setup
            fake_chip   : IN  STD_LOGIC := '0';
            resync      : IN  STD_LOGIC;  -- when doing jtag
            -- mimostar2
            CkRdC       : OUT STD_LOGIC;
            SYNC        : OUT STD_LOGIC;
            MxFirst     : IN  STD_LOGIC;
            -- to memory
            first_pixel : OUT STD_LOGIC;
            frame_err   : OUT STD_LOGIC;
            -- test connector
            TC          : OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
            );
    END COMPONENT;

    -- ADS5271
    COMPONENT adc
        GENERIC (
--=============================================================================================================
            BramAsRegister : INTEGER := 1;
            FfAsRegister   : INTEGER := 0
            --  FfAsRegister  BramAsRegister                                            
            --  0             0               No register at all, direct out.
            --  0             1               Block SelectRAM used as register.
            --  1             0               Normal FFs register (look out for extra BUFG's).
            --  1             1               DO NOT USE !
--==============================================================================================================
            );
        PORT (
            -- FPGA interface
            CLK50      : IN  STD_LOGIC;
            CLK100     : IN  STD_LOGIC;
            RST        : IN  STD_LOGIC;
            DCM_locked : OUT STD_LOGIC;
            -- ADS interface
            ADCLK      : OUT STD_LOGIC;
            Ads5273Rst : OUT STD_LOGIC;
            LclkP      : IN  STD_LOGIC;  -- Six times ADCLK
            LclkN      : IN  STD_LOGIC;
            AdClkP     : IN  STD_LOGIC;  -- Skewed and Data aligned version of the ADC sample clock. Used as strobe.
            AdClkN     : IN  STD_LOGIC;
            DatInCh0P  : IN  STD_LOGIC;
            DatInCh0N  : IN  STD_LOGIC;
            DatInCh1P  : IN  STD_LOGIC;
            DatInCh1N  : IN  STD_LOGIC;
            DatInCh2P  : IN  STD_LOGIC;
            DatInCh2N  : IN  STD_LOGIC;
            DatInCh3P  : IN  STD_LOGIC;
            DatInCh3N  : IN  STD_LOGIC;
            DatInCh4P  : IN  STD_LOGIC;
            DatInCh4N  : IN  STD_LOGIC;
            DatInCh5P  : IN  STD_LOGIC;
            DatInCh5N  : IN  STD_LOGIC;
            DatInCh6P  : IN  STD_LOGIC;
            DatInCh6N  : IN  STD_LOGIC;
            DatInCh7P  : IN  STD_LOGIC;
            DatInCh7N  : IN  STD_LOGIC;
            -- ADS SPI interface
            -- data out
            adc_data0  : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
            adc_data1  : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
            adc_data2  : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
            adc_data3  : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
            adc_data4  : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
            adc_data5  : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
            adc_data6  : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
            adc_data7  : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
            -- Test Connector
            TC         : OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
            );
    END COMPONENT;

    -- fake adc data
    COMPONENT fake_adc
        PORT (
            CLK50    : IN  STD_LOGIC;
            RST      : IN  STD_LOGIC;
            -- resync
            mxfirst  : IN  STD_LOGIC;
            -- adc data
            adc_data : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
            -- test connector
            TC       : OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
            );
    END COMPONENT;

    -- SRAM
    COMPONENT SRAM
        PORT (
            CLK50            : IN    STD_LOGIC;
            CLK100           : IN    STD_LOGIC;
            CLK_RD           : IN    STD_LOGIC;
            RST              : IN    STD_LOGIC;
            -- setup
            do_daq           : IN    STD_LOGIC;
            daq_busy         : OUT   STD_LOGIC;
            DAQ_CDS          : IN    STD_LOGIC;
            IO_MODE          : IN    STD_LOGIC;
            do_clr_mem       : IN    STD_LOGIC;
            clr_mem_busy     : OUT   STD_LOGIC;
            ack_daq_error    : IN    STD_LOGIC;
            -- random read write
            got_io           : OUT   STD_LOGIC;
            io_wr            : IN    STD_LOGIC;
            io_addr          : IN    STD_LOGIC_VECTOR (17 DOWNTO 0);
            data_wr          : IN    STD_LOGIC_VECTOR (143 DOWNTO 0);
            data_rd          : OUT   STD_LOGIC_VECTOR (143 DOWNTO 0);
            -- adc interface
            first_pixel      : IN    STD_LOGIC;
            adc_data         : IN    STD_LOGIC_VECTOR (143 DOWNTO 0);
            -- CDS interface
            CDS_first_pixel  : OUT   STD_LOGIC;
            CDS_data_valid   : OUT   STD_LOGIC;
            CDS_adc_samples  : OUT   STD_LOGIC_VECTOR (143 DOWNTO 0);
            CDS_sram_samples : OUT   STD_LOGIC_VECTOR (143 DOWNTO 0);
            -- SDRAM chips
            RAM_A            : OUT   STD_LOGIC_VECTOR (17 DOWNTO 0);
            RAM_DQ           : INOUT STD_LOGIC_VECTOR (143 DOWNTO 0);
            RAM_ADV          : OUT   STD_LOGIC;
            RAM_CK_A         : OUT   STD_LOGIC;
            RAM_CK_B         : OUT   STD_LOGIC;
            RAM_E2           : OUT   STD_LOGIC;
            RAM_FT           : OUT   STD_LOGIC;
            RAM_G            : OUT   STD_LOGIC;
            RAM_LBO          : OUT   STD_LOGIC;
            RAM_RnW          : OUT   STD_LOGIC;
            RAM_PE           : OUT   STD_LOGIC;
            RAM_ZQ           : OUT   STD_LOGIC;
            -- TC
            TC               : OUT   STD_LOGIC_VECTOR (7 DOWNTO 0)
            );
    END COMPONENT;

    SIGNAL CDS_adc_samples  : STD_LOGIC_VECTOR (143 DOWNTO 0);
    SIGNAL CDS_sram_samples : STD_LOGIC_VECTOR (143 DOWNTO 0);

    -- calculate CDS diff for linebuffer
    COMPONENT cds_diff
        PORT (
            CLK50           : IN  STD_LOGIC;
            RST             : IN  STD_LOGIC;
            first_pixel_in  : IN  STD_LOGIC;
            first_pixel_out : OUT STD_LOGIC;
            data_adc        : IN  STD_LOGIC_VECTOR (11 DOWNTO 0);
            data_sram       : IN  STD_LOGIC_VECTOR (11 DOWNTO 0);
            data            : OUT STD_LOGIC_VECTOR (11 DOWNTO 0)
            );
    END COMPONENT;


    -- linebuffer

    -- Cluster finder

    -- io arbitration
    COMPONENT io_arbitriation
        PORT (
            CLK                   : IN  STD_LOGIC;
            RST                   : IN  STD_LOGIC;
            -- SRAM io PORT
            get_io                : OUT STD_LOGIC;
            got_io                : IN  STD_LOGIC;
            io_wr                 : OUT STD_LOGIC;
            io_addr               : OUT STD_LOGIC_VECTOR (17 DOWNTO 0);
            io_data_wr            : OUT STD_LOGIC_VECTOR (143 DOWNTO 0);
            io_data_rd            : IN  STD_LOGIC_VECTOR (143 DOWNTO 0);
            -- full frame
            get_io_full_frame     : IN  STD_LOGIC;
            got_io_full_frame     : OUT STD_LOGIC;
            io_addr_full_frame    : IN  STD_LOGIC_VECTOR (17 DOWNTO 0);
            io_data_rd_full_frame : OUT STD_LOGIC_VECTOR (143 DOWNTO 0);
            -- nios IO
            get_io_nios           : IN  STD_LOGIC;
            got_io_nios           : OUT STD_LOGIC;
            io_wr_nios            : IN  STD_LOGIC;
            io_addr_nios          : IN  STD_LOGIC_VECTOR (17 DOWNTO 0);
            io_data_wr_nios       : IN  STD_LOGIC_VECTOR (143 DOWNTO 0);
            io_data_rd_nios       : OUT STD_LOGIC_VECTOR (143 DOWNTO 0);
            -- mem test
            get_io_mem_test       : IN  STD_LOGIC;
            got_io_mem_test       : OUT STD_LOGIC;
            io_wr_mem_test        : IN  STD_LOGIC;
            io_addr_mem_test      : IN  STD_LOGIC_VECTOR (17 DOWNTO 0);
            io_data_wr_mem_test   : IN  STD_LOGIC_VECTOR (143 DOWNTO 0);
            io_data_rd_mem_test   : OUT STD_LOGIC_VECTOR (143 DOWNTO 0);
            -- test connector
            TC                    : OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
            );
    END COMPONENT;

    -- full frame read
    COMPONENT full_frame_rd
        PORT (
            CLK              : IN  STD_LOGIC;
            RST              : IN  STD_LOGIC;
            -- setup
            do_read          : IN  STD_LOGIC;
            read_done        : OUT STD_LOGIC;
            sorted           : IN  STD_LOGIC;
            classic_sort     : IN  STD_LOGIC;
            adc_select       : IN  STD_LOGIC_VECTOR (2 DOWNTO 0);
            -- SRAM
            get_io           : OUT STD_LOGIC;
            got_io           : IN  STD_LOGIC;
            data_io          : IN  STD_LOGIC_VECTOR (143 DOWNTO 0);
            addr_io          : OUT STD_LOGIC_VECTOR (17 DOWNTO 0);
            -- FIFO
            read_addr        : OUT STD_LOGIC_VECTOR (17 DOWNTO 0);
            fifo_data        : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
            fifo_wr          : OUT STD_LOGIC;
            fifo_almost_full : IN  STD_LOGIC;
            -- test connector
            TC               : OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
            );
    END COMPONENT;

    -- memory test
    COMPONENT hft_mem_test
        PORT (
            CLK           : IN  STD_LOGIC;
            RST           : IN  STD_LOGIC;
            --
            do_mem_test   : IN  STD_LOGIC;
            mem_test_busy : OUT STD_LOGIC;
            mem_test_done : OUT STD_LOGIC;
            status        : OUT STD_LOGIC;
            -- failed tests
            fail_cnt      : OUT STD_LOGIC;
            fail_ff00     : OUT STD_LOGIC;
            fail_wr_rd    : OUT STD_LOGIC;
            -- clear mem
            clrmem        : OUT STD_LOGIC;
            clrmem_busy   : IN  STD_LOGIC;
            -- sram
            sram_idle     : IN  STD_LOGIC;
            get_io        : OUT STD_LOGIC;
            got_io        : IN  STD_LOGIC;
            addr          : OUT STD_LOGIC_VECTOR (17 DOWNTO 0);
            data_wr       : OUT STD_LOGIC_VECTOR (143 DOWNTO 0);
            data_rd       : IN  STD_LOGIC_VECTOR (143 DOWNTO 0);
            io_wr         : OUT STD_LOGIC;
            --
            tc            : OUT STD_LOGIC
            );
    END COMPONENT;


    -- write hot pixel


    -- adc data
    SIGNAL fake_adc_data : STD_LOGIC_VECTOR (11 DOWNTO 0);
    SIGNAL adc_data0     : STD_LOGIC_VECTOR (11 DOWNTO 0);
    SIGNAL adc_data1     : STD_LOGIC_VECTOR (11 DOWNTO 0);
    SIGNAL adc_data2     : STD_LOGIC_VECTOR (11 DOWNTO 0);
    SIGNAL adc_data3     : STD_LOGIC_VECTOR (11 DOWNTO 0);
    SIGNAL adc_data4     : STD_LOGIC_VECTOR (11 DOWNTO 0);
    SIGNAL adc_data5     : STD_LOGIC_VECTOR (11 DOWNTO 0);
    SIGNAL adc_data6     : STD_LOGIC_VECTOR (11 DOWNTO 0);
    SIGNAL adc_data7     : STD_LOGIC_VECTOR (11 DOWNTO 0);
    SIGNAL sram_adc_data : STD_LOGIC_VECTOR (143 DOWNTO 0);

    SIGNAL mimostar2_first_pixel : STD_LOGIC;

    SIGNAL clr_mem_int      : STD_LOGIC;
    SIGNAL clr_mem_busy_int : STD_LOGIC;
    -- random IO
    SIGNAL get_io           : STD_LOGIC;
    SIGNAL got_io           : STD_LOGIC;
    SIGNAL io_wr            : STD_LOGIC;
    SIGNAL io_addr          : STD_LOGIC_VECTOR (17 DOWNTO 0);
    SIGNAL io_data_wr       : STD_LOGIC_VECTOR (143 DOWNTO 0);
    SIGNAL io_data_rd       : STD_LOGIC_VECTOR (143 DOWNTO 0);

    -- full frame
    SIGNAL get_io_full_frame     : STD_LOGIC;
    SIGNAL got_io_full_frame     : STD_LOGIC;
    SIGNAL io_addr_full_frame    : STD_LOGIC_VECTOR (17 DOWNTO 0);
    SIGNAL io_data_rd_full_frame : STD_LOGIC_VECTOR (143 DOWNTO 0);

    -- nios IO
    SIGNAL get_io_nios     : STD_LOGIC;
    SIGNAL got_io_nios     : STD_LOGIC;
    SIGNAL io_addr_nios    : STD_LOGIC_VECTOR (17 DOWNTO 0);
    SIGNAL io_data_rd_nios : STD_LOGIC_VECTOR (143 DOWNTO 0);
    SIGNAL io_data_wr_nios : STD_LOGIC_VECTOR (143 DOWNTO 0);
    SIGNAL io_wr_nios      : STD_LOGIC;

    -- mem test
    SIGNAL mem_test_clr_mem : STD_LOGIC;
    SIGNAL fail_cnt         : STD_LOGIC;
    SIGNAL fail_ff00        : STD_LOGIC;
    SIGNAL fail_wr_rd       : STD_LOGIC;

    SIGNAL RSTn : STD_LOGIC;

BEGIN  -- daq_arch

    RSTn <= NOT RST;

    -- mimosstar2
    mimosstar2_inst : minostar2
        PORT MAP (
            CLK50       => CLK50,
            CLK100      => CLK100,
            RST         => RST,
            -- setup
            fake_chip   => fake_data_mode,
            resync      => do_resync,
            -- mimostar2
            CkRdC       => CkRdC,
            SYNC        => SYNC,
            MxFirst     => MxFirst,
            -- to memory
            first_pixel => mimostar2_first_pixel,
            frame_err   => OPEN,
            -- test connector
            TC          => OPEN
            );

    -- ADS5271
    adc_inst : adc
        GENERIC MAP (
            BramAsRegister => 0,
            FfAsRegister   => 1
            )
        PORT MAP (
            -- FPGA interface
            CLK50      => CLK50,
            CLK100     => CLK100,
            RST        => RST,
            DCM_locked => dcm_adc_locked,
            -- ADS interface
            ADCLK      => ADCLK,
            Ads5273Rst => Ads5273Rst,
            LclkP      => LclkP,
            LclkN      => LclkN,
            AdClkP     => AdClkP,
            AdClkN     => AdClkN,
            DatInCh0P  => DatInCh0P,
            DatInCh0N  => DatInCh0N,
            DatInCh1P  => DatInCh1P,
            DatInCh1N  => DatInCh1N,
            DatInCh2P  => DatInCh2P,
            DatInCh2N  => DatInCh2N,
            DatInCh3P  => DatInCh3P,
            DatInCh3N  => DatInCh3N,
            DatInCh4P  => DatInCh4P,
            DatInCh4N  => DatInCh4N,
            DatInCh5P  => DatInCh5P,
            DatInCh5N  => DatInCh5N,
            DatInCh6P  => DatInCh6P,
            DatInCh6N  => DatInCh6N,
            DatInCh7P  => DatInCh7P,
            DatInCh7N  => DatInCh7N,
            -- ADS SPI interface
            -- data out
            adc_data0  => adc_data0,
            adc_data1  => adc_data1,
            adc_data2  => adc_data2,
            adc_data3  => adc_data3,
            adc_data4  => adc_data4,
            adc_data5  => adc_data5,
            adc_data6  => adc_data6,
            adc_data7  => adc_data7,
            -- Test Connector
            TC         => OPEN
            );

    -- fake adc data
    fake_adc_inst : fake_adc
        PORT MAP (
            CLK50    => CLK50,
            RST      => RST,
            -- resync
            mxfirst  => mimostar2_first_pixel,
            -- adc data
            adc_data => fake_adc_data,
            -- test connector
            TC       => OPEN
            );


    -- ADC data mux for fake data mode
    PROCESS (CLK50, RST)
    BEGIN  -- PROCESS
        IF RST = '1' THEN               -- asynchronous reset (active high)
            sram_adc_data <= (OTHERS => '0');
        ELSIF CLK50'EVENT AND CLK50 = '1' THEN  -- rising clock edge
            --IF fake_data_mode = '1' THEN
            IF 0 = 1 THEN
                sram_adc_data                 <= (OTHERS => '0');
                sram_adc_data(11 DOWNTO 0)    <= fake_adc_data;
                sram_adc_data(27 DOWNTO 16)   <= fake_adc_data;
                sram_adc_data(43 DOWNTO 32)   <= fake_adc_data;
                sram_adc_data(59 DOWNTO 48)   <= fake_adc_data;
                sram_adc_data(75 DOWNTO 64)   <= fake_adc_data;
                sram_adc_data(91 DOWNTO 80)   <= fake_adc_data;
                sram_adc_data(107 DOWNTO 96)  <= fake_adc_data;
                sram_adc_data(123 DOWNTO 112) <= fake_adc_data;
            ELSE
                sram_adc_data                 <= (OTHERS => '0');
                sram_adc_data(11 DOWNTO 0)    <= adc_data0;
                sram_adc_data(27 DOWNTO 16)   <= adc_data1;
                sram_adc_data(43 DOWNTO 32)   <= adc_data2;
                sram_adc_data(59 DOWNTO 48)   <= adc_data3;
                sram_adc_data(75 DOWNTO 64)   <= adc_data4;
                sram_adc_data(91 DOWNTO 80)   <= adc_data5;
                sram_adc_data(107 DOWNTO 96)  <= adc_data6;
                sram_adc_data(123 DOWNTO 112) <= adc_data7;
            END IF;
        END IF;
    END PROCESS;

    -- SRAM
    SRAM_inst : SRAM
        PORT MAP (
            CLK50            => CLK50,
            CLK100           => CLK100,
            CLK_RD           => CLK100_RAM_RD,
            RST              => RST,
            -- setup
            do_daq           => do_daq,
            daq_busy         => daq_busy,
            DAQ_CDS          => '0',
            IO_MODE          => get_io,
            do_clr_mem       => clr_mem_int,  -- mem_test_clr_mem, --do_clr_mem,
            clr_mem_busy     => clr_mem_busy_int,  --clr_mem_busy,
            ack_daq_error    => ack_daq_error,
            -- random read write
            got_io           => got_io,
            io_wr            => io_wr,
            io_addr          => io_addr,
            data_wr          => io_data_wr,
            data_rd          => io_data_rd,
            -- adc interface
            first_pixel      => mimostar2_first_pixel,
            adc_data         => sram_adc_data,
            -- CDS interface
            CDS_first_pixel  => OPEN,
            CDS_data_valid   => OPEN,
            CDS_adc_samples  => CDS_adc_samples,
            CDS_sram_samples => CDS_sram_samples,
            -- SDRAM chips
            RAM_A            => RAM_A,
            RAM_DQ           => RAM_DQ,
            RAM_ADV          => RAM_ADV,
            RAM_CK_A         => RAM_CK_A,
            RAM_CK_B         => RAM_CK_B,
            RAM_E2           => RAM_E2,
            RAM_FT           => RAM_FT,
            RAM_G            => RAM_G,
            RAM_LBO          => RAM_LBO,
            RAM_RnW          => RAM_RnW,
            RAM_PE           => RAM_PE,
            RAM_ZQ           => RAM_ZQ,
            -- TC
            TC               => OPEN
            );
    clr_mem_int  <= mem_test_clr_mem OR do_clr_mem;
    clr_mem_busy <= clr_mem_busy_int;

    -- calculate CDS diff for linebuffer
    cds_diff_inst : cds_diff
        PORT MAP (
            CLK50           => CLK50,
            RST             => RST,
            first_pixel_in  => '0',
            first_pixel_out => OPEN,
            data_adc        => CDS_adc_samples(11 DOWNTO 0),
            data_sram       => CDS_sram_samples(11 DOWNTO 0),
            data            => OPEN
            );


    -- linebuffer

    -- cluster finder

    -- random IO arbitration
    io_arbitriation_inst : io_arbitriation
        PORT MAP (
            CLK                   => CLK100,
            RST                   => RST,
            -- SRAM io PORT
            get_io                => get_io,
            got_io                => got_io,
            io_wr                 => io_wr,
            io_addr               => io_addr,
            io_data_wr            => io_data_wr,
            io_data_rd            => io_data_rd,
            -- full frame
            get_io_full_frame     => get_io_full_frame,
            got_io_full_frame     => got_io_full_frame,
            io_addr_full_frame    => io_addr_full_frame,
            io_data_rd_full_frame => io_data_rd_full_frame,
            -- nios IO
            get_io_nios           => get_io_nios,
            got_io_nios           => got_io_nios,
            io_wr_nios            => io_wr_nios,
            io_addr_nios          => io_addr_nios,
            io_data_wr_nios       => io_data_wr_nios,
            io_data_rd_nios       => io_data_rd_nios,
            -- mem test
            get_io_mem_test       => '0',
            got_io_mem_test       => OPEN,
            io_wr_mem_test        => '0',
            io_addr_mem_test      => "00"&x"0000",
            io_data_wr_mem_test   => x"00000000"&x"00000000"&x"00000000"&x"00000000"&x"0000",
            io_data_rd_mem_test   => OPEN,
            -- test connector
            TC                    => OPEN
            );

    -- full frame read
    full_frame_rd_inst : full_frame_rd
        PORT MAP (
            CLK              => CLK50,
            RST              => RST,
            -- setup
            do_read          => do_full_frame,
            read_done        => full_frame_done,
            sorted           => '0',
            classic_sort     => '1',
            adc_select       => adc_select,
            -- SRAM
            get_io           => get_io_full_frame,
            got_io           => got_io_full_frame,
            data_io          => io_data_rd_full_frame,
            addr_io          => io_addr_full_frame,
            -- FIFO
            read_addr        => full_frame_addr,
            fifo_data        => full_frame_data,
            fifo_wr          => full_frame_wr,
            fifo_almost_full => '0',
            -- test connector
            TC               => OPEN
            );

    -- memory test
    hft_mem_test_inst : hft_mem_test
        PORT MAP (
            CLK           => CLK100,
            RST           => RSTn,
            --
            do_mem_test   => do_mem_test,
            mem_test_busy => mem_test_busy,
            mem_test_done => mem_test_done,
            status        => OPEN,
            -- failed tests
            fail_cnt      => fail_cnt,
            fail_ff00     => fail_ff00,
            fail_wr_rd    => fail_wr_rd,
            -- clear mem
            clrmem        => mem_test_clr_mem,
            clrmem_busy   => clr_mem_busy_int,
            -- sram
            sram_idle     => '1',
            get_io        => get_io_nios,
            got_io        => got_io_nios,
            addr          => io_addr_nios,
            data_wr       => io_data_wr_nios,
            data_rd       => io_data_rd_nios,
            io_wr         => io_wr_nios,
            --
            tc            => OPEN
            );
    mem_test_fail <= fail_cnt OR fail_ff00 OR fail_wr_rd;


    -- write hot pixel

    TC(0) <= '1' WHEN CDS_adc_samples = x"000000000000000000000000000000000000" OR CDS_sram_samples = x"000000000000000000000000000000000000" ELSE '0';


END daq_arch;










