LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_unsigned.ALL;
USE IEEE.std_logic_arith.ALL;


ENTITY fake_adc IS
    PORT (
        CLK50   : IN  STD_LOGIC;
        RST     : IN  STD_LOGIC;
        -- resync
        mxfirst : IN  STD_LOGIC;
        -- adc data
        adc_data : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
        -- test connector
        TC      : OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
        );
END fake_adc;

ARCHITECTURE fake_adc_arch OF fake_adc IS

    SIGNAL pixel_cnt : STD_LOGIC_VECTOR (11 DOWNTO 0);
    SIGNAL frame_cnt : STD_LOGIC_VECTOR (11 DOWNTO 0);
    
BEGIN  -- fake_adc_arch

    PROCESS (CLK50, RST)
    BEGIN  -- PROCESS
        IF RST = '1' THEN               -- asynchronous reset (active HIGH)
            pixel_cnt <= (OTHERS => '0');
            frame_cnt <= (OTHERS => '0');
        ELSIF CLK50'EVENT AND CLK50 = '1' THEN  -- rising clock edge
            IF mxfirst = '1' THEN
                pixel_cnt <= x"001";
                frame_cnt <= frame_cnt + 1;
            ELSE
                pixel_cnt <= pixel_cnt + 1;
            END IF;
        END IF;
    END PROCESS;

    adc_data <= pixel_cnt + (2**conv_integer(frame_cnt));

END fake_adc_arch;
