LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_unsigned.ALL;
USE IEEE.std_logic_arith.ALL;

ENTITY full_frame_rd IS
    PORT (
        CLK              : IN  STD_LOGIC;
        RST              : IN  STD_LOGIC;
        -- setup
        do_read          : IN  STD_LOGIC;
        read_done        : OUT STD_LOGIC;
        sorted           : IN  STD_LOGIC;
        classic_sort     : IN  STD_LOGIC;
        adc_select       : IN  STD_LOGIC_VECTOR (2 DOWNTO 0);
        -- SRAM
        get_io           : OUT STD_LOGIC;
        got_io           : IN  STD_LOGIC;
        data_io          : IN  STD_LOGIC_VECTOR (143 DOWNTO 0);
        addr_io          : OUT STD_LOGIC_VECTOR (17 DOWNTO 0);
        -- FIFO
        read_addr        : OUT STD_LOGIC_VECTOR (17 DOWNTO 0);
        fifo_data        : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
        fifo_wr          : OUT STD_LOGIC;
        fifo_almost_full : IN  STD_LOGIC;
        -- test connector
        TC               : OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
        );
END full_frame_rd;

ARCHITECTURE full_frame_rd_arch OF full_frame_rd IS

--    SIGNAL clk   : STD_LOGIC := '0';
    TYPE   statetype IS (IDLE, GET_IO_MODE, FULL_FRAME_RD, FULL_FRAME_ACK);
    SIGNAL state : statetype := IDLE;

    SIGNAL lines      : INTEGER RANGE 0 TO 127;
    SIGNAL pixels     : INTEGER RANGE 0 TO 65;
    SIGNAL subframes  : INTEGER RANGE 0 TO 9;
    SIGNAL marker_cnt : INTEGER RANGE -1 TO 19;

    SIGNAL frame_rd : STD_LOGIC := '0';

--    SIGNAL fifo_almost_full : STD_LOGIC := '0';
--    SIGNAL sorted           : STD_LOGIC := '0';
    SIGNAL classic : STD_LOGIC := '1';

--    SIGNAL got_io : STD_LOGIC := '1';

    SIGNAL fifo_wr_sig : STD_LOGIC;
    SIGNAL ram_we      : STD_LOGIC;
    SIGNAL write_sig   : STD_LOGIC;
    SIGNAL RAM_ADDR    : STD_LOGIC_VECTOR (17 DOWNTO 0);
    SIGNAL RAM_ADDR_conc    : STD_LOGIC_VECTOR (17 DOWNTO 0);
    SIGNAL IO_MODE     : STD_LOGIC;
    
BEGIN  -- tmp_arch

--    clk      <= NOT clk AFTER 5 ns;
--    frame_rd <= '1'     AFTER 250 ns, '0' AFTER 900 us, '1' AFTER 1 ms;
--    sorted   <= '1'     AFTER 900 us;

    addr_io   <= RAM_ADDR;
    read_addr <= RAM_ADDR;

    get_io <= IO_MODE;

    frame_rd <= do_read;
    classic  <= classic_sort;

    fifo_wr <= fifo_wr_sig;

    PROCESS(CLK, RST)
    BEGIN
        IF RST = '1' THEN
            read_done <= '0';
            state     <= IDLE;
        ELSIF CLK'EVENT AND CLK = '1' THEN
            read_done <= '0';
            CASE state IS
                WHEN IDLE =>
                    IO_MODE <= '0';
                    lines   <= 0;
                    IF classic = '1' THEN
                        pixels <= 0;
                    ELSE
                        pixels <= 2;
                    END IF;
                    subframes  <= 0;
                    marker_cnt <= -1;
                    RAM_ADDR   <= (OTHERS => '0');
                    IF frame_rd = '1' THEN
                        state <= GET_IO_MODE;
                    END IF;
                WHEN GET_IO_MODE =>
                    IO_MODE  <= '1';
                    RAM_ADDR <= (OTHERS => '0');
                    IF got_io = '1' THEN
                        state <= FULL_FRAME_RD;
                    END IF;
                WHEN FULL_FRAME_RD =>
                    IO_MODE <= '1';
                    IF frame_rd = '0' THEN
                        fifo_wr_sig <= '0';
                        ram_we      <= '1';
                        write_sig   <= '0';
                        RAM_ADDR    <= (OTHERS => '0');
                        state       <= IDLE;
                    ELSE
                        IF fifo_almost_full = '1' THEN
                            fifo_wr_sig <= '0';
                        ELSE
                            fifo_wr_sig <= '1';
                            IF sorted = '0' THEN
                                ram_we    <= '1';
                                write_sig <= '0';
                                -- do not sort pixels
                                RAM_ADDR  <= RAM_ADDR + 1;
                                IF RAM_ADDR = 660*128-2 THEN
                                    state <= FULL_FRAME_ACK;
                                ELSE
                                    state <= FULL_FRAME_RD;
                                END IF;
                            ELSE
                                -- sort pixels
                                IF lines = 128-1 AND pixels = 66-1 AND subframes = 10-1 AND (marker_cnt = 20-1 OR classic = '1') THEN
                                    state <= FULL_FRAME_ACK;
                                ELSE
                                    IF pixels = 66-1 AND subframes = 10-1 THEN
                                        IF marker_cnt = 20 -1 OR classic = '1' THEN
                                            lines <= lines + 1;
                                            IF classic = '1' THEN
                                                pixels <= 0;
                                            ELSE
                                                pixels <= 2;
                                            END IF;
                                            subframes <= 0;
                                        ELSE
                                            marker_cnt <= marker_cnt + 1;
                                        END IF;
                                    ELSE
                                        marker_cnt <= -1;
                                        IF subframes = 10-1 THEN
                                            pixels    <= pixels + 1;
                                            subframes <= 0;
                                        ELSE
                                            subframes <= subframes + 1;
                                        END IF;
                                    END IF;
                                    --        IF pixels = 66-1 AND subframes = 10-1 AND marker_cnt /= -1 THEN
                                    --            RAM_ADDR <= conv_std_logic_vector(lines*10*66 + marker_cnt, 18);
                                    --        ELSE
                                    --            RAM_ADDR <= conv_std_logic_vector(lines*(10*66) + subframes*66 + pixels, 18);
                                    --        END IF;

                                    state <= FULL_FRAME_RD;
                                END IF;
                                IF pixels = 66-1 AND subframes = 10-1 AND marker_cnt /= -1 THEN
                                    RAM_ADDR <= conv_std_logic_vector(lines*10*66 + marker_cnt, 18);
                                ELSE
 --                                  RAM_ADDR <= conv_std_logic_vector(lines*(10*66) + subframes*66 + pixels, 18);
                    RAM_ADDR <=RAM_ADDR_conc;
                                END IF;
                            END IF;
                        END IF;
                    END IF;
                WHEN FULL_FRAME_ACK =>
--                                   RAM_ADDR <= conv_std_logic_vector(lines*(10*66) + subframes*66 + pixels, 18);
                    read_done   <= '1';
                    IO_MODE     <= '1';
                    fifo_wr_sig <= '0';
                    ram_we      <= '1';
                    write_sig   <= '0';
--                    RAM_ADDR    <= (OTHERS => '0');
--                    RAM_ADDR    <= conv_std_logic_vector(lines*(10*66) + subframes*66 + pixels, 18);
                    RAM_ADDR <=RAM_ADDR_conc;
                    IF frame_rd = '0' THEN
                        state <= IDLE;
                    END IF;
                WHEN OTHERS =>
                    NULL;
            END CASE;
        END IF;
    END PROCESS;

    -- because Xilinx ISE 7.1 breakes otherwise
--   RAM_ADDR_conc    <= conv_std_logic_vector(lines*(10*66) + subframes*66 + pixels, 18);


    
    PROCESS (CLK, RST)
    BEGIN  -- PROCESS CLK
        IF RST = '1' THEN                   -- asynchronous reset (active high)
            fifo_data <= (OTHERS => '0');
        ELSIF CLK'EVENT AND CLK = '1' THEN  -- rising clock edge
--            fifo_data <= data_io(CONV_INTEGER(adc_select)*16+11 DOWNTO CONV_INTEGER(adc_select)*16);
            fifo_data <= data_io(11 DOWNTO 0);
        END IF;
    END PROCESS;

END full_frame_rd_arch;



