LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_UNSIGNED.ALL;
USE IEEE.std_logic_arith.ALL;


ENTITY hft_adc_test IS
    PORT (
        CLK_p       : IN  STD_LOGIC;
        CLK_n       : IN  STD_LOGIC;
        -- ADS interface
        ADC_CLK     : OUT STD_LOGIC;
        ADC_RESET_N : OUT STD_LOGIC;
        ADC_LCLK_P  : IN  STD_LOGIC;    -- Six times ADCLK
        ADC_LCLK_N  : IN  STD_LOGIC;
        ADC_ADCLK_P : IN  STD_LOGIC;  -- Skewed and Data aligned version of the ADC sample clock. Used as strobe.
        ADC_ADCLK_N : IN  STD_LOGIC;
        ADC_D1_P    : IN  STD_LOGIC;
        ADC_D1_N    : IN  STD_LOGIC;
        ADC_D2_P    : IN  STD_LOGIC;
        ADC_D2_N    : IN  STD_LOGIC;
        ADC_D3_P    : IN  STD_LOGIC;
        ADC_D3_N    : IN  STD_LOGIC;
        ADC_D4_P    : IN  STD_LOGIC;
        ADC_D4_N    : IN  STD_LOGIC;
        ADC_D5_P    : IN  STD_LOGIC;
        ADC_D5_N    : IN  STD_LOGIC;
        ADC_D6_P    : IN  STD_LOGIC;
        ADC_D6_N    : IN  STD_LOGIC;
        ADC_D7_P    : IN  STD_LOGIC;
        ADC_D7_N    : IN  STD_LOGIC;
        ADC_D8_P    : IN  STD_LOGIC;
        ADC_D8_N    : IN  STD_LOGIC;
        -- ADS527x
        nCS         : OUT STD_LOGIC;
        SCLK        : OUT STD_LOGIC;
        SDATA       : OUT STD_LOGIC;
        -- LEDs
        LED         : OUT STD_LOGIC_VECTOR (10 DOWNTO 1);
        TC          : OUT STD_LOGIC_VECTOR(7 DOWNTO 0)
        );
END hft_adc_test;

ARCHITECTURE arch_hft_adc_test OF hft_adc_test IS

    COMPONENT dcm100to50
        PORT (
            CLKIN_N_IN        : IN  STD_LOGIC;
            CLKIN_P_IN        : IN  STD_LOGIC;
            RST_IN            : IN  STD_LOGIC;
            CLKDV_OUT         : OUT STD_LOGIC;
            CLKIN_IBUFGDS_OUT : OUT STD_LOGIC;
            CLK0_OUT          : OUT STD_LOGIC;
            LOCKED_OUT        : OUT STD_LOGIC
            );
    END COMPONENT;

    SIGNAL CLK50 : STD_LOGIC;

    COMPONENT adc
        GENERIC (
            --=============================================================================================================
            BramAsRegister : INTEGER := 1;
            FfAsRegister   : INTEGER := 0
            --  FfAsRegister  BramAsRegister                                            
            --  0             0               No register at all, direct out.
            --  0             1               Block SelectRAM used as register.
            --  1             0               Normal FFs register (look out for extra BUFG's).
            --  1             1               DO NOT USE !
            --==============================================================================================================
            );
        PORT (
            -- FPGA interface
            CLK50      : IN  STD_LOGIC;
            CLK100     : IN  STD_LOGIC;
            RST        : IN  STD_LOGIC;
            DCM_locked : OUT STD_LOGIC;
            -- ADS interface
            ADCLK      : OUT STD_LOGIC;
            Ads5273Rst : OUT STD_LOGIC;
            LclkP      : IN  STD_LOGIC;  -- Six times ADCLK
            LclkN      : IN  STD_LOGIC;
            AdClkP     : IN  STD_LOGIC;  -- Skewed and Data aligned version of the ADC sample clock. Used as strobe.
            AdClkN     : IN  STD_LOGIC;
            DatInCh0P  : IN  STD_LOGIC;
            DatInCh0N  : IN  STD_LOGIC;
            DatInCh1P  : IN  STD_LOGIC;
            DatInCh1N  : IN  STD_LOGIC;
            DatInCh2P  : IN  STD_LOGIC;
            DatInCh2N  : IN  STD_LOGIC;
            DatInCh3P  : IN  STD_LOGIC;
            DatInCh3N  : IN  STD_LOGIC;
            DatInCh4P  : IN  STD_LOGIC;
            DatInCh4N  : IN  STD_LOGIC;
            DatInCh5P  : IN  STD_LOGIC;
            DatInCh5N  : IN  STD_LOGIC;
            DatInCh6P  : IN  STD_LOGIC;
            DatInCh6N  : IN  STD_LOGIC;
            DatInCh7P  : IN  STD_LOGIC;
            DatInCh7N  : IN  STD_LOGIC;
            -- ADS SPI interface
            -- data out
            adc_data0  : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
            adc_data1  : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
            adc_data2  : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
            adc_data3  : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
            adc_data4  : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
            adc_data5  : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
            adc_data6  : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
            adc_data7  : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
            -- Test Connector
            TC         : OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
            );
    END COMPONENT;

    SIGNAL ADC0 : STD_LOGIC_VECTOR (11 DOWNTO 0);
    SIGNAL ADC1 : STD_LOGIC_VECTOR (11 DOWNTO 0);
    SIGNAL ADC2 : STD_LOGIC_VECTOR (11 DOWNTO 0);
    SIGNAL ADC3 : STD_LOGIC_VECTOR (11 DOWNTO 0);
    SIGNAL ADC4 : STD_LOGIC_VECTOR (11 DOWNTO 0);
    SIGNAL ADC5 : STD_LOGIC_VECTOR (11 DOWNTO 0);
    SIGNAL ADC6 : STD_LOGIC_VECTOR (11 DOWNTO 0);
    SIGNAL ADC7 : STD_LOGIC_VECTOR (11 DOWNTO 0);

    SIGNAL TC_adc : STD_LOGIC_VECTOR (7 DOWNTO 0);


    COMPONENT set_adc_spi
        PORT (
            CLK50 : IN  STD_LOGIC;
            RST   : IN  STD_LOGIC;
            -- ADS527x
            nCS   : OUT STD_LOGIC;
            SCLK  : OUT STD_LOGIC;
            SDATA : OUT STD_LOGIC;
            -- test connector
            TC    : OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
            );
    END COMPONENT;

    SIGNAL RST : STD_LOGIC;

    
BEGIN  -- arch_hft_adc_test

    PROCESS (CLK50)
        VARIABLE cnt : INTEGER RANGE 0 TO 31 := 0;
    BEGIN  -- PROCESS
        IF CLK50'EVENT AND CLK50 = '1' THEN  -- rising clock edge
            IF cnt < 31 THEN
                cnt := cnt + 1;
                RST <= '0';
            ELSE
                RST <= '1';
            END IF;
        END IF;
    END PROCESS;

    inst_dcm100to50 : dcm100to50
        PORT MAP (
            CLKIN_N_IN        => CLK_n,
            CLKIN_P_IN        => CLK_p,
            RST_IN            => '0',
            CLKDV_OUT         => CLK50,
            CLKIN_IBUFGDS_OUT => OPEN,
            CLK0_OUT          => OPEN,
            LOCKED_OUT        => OPEN
            );

    inst_adc : adc
        GENERIC MAP (
            BramAsRegister => 0,
            FfAsRegister   => 1
            )
        PORT MAP (
            -- FPGA interface
            CLK50      => CLK50,
            CLK100     => '0',
            RST        => NOT RST,
            DCM_locked => open,
            -- ADS interface
            ADCLK      => ADC_CLK,
            Ads5273Rst => ADC_RESET_N,
            LclkP      => ADC_LCLK_P,
            LclkN      => ADC_LCLK_N,
            AdClkP     => ADC_ADCLK_P,
            AdClkN     => ADC_ADCLK_N,
            DatInCh0P  => ADC_D1_P,
            DatInCh0N  => ADC_D1_N,
            DatInCh1P  => ADC_D2_P,
            DatInCh1N  => ADC_D2_N,
            DatInCh2P  => ADC_D3_P,
            DatInCh2N  => ADC_D3_N,
            DatInCh3P  => ADC_D4_P,
            DatInCh3N  => ADC_D4_N,
            DatInCh4P  => ADC_D5_P,
            DatInCh4N  => ADC_D5_N,
            DatInCh5P  => ADC_D6_P,
            DatInCh5N  => ADC_D6_N,
            DatInCh6P  => ADC_D7_P,
            DatInCh6N  => ADC_D7_N,
            DatInCh7P  => ADC_D8_P,
            DatInCh7N  => ADC_D8_N,
            -- ADS SPI interface
            -- data out
            adc_data0  => ADC0,
            adc_data1  => ADC1,
            adc_data2  => ADC2,
            adc_data3  => ADC3,
            adc_data4  => ADC4,
            adc_data5  => ADC5,
            adc_data6  => ADC6,
            adc_data7  => ADC7,
            -- Test Connector
            TC         => TC_adc
            );


    LED(1) <= '1' WHEN adc0 = 0 ELSE '0';
    LED(2) <= '1' WHEN adc1 = 0 ELSE '0';
    LED(3) <= '1' WHEN adc2 = 0 ELSE '0';
    LED(4) <= '1' WHEN adc3 = 0 ELSE '0';
    LED(5) <= '1' WHEN adc4 = 0 ELSE '0';
    LED(6) <= '1' WHEN adc5 = 0 ELSE '0';
    LED(7) <= '1' WHEN adc6 = 0 ELSE '0';
    LED(8) <= '1' WHEN adc7 = 0 ELSE '0';

--    LED(9)  <= '1';
--    LED(10) <= '1';

    PROCESS (CLK50)
        VARIABLE cnt    : INTEGER   := 0;
        VARIABLE LEDint : STD_LOGIC := '0';
    BEGIN  -- PROCESS
        IF CLK50'EVENT AND CLK50 = '1' THEN  -- rising clock edge
            IF cnt < 10000000 THEN
                cnt := cnt + 1;
            ELSE
                cnt    := 0;
                LEDint := NOT LEDint;
            END IF;
            LED(10) <= LEDint; -- OR NOT TC_adc(2);
        END IF;
    END PROCESS;

    PROCESS (TC_adc(0))
        VARIABLE cnt    : INTEGER   := 0;
        VARIABLE LEDint : STD_LOGIC := '0';
    BEGIN  -- PROCESS
        IF TC_adc(0)'EVENT AND TC_adc(0) = '1' THEN  -- rising clock edge
            IF cnt < 10000000 THEN
                cnt := cnt + 1;
            ELSE
                cnt    := 0;
                LEDint := NOT LEDint;
            END IF;
            LED(9) <= LEDint;
        END IF;
    END PROCESS;

    TC(0) <= TC_adc(1);


    set_adc_spi_inst : set_adc_spi
        PORT MAP (
            CLK50 => CLK50,
            RST   => '0',               -- turned off
            -- ADS527x
            nCS   => nCS,
            SCLK  => SCLK,
            SDATA => SDATA,
            -- test connector
            TC    => OPEN
            );


END arch_hft_adc_test;

