LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_unsigned.ALL;
USE IEEE.std_logic_arith.ALL;

ENTITY hft_top IS
    PORT (
        CLK_p    : IN    STD_LOGIC;
        CLK_n    : IN    STD_LOGIC;
        -- SRAM
        RAM_A    : OUT   STD_LOGIC_VECTOR (17 DOWNTO 0);
        RAM_DQ   : INOUT STD_LOGIC_VECTOR (143 DOWNTO 0);
        RAM_ADV  : OUT   STD_LOGIC;
        RAM_CK_A : OUT   STD_LOGIC;
        RAM_CK_B : OUT   STD_LOGIC;
        RAM_E2   : OUT   STD_LOGIC;
        RAM_FT   : OUT   STD_LOGIC;
        RAM_G    : OUT   STD_LOGIC;
        RAM_LBO  : OUT   STD_LOGIC;
        RAM_RnW  : OUT   STD_LOGIC;
        RAM_PE   : OUT   STD_LOGIC;
        RAM_ZQ   : OUT   STD_LOGIC;
        -- LEDs
        LED      : OUT   STD_LOGIC_VECTOR (10 DOWNTO 1);
        -- TC
        TC       : OUT   STD_LOGIC_VECTOR (7 DOWNTO 0)
        );
END hft_top;

ARCHITECTURE hft_top_arch OF hft_top IS

    COMPONENT SRAM
        PORT (
            CLK100          : IN    STD_LOGIC;
        CLK_RD          : IN    STD_LOGIC;
            RST             : IN    STD_LOGIC;
            -- setup
            DAQ_MODE        : IN    STD_LOGIC;
            IO_MODE         : IN    STD_LOGIC;
            IO_WR           : IN    STD_LOGIC;
            CLR_MODE        : IN    STD_LOGIC;
            CLR_BUSY        : OUT   STD_LOGIC;
            ACK_ERROR       : IN    STD_LOGIC;
            -- random read write
            io_addr         : IN    STD_LOGIC_VECTOR (17 DOWNTO 0);
            data_wr         : IN    STD_LOGIC_VECTOR (143 DOWNTO 0);
            data_rd         : OUT   STD_LOGIC_VECTOR (143 DOWNTO 0);
            -- adc interface
            first_pixel     : IN    STD_LOGIC;
            -- CDS interface
            CDS_first_pixel : OUT   STD_LOGIC;
            CDS_data_valid  : OUT   STD_LOGIC;
            --        CDS_adc_samples :       ADC_VECTOR;
            -- SDRAM chips
            RAM_A           : OUT   STD_LOGIC_VECTOR (17 DOWNTO 0);
            RAM_DQ          : INOUT STD_LOGIC_VECTOR (143 DOWNTO 0);
            RAM_ADV         : OUT   STD_LOGIC;
            RAM_CK_A        : OUT   STD_LOGIC;
            RAM_CK_B        : OUT   STD_LOGIC;
            RAM_E2          : OUT   STD_LOGIC;
            RAM_FT          : OUT   STD_LOGIC;
            RAM_G           : OUT   STD_LOGIC;
            RAM_LBO         : OUT   STD_LOGIC;
            RAM_RnW         : OUT   STD_LOGIC;
            RAM_PE          : OUT   STD_LOGIC;
            RAM_ZQ          : OUT   STD_LOGIC;
            -- TC
            TC              : OUT   STD_LOGIC_VECTOR (7 DOWNTO 0)
            );
    END COMPONENT;

    COMPONENT mem_test
        PORT (
            CLK     : IN  STD_LOGIC;
            RST     : IN  STD_LOGIC;
            --
            status  : OUT STD_LOGIC;
            --
            addr    : OUT STD_LOGIC_VECTOR (17 DOWNTO 0);
            data_wr : OUT STD_LOGIC_VECTOR (143 DOWNTO 0);
            data_rd : IN  STD_LOGIC_VECTOR (143 DOWNTO 0);
            clrmem  : OUT STD_LOGIC;
            io_mode : OUT STD_LOGIC;
            io_wr   : OUT STD_LOGIC;
            -- led
            LED     : OUT STD_LOGIC_VECTOR (10 DOWNTO 1);
            --
            tc      : OUT STD_LOGIC
            );
    END COMPONENT;

    COMPONENT dcm100mhz IS
        PORT (
            CLKIN_N_IN        : IN  STD_LOGIC;
            CLKIN_P_IN        : IN  STD_LOGIC;
            --RST_IN            : IN  STD_LOGIC;
            CLKIN_IBUFGDS_OUT : OUT STD_LOGIC;
            CLK0_OUT          : OUT STD_LOGIC;
            CLK90_OUT         : OUT STD_LOGIC;
            CLK180_OUT        : OUT STD_LOGIC;
            CLK270_OUT        : OUT STD_LOGIC;
            LOCKED_OUT        : OUT STD_LOGIC
            );
    END COMPONENT;


--    component IBUF_LVDS_33
--        port (
--I : in std_logic;
--O : out std_logic
--);
--end component;

    COMPONENT IBUFDS
        PORT (O  : OUT STD_ULOGIC;
              I  : IN  STD_ULOGIC;
              IB : IN  STD_ULOGIC);
    END COMPONENT;


    SIGNAL CLK : STD_LOGIC;
    SIGNAL CLK_0 : STD_LOGIC;
    SIGNAL CLK_RD : STD_LOGIC;

    SIGNAL RST     : STD_LOGIC;
    SIGNAL RST_cnt : INTEGER RANGE 0 TO 3 := 3;

    SIGNAL CLR_MODE : STD_LOGIC := '0';
    SIGNAL IO_MODE  : STD_LOGIC := '0';
    SIGNAL IO_WR    : STD_LOGIC := '0';

    SIGNAL addr    : STD_LOGIC_VECTOR (23 DOWNTO 0);
    SIGNAL data_wr : STD_LOGIC_VECTOR (143 DOWNTO 0);
    SIGNAL data_rd : STD_LOGIC_VECTOR (143 DOWNTO 0);

    SIGNAL LEDint : STD_LOGIC_VECTOR (10 DOWNTO 1);
    SIGNAL TCint  : STD_LOGIC_VECTOR (7 DOWNTO 0);

    --ATTRIBUTE iostandard : STRING;
    -- ATTRIBUTE iostandard OF CLKp: label IS LVDS_25;
    
BEGIN  -- hft_top_arch

    --CLK <= CLK_p;
    --clkp : IBUF_LVDS_33 port map (I=>CLK_p, O=>CLK);
    --clkp : IBUFDS PORT MAP (CLK, CLK_p, CLK_n);

    dcm100mhz_inst : dcm100mhz
        PORT MAP (
            CLKIN_N_IN        => CLK_n,
            CLKIN_P_IN        => CLK_p,
            --RST_IN            => RST,
            CLKIN_IBUFGDS_OUT => CLK,
            CLK0_OUT          => CLK_0,
            CLK90_OUT         => open, --CLK_RD,
            CLK180_OUT        => OPEN, --CLK_RD, --open,
            CLK270_OUT        => CLK_RD, --open,
            LOCKED_OUT        => open
            );
    --CLK_RD <= CLK_0;

    
    PROCESS (CLK)
    BEGIN  -- PROCESS
        IF CLK'EVENT AND CLK = '1' THEN  -- rising clock edge
            IF RST_cnt /= 0 THEN
                RST_cnt <= RST_cnt - 1;
                RST     <= '0';
            ELSE
                RST <= '1';
            END IF;
        END IF;
    END PROCESS;


--    CLR_MODE <= '1' AFTER 10000 ns, '0' AFTER 20000 ns;
--    IO_MODE  <= '1' AFTER 5000 us;

--    PROCESS (CLK, RST)
--    BEGIN  -- PROCESS
--        IF RST = '0' THEN                   -- asynchronous reset (active low)
--            addr <= (OTHERS => '0');
--        ELSIF CLK'EVENT AND CLK = '1' THEN  -- rising clock edge
--            addr <= addr + 1;
--        END IF;
--    END PROCESS;

    SRAM_inst : SRAM
        PORT MAP (
            CLK100          => CLK_0,
        CLK_RD          => CLK_RD,
            RST             => RST,
            -- setup
            DAQ_MODE        => '0',
            IO_MODE         => IO_MODE,
            IO_WR           => IO_WR,
            CLR_MODE        => CLR_MODE,
            CLR_BUSY        => OPEN,
            ACK_ERROR       => '0',
            -- random read write
            io_addr         => addr (17 DOWNTO 0),  --(OTHERS => '0'),
            data_wr         => data_wr,
            data_rd         => data_rd,
            -- adc interface
            first_pixel     => '0',
            -- CDS interface
            CDS_first_pixel => OPEN,
            CDS_data_valid  => OPEN,
            --        CDS_adc_samples :       ADC_VECTOR;
            -- SDRAM chips
            RAM_A           => RAM_A,
            RAM_DQ          => RAM_DQ,
            RAM_ADV         => RAM_ADV,
            RAM_CK_A        => RAM_CK_A,
            RAM_CK_B        => RAM_CK_B,
            RAM_E2          => RAM_E2,
            RAM_FT          => RAM_FT,
            RAM_G           => RAM_G,
            RAM_LBO         => RAM_LBO,
            RAM_RnW         => RAM_RnW,
            RAM_PE          => RAM_PE,
            RAM_ZQ          => RAM_ZQ,
            -- TC
            TC              => TCint
            );

    inst_mem_test : mem_test
        PORT MAP (
            CLK     => CLK_0,
            RST     => RST,
            --
            status  => OPEN,
            --
            addr    => addr (17 DOWNTO 0),
            data_wr => data_wr,
            data_rd => data_rd,
            clrmem  => CLR_MODE,
            io_mode => IO_MODE,
            io_wr   => IO_WR,
            -- led
            LED     => LEDint,
            --
            tc      => OPEN
            );

    PROCESS (LEDint, TCint)
    BEGIN  -- PROCESS
        LED    <= LEDint;
        LED(7) <= TCint(0);
    END PROCESS;


END hft_top_arch;









