LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_unsigned.ALL;
USE IEEE.std_logic_arith.ALL;

LIBRARY UNISIM;
USE UNISIM.ALL;


ENTITY hft_daughtercard_top IS
    PORT (
        CLK_p       : IN    STD_LOGIC;
        CLK_n       : IN    STD_LOGIC;
        -- Mimostar 2
        CkRdC_p     : OUT   STD_LOGIC;
        CkRdC_n     : OUT   STD_LOGIC;
        SYNC_p      : OUT   STD_LOGIC;
        SYNC_n      : OUT   STD_LOGIC;
        MxFirst     : IN    STD_LOGIC;
        RSTB        : OUT   STD_LOGIC;
        -- ADS interface
        ADC_CLK     : OUT   STD_LOGIC;
        ADC_RESET_N : OUT   STD_LOGIC;
        ADC_LCLK_P  : IN    STD_LOGIC;  -- Six times ADCLK
        ADC_LCLK_N  : IN    STD_LOGIC;
        ADC_ADCLK_P : IN    STD_LOGIC;  -- Skewed and Data aligned version of the ADC sample clock. Used as strobe.
        ADC_ADCLK_N : IN    STD_LOGIC;
        ADC_D1_P    : IN    STD_LOGIC;
        ADC_D1_N    : IN    STD_LOGIC;
        ADC_D2_P    : IN    STD_LOGIC;
        ADC_D2_N    : IN    STD_LOGIC;
        ADC_D3_P    : IN    STD_LOGIC;
        ADC_D3_N    : IN    STD_LOGIC;
        ADC_D4_P    : IN    STD_LOGIC;
        ADC_D4_N    : IN    STD_LOGIC;
        ADC_D5_P    : IN    STD_LOGIC;
        ADC_D5_N    : IN    STD_LOGIC;
        ADC_D6_P    : IN    STD_LOGIC;
        ADC_D6_N    : IN    STD_LOGIC;
        ADC_D7_P    : IN    STD_LOGIC;
        ADC_D7_N    : IN    STD_LOGIC;
        ADC_D8_P    : IN    STD_LOGIC;
        ADC_D8_N    : IN    STD_LOGIC;
        -- ADS527x SPI
        nCS         : OUT   STD_LOGIC;
        SCLK        : OUT   STD_LOGIC;
        SDATA       : OUT   STD_LOGIC;
        -- SRAM
        RAM_A       : OUT   STD_LOGIC_VECTOR (17 DOWNTO 0);
        RAM_DQ      : INOUT STD_LOGIC_VECTOR (143 DOWNTO 0);
        RAM_ADV     : OUT   STD_LOGIC;
        RAM_CK_A    : OUT   STD_LOGIC;
        RAM_CK_B    : OUT   STD_LOGIC;
        RAM_E2      : OUT   STD_LOGIC;
        RAM_FT      : OUT   STD_LOGIC;
        RAM_G       : OUT   STD_LOGIC;
        RAM_LBO     : OUT   STD_LOGIC;
        RAM_RnW     : OUT   STD_LOGIC;
        RAM_PE      : OUT   STD_LOGIC;
        RAM_ZQ      : OUT   STD_LOGIC;
        -- LEDs
 		leo	: INout	STD_LOGIC_VECTOR(60 downto 1) ;-- the data I.E. the address of cluster center
		sleo	: INout	STD_LOGIC_VECTOR(72 downto 62) ;-- the data I.E. the address of cluster center
		TCD	: IN	STD_LOGIC_VECTOR(5 downto 0) 

        );
END hft_daughtercard_top;




ARCHITECTURE hft_daughtercard_top_arch OF hft_daughtercard_top IS

    COMPONENT ROC
        -- synthesis translate_off 
        GENERIC (WIDTH :     TIME       := 100 ns);
        -- synthesis translate_on 
        PORT (O        : OUT STD_ULOGIC := '1');
    END COMPONENT;

    COMPONENT IBUFGDS_LVDS_33
        PORT (
            I  : IN  STD_LOGIC;
            IB : IN  STD_LOGIC;
            O  : OUT STD_LOGIC
            );
    END COMPONENT;

    COMPONENT OBUFDS_LVDS_33
        PORT(
            O  : OUT STD_ULOGIC;
            OB : OUT STD_ULOGIC;
            I  : IN  STD_ULOGIC
            );
    END COMPONENT;

    COMPONENT dcm50to100
        PORT (
  U1_CLKIN_N_IN        : in    std_logic; 
	       U1_CLKIN_P_IN        : in    std_logic; 
	            RHIC_CLK_IN   : in    std_logic; 
            CLK_SELECTION_IN : in    std_logic; 
           --U1_CLKIN_IN        : IN  STD_LOGIC;
            U1_RST_IN            : IN  STD_LOGIC;
            --U1_CLKIN_IBUFG_OUT : OUT STD_LOGIC;
            U1_CLKIN_IBUFGDS_OUT : OUT STD_LOGIC;
            --U1_CLK0_OUT          : OUT STD_LOGIC;
            U1_CLK2X_OUT         : OUT STD_LOGIC;
            U1_STATUS_OUT        : OUT STD_LOGIC_VECTOR (2 DOWNTO 0);
            U2_CLKDV_OUT         : OUT STD_LOGIC;
            U2_CLK0_OUT          : OUT STD_LOGIC;
            U2_CLK270_OUT        : OUT STD_LOGIC;
            U2_LOCKED_OUT        : OUT STD_LOGIC;
            U2_STATUS_OUT        : OUT STD_LOGIC_VECTOR (2 DOWNTO 0)
            );
    END COMPONENT;

    COMPONENT daq
        PORT (
            CLK50           : IN    STD_LOGIC;
            CLK100          : IN    STD_LOGIC;
            CLK100_RAM_RD   : IN    STD_LOGIC;
            RST             : IN    STD_LOGIC;
            RST_DCM         : IN    STD_LOGIC;
            dcm_adc_locked  : OUT   STD_LOGIC;
            -- setup
            idle            : OUT   STD_LOGIC;
            fake_data_mode  : IN    STD_LOGIC;
            do_resync       : IN    STD_LOGIC;  -- when doing jtag
            resync_status   : OUT   STD_LOGIC;
            do_daq          : IN    STD_LOGIC;
            daq_busy        : OUT   STD_LOGIC;
            do_cds_daq      : IN    STD_LOGIC;
            cds_daq_busy    : OUT   STD_LOGIC;
            get_nios_io     : IN    STD_LOGIC;
            got_nios_io     : OUT   STD_LOGIC;
            do_clr_mem      : IN    STD_LOGIC;
            clr_mem_busy    : OUT   STD_LOGIC;
            daq_error       : OUT   STD_LOGIC;
            ack_daq_error   : IN    STD_LOGIC;
            do_mem_test     : IN    STD_LOGIC;
            mem_test_busy   : OUT   STD_LOGIC;
            mem_test_done   : OUT   STD_LOGIC;
            -- random read write
            nios_addr       : IN    STD_LOGIC_VECTOR (17 DOWNTO 0);
            nios_wr         : IN    STD_LOGIC;
            nios_data_wr    : IN    STD_LOGIC_VECTOR (143 DOWNTO 0);
            nios_data_rd    : OUT   STD_LOGIC_VECTOR (143 DOWNTO 0);
            -- full frame read
            do_full_frame   : IN    STD_LOGIC;
            full_frame_busy : OUT   STD_LOGIC;
            adc_select      : IN    STD_LOGIC_VECTOR (2 DOWNTO 0);
        sort_select      : IN    STD_LOGIC_VECTOR (3 DOWNTO 0);
            full_frame_addr : OUT   STD_LOGIC_VECTOR (17 DOWNTO 0);
            full_frame_data : OUT   STD_LOGIC_VECTOR (11 DOWNTO 0);
            full_frame_wr   : OUT   STD_LOGIC;
            full_frame_done : OUT   STD_LOGIC;
	         frame_data    : OUT   STD_LOGIC_VECTOR (143 DOWNTO 0);
		      frame_address    : OUT   STD_LOGIC_VECTOR (17 DOWNTO 0);
           -- mem test
            mem_test_fail   : OUT   STD_LOGIC;
            -- mimostar2
            CkRdC           : OUT   STD_LOGIC;
            SYNC            : OUT   STD_LOGIC;
            MxFirst         : IN    STD_LOGIC;
            -- ADS5271
            ADCLK           : OUT   STD_LOGIC;
            Ads5273Rst      : OUT   STD_LOGIC;
            LclkP           : IN    STD_LOGIC;  -- Six times ADCLK
            LclkN           : IN    STD_LOGIC;
            AdClkP          : IN    STD_LOGIC;  -- Skewed and Data aligned version of the ADC sample clock. Used as strobe.
            AdClkN          : IN    STD_LOGIC;
            DatInCh0P       : IN    STD_LOGIC;
            DatInCh0N       : IN    STD_LOGIC;
            DatInCh1P       : IN    STD_LOGIC;
            DatInCh1N       : IN    STD_LOGIC;
            DatInCh2P       : IN    STD_LOGIC;
            DatInCh2N       : IN    STD_LOGIC;
            DatInCh3P       : IN    STD_LOGIC;
            DatInCh3N       : IN    STD_LOGIC;
            DatInCh4P       : IN    STD_LOGIC;
            DatInCh4N       : IN    STD_LOGIC;
            DatInCh5P       : IN    STD_LOGIC;
            DatInCh5N       : IN    STD_LOGIC;
            DatInCh6P       : IN    STD_LOGIC;
            DatInCh6N       : IN    STD_LOGIC;
            DatInCh7P       : IN    STD_LOGIC;
            DatInCh7N       : IN    STD_LOGIC;
            -- sram
            RAM_A           : OUT   STD_LOGIC_VECTOR (17 DOWNTO 0);
            RAM_DQ          : INOUT STD_LOGIC_VECTOR (143 DOWNTO 0);
            RAM_ADV         : OUT   STD_LOGIC;
            RAM_CK_A        : OUT   STD_LOGIC;
            RAM_CK_B        : OUT   STD_LOGIC;
            RAM_E2          : OUT   STD_LOGIC;
            RAM_FT          : OUT   STD_LOGIC;
            RAM_G           : OUT   STD_LOGIC;
            RAM_LBO         : OUT   STD_LOGIC;
            RAM_RnW         : OUT   STD_LOGIC;
            RAM_PE          : OUT   STD_LOGIC;
            RAM_ZQ          : OUT   STD_LOGIC;
            -- random IO
            -- to cluster finder
            pixel_addr      : OUT   STD_LOGIC_VECTOR (17 DOWNTO 0);
            cluster_we      : OUT   STD_LOGIC_VECTOR (7 DOWNTO 0);
            -- test connector
		 		align : out std_logic;
        		threshold_i : in std_logic_vector (8 downto 0);
				threshold_o : out std_logic_vector (143 downto 0);
        		adc_o : out std_logic_vector (71 downto 0);
            TC              : OUT   STD_LOGIC_VECTOR (7 DOWNTO 0)
            );
    END COMPONENT;

    COMPONENT set_adc_spi IS
        PORT (
            CLK50 : IN  STD_LOGIC;
            RST   : IN  STD_LOGIC;
            -- ADS527x
            conf  : IN  STD_LOGIC_VECTOR(2 DOWNTO 0);
            nCS   : OUT STD_LOGIC;
            SCLK  : OUT STD_LOGIC;
            SDATA : OUT STD_LOGIC;
				            -- test connector
            TC    : OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
            );
    END COMPONENT;



	 --------------------------------------------------------------------------
	 -------for SXM
	 ---------------------------------------------------------------------------
	 component clusterfifos
generic (
	ADDRSIZE 	: integer  :=9 ;
	DATASIZE 	: integer  :=32;
	WAIT_WINDOW_LENGTH 	: integer  :=6000;
	CLUSTER_WINDOW_LENGTH	:	integer :=10;
	FIFO_LENGTH	:	integer :=200;
	CHIP_ID	:	integer :=10
	 );
port (
 	reset_n : in std_logic ;

-- clocks
	pix_clk 	: in std_logic ; -- same clock that moves pixels through the cluster finder
--	dbdb_clk	: in std_lobic ; -- read end clk, Daughter Board Data Bus

-- trigger data
	trg_word: in std_logic_vector(31 downto 0) ;	
	thresholdr_i: in std_logic_vector(8 downto 0) ;	
	thresholdc_i: in std_logic_vector(8 downto 0) ;	
	adc_i: in std_logic_vector(11 downto 0) ;	

--   idle_state : in std_logic_vector(4 downto 0);
														
															-- this gets loaded into the beginning of each
															-- event block 
--	chip_id : in std_logic_vector(11 downto 0) ; -- ladder(11 downto 6) & chip(5 downto 0)
												-- this also gets loaded at the beginning of each block
															
-- input data and interface control
	data_in: in std_logic_vector(17 downto 0) ;															
	got_cluster	: in std_logic;  --from cluster finder, found one
	start			: in std_logic; -- contains TRG command, DAQ command, token
--	got_cluster : in std_logic; -- from cluster finder, found one
	idle_flag	:out std_logic; --true when module is in idle waiting for trigger
--	writting_flag	:out std_logic; --true when module is in idle waiting for trigger
--	do_aborts	:	in std_logic; -- a control bit to enable module to respond to trigger aborts
	
-- output data and interface control
--	address		: in  std_logic_vector(5 downto 0);-- each CFM for the ladder has unique address
		fifo_data1			: INout	STD_LOGIC ;-- the data I.E. the address of cluster center
		fifo_data2			: INout	STD_LOGIC_VECTOR(19 downto 0) ;-- the data I.E. the address of cluster center
		fifo_data3			: INout	STD_LOGIC ;-- the data I.E. the address of cluster center
		fifo_data4			: INout	STD_LOGIC ;-- the data I.E. the address of cluster center
		fifo_data5			: INout	STD_LOGIC_VECTOR(8 downto 0) ;-- the data I.E. the address of cluster center
	read_ready	: out std_logic; --true when module is ready to be read out, tri stated.
	sum_ready	: out std_logic; --true when module is ready to be read out, tri stated.
	rdreq	:	in std_logic; -- true causes fifo to clock data, q, out until empty (when this CFM addressed)

----new 
	read_clk 		: in std_logic;
 
 	chip_address	: in std_logic_vector(6 downto 0)
		   
		);
end component;
component writedeligater 
    Port ( 
		 	reset_n : in std_logic ;
			 clk : in std_logic;
           trg_come : in std_logic;
           trigger_counter : out std_logic_vector(11 downto 0);
           fifo_select : out std_logic_vector(4 downto 0);
           idle_state : in std_logic_vector(4 downto 0)--;
			  --busy_state : in std_logic_vector(4 downto 0)
		  
			  );

end component;
component ClusterGeneratorFake 
port (
	reset_n : in std_logic ;
	clk 	: in std_logic ; -- fast pixel clock, ultimately 100 MHz
	PixelAddress : out std_logic_vector(17 downto 0); -- circular memory address'
	PixelAddressto : out std_logic_vector(17 downto 0); -- circular memory address'
	GotCluster : out std_logic -- flag indicating cluster finder has found a cluster
) ;
end component ;

component switch 
    Port ( clk : in std_logic;
           reset : in std_logic;
--			   ava_clk : in std_logic;
        RST             : OUT    STD_LOGIC;
        RST_DCM         : OUT    STD_LOGIC;
        dcm_adc_locked  : IN   STD_LOGIC;
        -- setup
        idle            : IN   STD_LOGIC;
        fake_data_mode  : OUT    STD_LOGIC;
        do_resync       : OUT    STD_LOGIC;  -- when doOUTg jtag
        resync_status   : IN   STD_LOGIC;
        do_daq          : OUT    STD_LOGIC;
        daq_busy        : IN   STD_LOGIC;
        do_cds_daq      : OUT    STD_LOGIC;
        cds_daq_busy    : IN   STD_LOGIC;
        get_nios_io     : OUT    STD_LOGIC;
        got_nios_io     : IN   STD_LOGIC;
        do_clr_mem      : OUT    STD_LOGIC;
        clr_mem_busy    : IN   STD_LOGIC;
        do_full_frame   : OUT    STD_LOGIC;
        full_frame_busy : IN STD_LOGIC;
        daq_error       : IN   STD_LOGIC;
        ack_daq_error   : OUT    STD_LOGIC;
        do_mem_test     : OUT    STD_LOGIC;
        mem_test_busy   : IN   STD_LOGIC;
        mem_test_done   : IN   STD_LOGIC;
        -- random read write
        nios_addr       : OUT    STD_LOGIC_VECTOR (17 DOWNTO 0);
        nios_wr         : OUT    STD_LOGIC;
        nios_data_wr    : OUT    STD_LOGIC_VECTOR (143 DOWNTO 0);
        nios_data_rd    : IN   STD_LOGIC_VECTOR (143 DOWNTO 0);
        -- full frame read
        adc_select      : OUT    STD_LOGIC_VECTOR (2 DOWNTO 0);
        sort_select      : OUT    STD_LOGIC_VECTOR (3 DOWNTO 0);
        full_frame_addr : IN   STD_LOGIC_VECTOR (17 DOWNTO 0);
        full_frame_data : IN   STD_LOGIC_VECTOR (11 DOWNTO 0);
        full_frame_wr   : IN   STD_LOGIC;
        full_frame_done : IN   STD_LOGIC;
 	         frame_data    : in   STD_LOGIC_VECTOR (143 DOWNTO 0);
		      frame_address    : in   STD_LOGIC_VECTOR (17 DOWNTO 0);
       -- mem test
        mem_test_fail   : IN   STD_LOGIC;
        -- mimostar2
        CkRdC           : IN   STD_LOGIC;
        SYNC            : IN   STD_LOGIC;
        MxFirst         : OUT    STD_LOGIC;
        pixel_addr : IN STD_LOGIC_VECTOR (17 DOWNTO 0);
        cluster_we : IN STD_LOGIC_VECTOR ( 7 DOWNTO 0);
        ostatus : IN STD_LOGIC_VECTOR (31 DOWNTO 0);

           addr : in std_logic_vector(6 downto 0);
		fifo_data1			: INout	STD_LOGIC ;-- the data I.E. the address of cluster center
		fifo_data2			: INout	STD_LOGIC_VECTOR(19 downto 0) ;-- the data I.E. the address of cluster center
		fifo_data3			: INout	STD_LOGIC ;-- the data I.E. the address of cluster center
		fifo_data4			: INout	STD_LOGIC ;-- the data I.E. the address of cluster center
		fifo_data5			: INout	STD_LOGIC_VECTOR(8 downto 0) ;-- the data I.E. the address of cluster center
           req : in std_logic;
           ready : out std_logic;
        threshold_o : out std_logic_vector (8 downto 0);
		  	Token			: in std_logic_vector (11 downto 0);	
			trg_come		: in std_logic	;
--			read_ready		: in STD_LOGIC_VECTOR(4 downto 0); 
--         fifo_addr_out		: out STD_LOGIC_VECTOR(6 downto 0);
   ready_flag		: in STD_LOGIC; 
 trigger_flag		: in STD_LOGIC 
						  );
end component;
component myfifo
	PORT
	(
		data		: IN STD_LOGIC_VECTOR (31 DOWNTO 0);
		wrreq		: IN STD_LOGIC ;
		rdreq		: IN STD_LOGIC ;
		rdclk		: IN STD_LOGIC ;
		wrclk		: IN STD_LOGIC ;
		aclr		: IN STD_LOGIC  := '0';
		q			: OUT STD_LOGIC_VECTOR (31 DOWNTO 0);
		rdempty		: OUT STD_LOGIC ;
		wrfull		: OUT STD_LOGIC 
	);
end component;

component trigger 
    Port ( clk : in std_logic;
           rhic_clk : in std_logic;
           reset_n : in std_logic;
           trig : in std_logic_vector(3 downto 0);
           trig_word : out std_logic_vector(11 downto 0);
           trig_come : out std_logic);
end component;
component trigger_decoder 
    port (
         RHIC			: IN std_logic ;	  		-- ~ 10 MHz in
			RHICx5		: IN std_logic ;	  		-- 5 time RHIC clock in
         rst	      : IN std_logic; 			-- System reset
			Data			: IN std_logic_vector(3 downto 0);
			Token			: OUT std_logic_vector (11 downto 0);
			comd			: OUT std_logic_vector (3 downto 0);	
			trg_come		: OUT std_logic	
    );
end component;
component BUFG
--   generic  (
--      IOSTANDARD => "DEFAULT") ;
   port (
      O : OUT std_logic ; -- Clock buffer output
      I : IN std_logic  -- Clock buffer input (connect directly to top-level port)
   );
end component;

component BUFGMUX
   port  (
      O : OUT std_logic ; -- Clock buffer output
      I0 : IN std_logic ;  -- Clock0 input
      I1 : IN std_logic ;  -- Clock1 input
      S : IN std_logic      -- Clock select input
   );
end component;
 component IBUFGDS
  port (O : out STD_ULOGIC;
	I : in STD_ULOGIC;
	IB : in STD_ULOGIC);
 end component;

    SIGNAL CLKin         : STD_LOGIC;
    SIGNAL CLK50         : STD_LOGIC;
    SIGNAL CLK100        : STD_LOGIC;
    SIGNAL CLK100_RAM_RD : STD_LOGIC;
    SIGNAL RST           : STD_LOGIC;
    SIGNAL RSTn          : STD_LOGIC;

    SIGNAL dcm_sys_locked : STD_LOGIC;
    SIGNAL dcm_adc_locked : STD_LOGIC;

    -- Mimostar2
    SIGNAL CkRdC : STD_LOGIC;
    SIGNAL SYNC  : STD_LOGIC;




	 --ssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss
signal 	clock :  std_logic ;
signal 	clock_n :  std_logic ;
signal 	clock1 :  std_logic ;
signal 	clock2 :  std_logic ;
signal	DCMs_Locked 		:  std_logic;
signal	reset_n 		:  std_logic:='1';


signal    comd :  std_logic_vector(3 downto 0);
signal    count :  std_logic_vector(23 downto 0):=(others=>'0');
signal    lig:  std_logic_vector(9 downto 0):="0000000000";
signal    light:  std_logic_vector(9 downto 0):="0000000000";






signal din:  std_logic_VECTOR(31 downto 0);
signal rd_clk:  std_logic;
signal rd_en:  std_logic;
signal wr_clk:  std_logic;
signal wr_en:  std_logic:='0';
signal dout:  std_logic_VECTOR(31 downto 0);
signal empty:  std_logic;
signal full:  std_logic;



signal				ram_r_wn : std_logic:='1';
signal			ram_dqin :std_logic_vector(9 downto 0);
signal			ram_dqout :std_logic_vector(9 downto 0):="0000000000";

-- for altera

--signal	reset_nn 		:  std_logic:='1';
signal	system_clk 		:  std_logic:='0';
signal	trg_word		:  std_logic_vector(31 downto 0):="00000000000000000000000000000011";
signal	read_clk 		:  std_logic;
signal	rdreq			:  std_logic;
--signal	rdreqt			:  std_logic;
signal	read_ready	:  std_logic;
--signal	data_int		:  std_logic_vector(17 downto 0);
signal	data_in		:  std_logic_vector(17 downto 0);
--signal	data_outt				:  std_logic_vector(31 downto 0);
signal	indata				:  std_logic_vector(31 downto 0):=(others=>'0');
signal	dataout				:  std_logic_vector(31 downto 0):=(others=>'0');
signal	got_cluster			:  STD_LOGIC_VECTOR(7 downto 0);
--signal	got_clustert			:  std_logic;

signal	start				:  STD_LOGIC_vector(14 downto 0);
signal	start_deli				:  STD_LOGIC_vector(4 downto 0);
signal idle_flag 			:  STD_LOGIC_vector(14 downto 0);
signal idle_deli 			:  STD_LOGIC_vector(4 downto 0);
signal busy_deli 			:  STD_LOGIC_vector(4 downto 0);
  
signal	trg_come			:  std_logic;
signal	trig_come			:  std_logic;
signal	trig_word		:  std_logic_vector(11 downto 0):="000000000000";
signal	trig_counter		:  std_logic_vector(18 downto 0):=(others=>'0');
signal	trig_wordp		:  std_logic_vector(11 downto 0):="000000000000";
signal	four_word		:  std_logic_vector(3 downto 0);
signal	rhic_check		:  std_logic_vector(9 downto 0):=(others=>'0');
signal	rhic_here		:  std_logic_vector(1 downto 0):=(others=>'0');
signal	rhic_lost			:  std_logic;
signal	latch_rhic_lost			:  std_logic;
signal	latch_leo39			:  std_logic;
signal	clock_control			:  std_logic;


signal	fifo_addr			:  STD_LOGIC_vector(6 downto 0);
--signal	fifo_addrf			:  STD_LOGIC_vector(5 downto 0);



signal	main_clk			:  STD_LOGIC;
signal	local_clk			:  STD_LOGIC;

signal	ava_wreq			:  STD_LOGIC;



signal	trigger_sum :  std_logic;
signal	ava_rdatat :  std_logic_vector(31 downto 0) ;
signal	ava_addrt :  std_logic_vector(5 downto 0) ;
signal	sys_status :  std_logic_vector(11 downto 0) ;


 
signal readready_deli 			: STD_LOGIC_vector(4 downto 0);
signal sum_readready 			: STD_LOGIC_vector(14 downto 0);
signal write_data 			: STD_LOGIC_vector(31 downto 0);
signal read_data 			: STD_LOGIC_vector(31 downto 0);
signal read_req : STD_LOGIC:='0';	 -- read request signal
signal write_req : STD_LOGIC:='0';	 -- write request signal
signal rdempty : STD_LOGIC:='0';				--no use
signal wrfull : STD_LOGIC:='0';






signal	in_wreq 		:  std_logic;
signal	out_rreq 		:  std_logic;
signal	in_full 		:  std_logic;
signal	out_empty 		:  std_logic;


signal	PixelAddressto :  std_logic_vector(17 downto 0); 
-- for clusterfifos

--for config
--signal           reset_n :  std_logic;
signal           from_adc :  std_logic_vector(11 downto 0);
signal           from_addr :  std_logic_vector(17 downto 0);
signal           from_config_in :  std_logic_vector(31 downto 0);
signal           to_sram_addr :  std_logic_vector(17 downto 0);
signal           to_sram_data :  std_logic_vector(143 downto 0);
signal           to_sram_writen : std_logic;
signal           to_config_in_rreq :  std_logic;
signal           from_config_in_empty :  std_logic;
signal           to_config_out :  std_logic_vector(31 downto 0);
signal           to_config_out_wreq :  std_logic;
signal           from_config_out_full :  std_logic;
signal			  to_adc:  std_logic_vector(1 downto 0);
signal			  command:  std_logic_vector(9 downto 0);

signal			  fifo_r :  std_logic_vector(31 downto 0);
signal           fifo_r_req :  std_logic;
signal           fifo_w :  std_logic_vector(31 downto 0);
signal           fifo_w_req :  std_logic;
signal           fifo_r_empty :  std_logic;
signal           fifo_w_full :  std_logic;
--signal			  avalon_clk : std_logic

-- for clusterfifos
signal           my_clk :  std_logic;
signal           rhic_clk :  std_logic;
signal           five_rhic_clk :  std_logic;

signal R_RST         :     STD_LOGIC;
signal R_RST_DCM :     STD_LOGIC;
signal R_dcm_adc_locked  :    STD_LOGIC;
 -- setup
signal R_idle    :    STD_LOGIC;
signal R_fake_data_mode  :     STD_LOGIC;
signal R_do_resync       :     STD_LOGIC;  -- when dog jtag
signal R_resync_status   :    STD_LOGIC;
signal R_do_daq  :     STD_LOGIC;
signal R_daq_busy:    STD_LOGIC;
signal R_do_cds_daq      :     STD_LOGIC;
signal R_cds_daq_busy    :    STD_LOGIC;
signal R_get_nios_io     :     STD_LOGIC;
signal R_got_nios_io     :    STD_LOGIC;
signal R_do_clr_mem      :     STD_LOGIC;
signal R_clr_mem_busy    :    STD_LOGIC;
signal R_do_full_frame   :     STD_LOGIC;
signal R_full_frame_busy :  STD_LOGIC;
signal R_daq_error       :    STD_LOGIC;
signal R_ack_daq_error   :     STD_LOGIC;
signal R_do_mem_test     :     STD_LOGIC;
signal R_mem_test_busy   :    STD_LOGIC;
signal R_mem_test_done   :    STD_LOGIC;
 -- random read write
signal R_nios_addr       :     STD_LOGIC_VECTOR (17 DOWNTO 0);
signal R_nios_wr :     STD_LOGIC;
signal R_nios_data_wr    :     STD_LOGIC_VECTOR (143 DOWNTO 0);
signal R_nios_data_rd    :    STD_LOGIC_VECTOR (143 DOWNTO 0);
signal R_adc_select      :     STD_LOGIC_VECTOR (2 DOWNTO 0);
signal R_sort_select      :     STD_LOGIC_VECTOR (3 DOWNTO 0);
signal R_full_frame_addr :    STD_LOGIC_VECTOR (17 DOWNTO 0);
signal R_full_frame_data :    STD_LOGIC_VECTOR (11 DOWNTO 0);
signal R_full_frame_wr   :    STD_LOGIC;
signal R_full_frame_done :    STD_LOGIC;
 -- mem test
signal  R_frame_data    :    STD_LOGIC_VECTOR (143 DOWNTO 0);
signal  R_frame_address    :    STD_LOGIC_VECTOR (17 DOWNTO 0);
signal R_mem_test_fail   :    STD_LOGIC;
 -- mimostar2
signal R_CkRdC   :    STD_LOGIC;
signal R_SYNC    :    STD_LOGIC;
signal R_MxFirst :     STD_LOGIC;
signal R_pixel_addr :  STD_LOGIC_VECTOR (17 DOWNTO 0);
signal R_cluster_we :  STD_LOGIC_VECTOR ( 7 DOWNTO 0);
signal R_ostatus :  STD_LOGIC_VECTOR ( 31 DOWNTO 0);
signal		  R_req :  std_logic;
signal		  R_align :  std_logic;
signal        R_threshold :  std_logic_vector (8 downto 0);
signal        R_threshold_o :  std_logic_vector (143 downto 0);
signal        R_adc_o :  std_logic_vector (71 downto 0);

signal		  als_flag1 :  std_logic;
signal		  als_flag2 :  std_logic;
signal		  als_flag3 :  std_logic;

--ssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss




    ---------------------------------------------------------------------------
    -- testing
    ---------------------------------------------------------------------------
    TYPE   state_type IS (IDLE, DO_MEM_TEST, WAIT_MEM_TEST_DONE, END_MEM_TEST, CLR_MEM, WAIT_CLR_MEM, RESYNC, WAIT_FOREVER, WAIT_DAQ_DONE, READ_FRAME, DONE);
    SIGNAL state : state_type;
    SIGNAL busy  : STD_LOGIC;

    SIGNAL do_resync  : STD_LOGIC;
    SIGNAL do_clr_mem : STD_LOGIC;
    SIGNAL do_daq     : STD_LOGIC;

    SIGNAL daq_busy      : STD_LOGIC;
    SIGNAL do_frame      : STD_LOGIC;
    SIGNAL frame_done    : STD_LOGIC;
    SIGNAL full_frame_wr : STD_LOGIC;

    SIGNAL full_frame_data : STD_LOGIC_VECTOR (11 DOWNTO 0);

    SIGNAL RST_CLK : STD_LOGIC;
    SIGNAL RST_DCM : STD_LOGIC;

    SIGNAL TCdaq         : STD_LOGIC_VECTOR (7 DOWNTO 0);
    SIGNAL adc_reset_int : STD_LOGIC;

    SIGNAL mem_test      : STD_LOGIC;
    SIGNAL mem_test_busy : STD_LOGIC;
    SIGNAL mem_test_done : STD_LOGIC;
    SIGNAL mem_test_fail : STD_LOGIC;
 --------------------------------------------------------------------
  SIGNAL       jumpers     :     STD_LOGIC_VECTOR (3 DOWNTO 0);
  SIGNAL      LED         :    STD_LOGIC_VECTOR (10 DOWNTO 1);
  SIGNAL      TC          :    STD_LOGIC_VECTOR(7 DOWNTO 0);
 --------------------------------------------------------------------
 -------------10/10/2006 added by Michal to perform chip calibrations from shot noise
signal led_counter	: INTEGER RANGE 0 TO 660*128*1000;
--signal trig_length	: INTEGER := 35000;
--signal trig_pixel		: INTEGER := 660*128+300;
signal trig_length	: INTEGER := 35000;
signal trig_pixel		: INTEGER := 660*12+300;
signal ready_flag		: STD_LOGIC:='0'; 
signal trigger_flag		: STD_LOGIC:='0'; 
signal trigger_delay		: STD_LOGIC:='0'; 
signal trigger_delay1		: STD_LOGIC:='0'; 
signal flag_counter	: STD_LOGIC_VECTOR (31 DOWNTO 0);
signal length_counter	: STD_LOGIC_VECTOR (31 DOWNTO 0):=CONV_STD_LOGIC_VECTOR(50000000,32);
signal buffer_counter	: STD_LOGIC_VECTOR (31 DOWNTO 0);
signal trigger_counter	: STD_LOGIC_VECTOR (11 DOWNTO 0):=(others=>'0');
SIGNAL ALS_trigger : STD_LOGIC:='0';
--------------
BEGIN  -- hft_daughtercard_top_arch

leo(16)<=LED(10);
leo(17)<=LED(9);
leo(18)<=LED(8);
leo(19)<=LED(7);
leo(21)<=LED(6);
leo(22)<=LED(5);
leo(24)<=LED(4);
--leo(25)<=LED(3);
--leo(27)<=LED(2);
--leo(28)<=LED(1);
leo(12)<='Z';






--indata<= leo(14) & leo(59 downto 29) ;


--leo(23)<=dataout(10) ;
--leo(20)<=dataout(9) ;
--leo(37 downto 29)<=dataout(8 downto 0) ;


leo(8)<=read_ready; --read ready
rdreq<=leo(13);-- read request
--leo(12)is the error signal
system_clk<=leo(11);
 reset_n<=leo(10);

--fifo_addr<=leo(7 downto 1);
 trg_come<=	leo(9);
   four_word<=TCD(3)&TCD(2)&TCD(1)&TCD(0);-- when command="01" else (others=>'0');
--  trigger_sum<= trig_come or trg_come ;--when command="01" else '0';
  trigger_sum<= ALS_trigger ;--or trg_come ;--when command="01" else '0';
	sys_status<="000000000" & R_align & TCdaq(0) & dcm_adc_locked ;
	ALS_trigger<=leo(28);
  clusterfifos_i_0 : clusterfifos
generic map(
	ADDRSIZE 	=>9 ,
	DATASIZE 	=>32,
	WAIT_WINDOW_LENGTH =>1376,
	CLUSTER_WINDOW_LENGTH	=>84480,
	FIFO_LENGTH	 =>0,
	CHIP_ID	=>0
	 )
port map (
	reset_n=>reset_n,
	pix_clk =>clock,
	trg_word=>trg_word,
   thresholdr_i=>R_threshold_o(17 downto 9),
	thresholdc_i=>R_threshold_o(8 downto 0),	
	adc_i=>sys_status,
	data_in	=>data_in,
	got_cluster=>got_cluster(0),
	start=>start(0),
	idle_flag=>idle_flag(0),
	   fifo_data1=>leo(14),
		fifo_data2=>leo(59 downto 40),
		fifo_data3=>leo(23),
		fifo_data4=>leo(20),
		fifo_data5=>leo(37 downto 29),
	read_ready=>read_ready,
	sum_ready=>leo(25),
	rdreq	=>rdreq,
	read_clk=>system_clk,
	chip_address=>fifo_addr

		);

 clusterfifos_i_1 : clusterfifos
generic map(
	ADDRSIZE 	=>9 ,
	DATASIZE 	=>32,
	WAIT_WINDOW_LENGTH =>1376,
	CLUSTER_WINDOW_LENGTH	=>84480,
	FIFO_LENGTH	 =>0,
	CHIP_ID	=>1
	 )
port map (
	reset_n=>reset_n,
	pix_clk =>clock,
	trg_word=>trg_word,
   thresholdr_i=>R_threshold_o(35 downto 27),
	thresholdc_i=>R_threshold_o(26 downto 18),	
	adc_i=>sys_status,   
	data_in	=>data_in,
	got_cluster=>got_cluster(1),
	start=>start(1),
	idle_flag=>idle_flag(1),
	   fifo_data1=>leo(14),
		fifo_data2=>leo(59 downto 40),
		fifo_data3=>leo(23),
		fifo_data4=>leo(20),
		fifo_data5=>leo(37 downto 29),
	read_ready=>read_ready,
	sum_ready=>leo(25),
	rdreq	=>rdreq,
	read_clk=>system_clk,
	chip_address=>fifo_addr

		);

 clusterfifos_i_2 : clusterfifos
generic map(
	ADDRSIZE 	=>9 ,
	DATASIZE 	=>32,
	WAIT_WINDOW_LENGTH =>1376,
	CLUSTER_WINDOW_LENGTH	=>84480,
	FIFO_LENGTH	 =>1,
	CHIP_ID	=>2
	 )
port map (
	reset_n=>reset_n,
	pix_clk =>clock,
	trg_word=>trg_word,
   thresholdr_i=>R_threshold_o(53 downto 45),
	thresholdc_i=>R_threshold_o(44 downto 36),	
	adc_i=>sys_status,   
	data_in	=>data_in,
	got_cluster=>got_cluster(2),
	start=>start(2),
	idle_flag=>idle_flag(2),
	   fifo_data1=>leo(14),
		fifo_data2=>leo(59 downto 40),
		fifo_data3=>leo(23),
		fifo_data4=>leo(20),
		fifo_data5=>leo(37 downto 29),
	read_ready=>read_ready,
	sum_ready=>leo(25),
	rdreq	=>rdreq,
	read_clk=>system_clk,
	chip_address=>fifo_addr

		);

 clusterfifos_i_3 : clusterfifos
generic map(
	ADDRSIZE 	=>9 ,
	DATASIZE 	=>32,
	WAIT_WINDOW_LENGTH =>1376,
	CLUSTER_WINDOW_LENGTH	=>84480,
	FIFO_LENGTH	 =>0,
	CHIP_ID	=>3
	 )
port map (
	reset_n=>reset_n,
	pix_clk =>clock,
	trg_word=>trg_word,
   thresholdr_i=>R_threshold_o(17 downto 9),
	thresholdc_i=>R_threshold_o(8 downto 0),	
	adc_i=>sys_status,   
	data_in	=>data_in,
	got_cluster=>got_cluster(0),
	start=>start(3),
	idle_flag=>idle_flag(3),
	   fifo_data1=>leo(14),
		fifo_data2=>leo(59 downto 40),
		fifo_data3=>leo(23),
		fifo_data4=>leo(20),
		fifo_data5=>leo(37 downto 29),
	read_ready=>read_ready,
	sum_ready=>leo(25),
	rdreq	=>rdreq,
	read_clk=>system_clk,
	chip_address=>fifo_addr

		);

 clusterfifos_i_4 : clusterfifos
generic map(
	ADDRSIZE 	=>9 ,
	DATASIZE 	=>32,
	WAIT_WINDOW_LENGTH =>1376,
	CLUSTER_WINDOW_LENGTH	=>84480,
	FIFO_LENGTH	 =>0,
	CHIP_ID	=>4
	 )
port map (
	reset_n=>reset_n,
	pix_clk =>clock,
	trg_word=>trg_word,
   thresholdr_i=>R_threshold_o(35 downto 27),
	thresholdc_i=>R_threshold_o(26 downto 18),	
	adc_i=>sys_status,   
	data_in	=>data_in,
	got_cluster=>got_cluster(1),
	start=>start(4),
	idle_flag=>idle_flag(4),
	   fifo_data1=>leo(14),
		fifo_data2=>leo(59 downto 40),
		fifo_data3=>leo(23),
		fifo_data4=>leo(20),
		fifo_data5=>leo(37 downto 29),
	read_ready=>read_ready,
	sum_ready=>leo(25),
	rdreq	=>rdreq,
	read_clk=>system_clk,
	chip_address=>fifo_addr

		);

 clusterfifos_i_5 : clusterfifos
generic map(
	ADDRSIZE 	=>9 ,
	DATASIZE 	=>32,
	WAIT_WINDOW_LENGTH =>1376,
	CLUSTER_WINDOW_LENGTH	=>84480,
	FIFO_LENGTH	 =>1,
	CHIP_ID	=>5
	 )
port map (
	reset_n=>reset_n,
	pix_clk =>clock,
	trg_word=>trg_word,
   thresholdr_i=>R_threshold_o(53 downto 45),
	thresholdc_i=>R_threshold_o(44 downto 36),	
	adc_i=>sys_status,    
	data_in	=>data_in,
	got_cluster=>got_cluster(2),
	start=>start(5),
	idle_flag=>idle_flag(5),
	   fifo_data1=>leo(14),
		fifo_data2=>leo(59 downto 40),
		fifo_data3=>leo(23),
		fifo_data4=>leo(20),
		fifo_data5=>leo(37 downto 29),
	read_ready=>read_ready,
	sum_ready=>leo(25),
	rdreq	=>rdreq,
	read_clk=>system_clk,
	chip_address=>fifo_addr

		);
		
		
		
		
		
		
		
		 clusterfifos_i_6 : clusterfifos
generic map(
	ADDRSIZE 	=>9 ,
	DATASIZE 	=>32,
	WAIT_WINDOW_LENGTH =>1376,
	CLUSTER_WINDOW_LENGTH	=>84480,
	FIFO_LENGTH	 =>0,
	CHIP_ID	=>6
	 )
port map (
	reset_n=>reset_n,
	pix_clk =>clock,
	trg_word=>trg_word,
   thresholdr_i=>R_threshold_o(17 downto 9),
	thresholdc_i=>R_threshold_o(8 downto 0),	
	adc_i=>sys_status,   
	data_in	=>data_in,
	got_cluster=>got_cluster(0),
	start=>start(6),
	idle_flag=>idle_flag(6),
	   fifo_data1=>leo(14),
		fifo_data2=>leo(59 downto 40),
		fifo_data3=>leo(23),
		fifo_data4=>leo(20),
		fifo_data5=>leo(37 downto 29),
	read_ready=>read_ready,
	sum_ready=>leo(25),
	rdreq	=>rdreq,
	read_clk=>system_clk,
	chip_address=>fifo_addr

		);

 clusterfifos_i_7 : clusterfifos
generic map(
	ADDRSIZE 	=>9 ,
	DATASIZE 	=>32,
	WAIT_WINDOW_LENGTH =>1376,
	CLUSTER_WINDOW_LENGTH	=>84480,
	FIFO_LENGTH	 =>0,
	CHIP_ID	=>7
	 )
port map (
	reset_n=>reset_n,
	pix_clk =>clock,
	trg_word=>trg_word,
   thresholdr_i=>R_threshold_o(35 downto 27),
	thresholdc_i=>R_threshold_o(26 downto 18),	
	adc_i=>sys_status,   
	data_in	=>data_in,
	got_cluster=>got_cluster(1),
	start=>start(7),
	idle_flag=>idle_flag(7),
	   fifo_data1=>leo(14),
		fifo_data2=>leo(59 downto 40),
		fifo_data3=>leo(23),
		fifo_data4=>leo(20),
		fifo_data5=>leo(37 downto 29),
	read_ready=>read_ready,
	sum_ready=>leo(25),
	rdreq	=>rdreq,
	read_clk=>system_clk,
	chip_address=>fifo_addr

		);

 clusterfifos_i_8 : clusterfifos
generic map(
	ADDRSIZE 	=>9 ,
	DATASIZE 	=>32,
	WAIT_WINDOW_LENGTH =>1376,
	CLUSTER_WINDOW_LENGTH	=>84480,
	FIFO_LENGTH	 =>1,
	CHIP_ID	=>8
	 )
port map (
	reset_n=>reset_n,
	pix_clk =>clock,
	trg_word=>trg_word,
   thresholdr_i=>R_threshold_o(53 downto 45),
	thresholdc_i=>R_threshold_o(44 downto 36),	
	adc_i=>sys_status,    
	data_in	=>data_in,
	got_cluster=>got_cluster(2),
	start=>start(8),
	idle_flag=>idle_flag(8),
	   fifo_data1=>leo(14),
		fifo_data2=>leo(59 downto 40),
		fifo_data3=>leo(23),
		fifo_data4=>leo(20),
		fifo_data5=>leo(37 downto 29),
	read_ready=>read_ready,
	sum_ready=>leo(25),
	rdreq	=>rdreq,
	read_clk=>system_clk,
	chip_address=>fifo_addr

		);
		
		
		
		
		
		
		
	 clusterfifos_i_9 : clusterfifos
generic map(
	ADDRSIZE 	=>9 ,
	DATASIZE 	=>32,
	WAIT_WINDOW_LENGTH =>1376,
	CLUSTER_WINDOW_LENGTH	=>84480,
	FIFO_LENGTH	 =>0,
	CHIP_ID	=>9
	 )
port map (
	reset_n=>reset_n,
	pix_clk =>clock,
	trg_word=>trg_word,
   thresholdr_i=>R_threshold_o(17 downto 9),
	thresholdc_i=>R_threshold_o(8 downto 0),	
	adc_i=>sys_status,   
	data_in	=>data_in,
	got_cluster=>got_cluster(0),
	start=>start(9),
	idle_flag=>idle_flag(9),
	   fifo_data1=>leo(14),
		fifo_data2=>leo(59 downto 40),
		fifo_data3=>leo(23),
		fifo_data4=>leo(20),
		fifo_data5=>leo(37 downto 29),
	read_ready=>read_ready,
	sum_ready=>leo(25),
	rdreq	=>rdreq,
	read_clk=>system_clk,
	chip_address=>fifo_addr

		);

 clusterfifos_i_10 : clusterfifos
generic map(
	ADDRSIZE 	=>9 ,
	DATASIZE 	=>32,
	WAIT_WINDOW_LENGTH =>1376,
	CLUSTER_WINDOW_LENGTH	=>84480,
	FIFO_LENGTH	 =>0,
	CHIP_ID	=>10
	 )
port map (
	reset_n=>reset_n,
	pix_clk =>clock,
	trg_word=>trg_word,
   thresholdr_i=>R_threshold_o(35 downto 27),
	thresholdc_i=>R_threshold_o(26 downto 18),	
	adc_i=>sys_status,   
	data_in	=>data_in,
	got_cluster=>got_cluster(1),
	start=>start(10),
	idle_flag=>idle_flag(10),
	   fifo_data1=>leo(14),
		fifo_data2=>leo(59 downto 40),
		fifo_data3=>leo(23),
		fifo_data4=>leo(20),
		fifo_data5=>leo(37 downto 29),
	read_ready=>read_ready,
	sum_ready=>leo(25),
	rdreq	=>rdreq,
	read_clk=>system_clk,
	chip_address=>fifo_addr

		);

 clusterfifos_i_11 : clusterfifos
generic map(
	ADDRSIZE 	=>9 ,
	DATASIZE 	=>32,
	WAIT_WINDOW_LENGTH =>1376,
	CLUSTER_WINDOW_LENGTH	=>84480,
	FIFO_LENGTH	 =>1,
	CHIP_ID	=>11
	 )
port map (
	reset_n=>reset_n,
	pix_clk =>clock,
	trg_word=>trg_word,
   thresholdr_i=>R_threshold_o(53 downto 45),
	thresholdc_i=>R_threshold_o(44 downto 36),	
	adc_i=>sys_status,    
	data_in	=>data_in,
	got_cluster=>got_cluster(2),
	start=>start(11),
	idle_flag=>idle_flag(11),
	   fifo_data1=>leo(14),
		fifo_data2=>leo(59 downto 40),
		fifo_data3=>leo(23),
		fifo_data4=>leo(20),
		fifo_data5=>leo(37 downto 29),
	read_ready=>read_ready,
	sum_ready=>leo(25),
	rdreq	=>rdreq,
	read_clk=>system_clk,
	chip_address=>fifo_addr

		);
				
		



writedeligater_i_0: writedeligater 
    Port map ( 
	reset_n=>reset_n,
	--clk =>leo(30),
   --clk =>system_clk,
   clk =>clock,
    trg_come =>trigger_sum,
	 trigger_counter=>trigger_counter,
	 fifo_select =>start_deli(4 downto 0),
    idle_state =>idle_deli(4 downto 0)
--    busy_state =>busy_deli(4 downto 0)

	 );



switch_i: switch 
    Port map ( clk =>CLK50,
           reset =>reset_n,
 --			   ava_clk=>system_clk,
       RST             =>R_RST,
        RST_DCM         =>R_RST_DCM,
        dcm_adc_locked  =>R_dcm_adc_locked,
        idle				=>R_idle,
        fake_data_mode =>R_fake_data_mode	,
        do_resync       =>R_do_resync	,
        resync_status  =>R_resync_status	,
        do_daq         =>R_do_daq		 ,
        daq_busy       =>R_daq_busy		  ,
        do_cds_daq      =>R_do_cds_daq	  ,
        cds_daq_busy    =>R_cds_daq_busy	,
        get_nios_io     =>R_get_nios_io	,
        got_nios_io     =>R_got_nios_io	 ,
        do_clr_mem      =>R_do_clr_mem		  ,
        clr_mem_busy    =>R_clr_mem_busy	  ,
        do_full_frame   =>R_do_full_frame	  ,
        full_frame_busy =>R_full_frame_busy	,
        daq_error       =>R_daq_error			,
        ack_daq_error   =>R_ack_daq_error		,
        do_mem_test     =>R_do_mem_test		,
        mem_test_busy   =>R_mem_test_busy		,
        mem_test_done   =>R_mem_test_done		,
        -- random read write
        nios_addr       =>R_nios_addr			,
        nios_wr        =>R_nios_wr				,
        nios_data_wr    =>R_nios_data_wr		,
        nios_data_rd    =>R_nios_data_rd		 ,
        -- full frame read
        adc_select      =>R_adc_select			,
        sort_select     =>R_sort_select		,
        full_frame_addr =>R_full_frame_addr	 ,
        full_frame_data =>R_full_frame_data	 ,
        full_frame_wr   =>R_full_frame_wr		  ,
        full_frame_done =>R_full_frame_done	  ,
				frame_data		 =>R_frame_data,
				frame_address	 =>R_frame_address,        -- mem test
        mem_test_fail  =>R_mem_test_fail		  ,
        -- mimostar2
        CkRdC          =>R_CkRdC						,
        SYNC            =>R_SYNC						,
        MxFirst         =>R_MxFirst					,
        pixel_addr =>R_pixel_addr					,
        cluster_we =>R_cluster_we					,
        ostatus    =>R_ostatus        				,			  
		  addr =>leo(7 downto 1),
 --     addr =>"0000000",
      fifo_data1=>leo(14),
		fifo_data2=>leo(59 downto 40),
		fifo_data3=>leo(23),
		fifo_data4=>leo(20),
		fifo_data5=>leo(37 downto 29),
			  req =>	rdreq,
           ready =>read_ready,
        threshold_o=>R_threshold,
			  	Token	=>trig_word,	
			trg_come	=>trg_come,
--			read_ready=>readready_deli,
--			read_ready=>"00000",
--			fifo_addr_out=>open,
			ready_flag	=>ready_flag,  
         trigger_flag	=> trigger_flag 
			  );
---
 R_ostatus(2 downto 0)<=R_adc_select;
---
--ClusterGeneratorFake_i: ClusterGeneratorFake 
--port map (
--	reset_n=>reset_n,
--	clk =>clock,
--	PixelAddress=>data_in,
--	PixelAddressto=>PixelAddressto,
--	GotCluster=>got_cluster
--) ;

    PROCESS (RST_CLK)
        VARIABLE RSTcnt : INTEGER RANGE 0 TO 100000 := 0;
    BEGIN  -- PROCESS
        IF RST_CLK'EVENT AND RST_CLK = '1' THEN  -- rising clock edge
		  latch_rhic_lost<=rhic_lost;
		  latch_leo39<=leo(39);
            IF RSTcnt < 1000 THEN
                RST_DCM <= '1';
            ELSE
                RST_DCM <= '0';
            END IF;
            IF RSTcnt < 9999 THEN
                RST <= '1';
                --RSTcnt := RSTcnt + 1;
            ELSE
                RST <= '0';  --jumpers(3); --'0' ;
            END IF;
            IF RSTcnt < 99000 THEN
                RSTcnt := RSTcnt + 1;
            ELSE
                IF dcm_sys_locked = '0' or R_RST_DCM='1' THEN
                    RSTcnt := 0;
                END IF;
            END IF;
						if (rhic_lost='1' and latch_rhic_lost='0') or (rhic_lost='0' and latch_rhic_lost='1') then
							RSTcnt := 0;
						end if;
						if (leo(39)='1' and latch_leo39='0') or (leo(39)='0' and latch_leo39='1') then
							RSTcnt := 0;
						end if;
				END IF;
    END PROCESS;
    RSTn <= NOT RST;


    -- LVDS clock input
--    LVDS_CLK_in : IBUFGDS_LVDS_33
--        PORT MAP (
--            I  => CLK_p,
--            IB => CLK_n,
--            O  => CLKin
--            );

    -- 50MHz IN
    -- 50MHz OUT
    -- 100MHz OUT
    -- 100MHz 270deg out

--   U1_CLKIN_IBUFGDS_INST : IBUFGDS
--      port map (I=>CLK_p,
--                IB=>CLK_n,
--                O=>local_clk);

    dcm50to100_inst : dcm50to100
        PORT MAP (
            U1_CLKIN_N_IN        => CLK_n,
            U1_CLKIN_P_IN        => CLK_p,
            --U1_CLKIN_IN        => CLKin,
            RHIC_CLK_IN        => five_rhic_clk,
            CLK_SELECTION_IN        => clock_control,
            U1_RST_IN            => RST_DCM,
            --U1_CLKIN_IBUFG_OUT => RST_CLK,
            U1_CLKIN_IBUFGDS_OUT => RST_CLK,
--              U1_CLKIN_IBUFGDS_OUT => open,
            --U1_CLK0_OUT          => CLK50,
            U1_CLK2X_OUT         => OPEN,
            U1_STATUS_OUT        => OPEN,
            U2_CLKDV_OUT         => CLK50,
            U2_CLK0_OUT          => CLK100,
            U2_CLK270_OUT        => CLK100_RAM_RD,
            U2_LOCKED_OUT        => dcm_sys_locked,
            U2_STATUS_OUT        => OPEN
            );

clock_control<=leo(39) and (not rhic_lost);
--		  
    daq_inst : daq
        PORT MAP (
            CLK50           => CLK50,
            CLK100          => CLK100,
            CLK100_RAM_RD   => CLK100_RAM_RD,
            RST             => R_RST,
            RST_DCM         => R_RST_DCM,
            dcm_adc_locked  => dcm_adc_locked,
            -- setup
            idle            => OPEN,
            fake_data_mode  => '0',
            do_resync       => R_do_resync,
            resync_status   => OPEN,
            do_daq          => R_do_daq,
            daq_busy        => R_daq_busy,
            do_cds_daq      => '0',
            cds_daq_busy    => OPEN,
            get_nios_io     => R_get_nios_io,
            got_nios_io     => R_got_nios_io,
            do_clr_mem      => do_clr_mem,
            clr_mem_busy    => busy,
            daq_error       => R_daq_error,
            ack_daq_error   => R_ack_daq_error,
            do_mem_test     => mem_test,
            mem_test_busy   => mem_test_busy,
            mem_test_done   => mem_test_done,
            -- random read write
            nios_addr       => R_nios_addr,
            nios_wr         => R_nios_wr,
            nios_data_wr    => R_nios_data_wr,
            nios_data_rd    => R_nios_data_rd,
            -- full frame read
            do_full_frame   => do_frame,
            full_frame_busy => OPEN,
            adc_select      => R_adc_select,
        sort_select      => R_sort_select,
            full_frame_addr => OPEN,
            full_frame_data => full_frame_data,
            full_frame_wr   => full_frame_wr,
            full_frame_done => frame_done,
				frame_data		 =>R_frame_data,
				frame_address	 =>R_frame_address,

            -- mem test
            mem_test_fail   => mem_test_fail,
            -- mimostar2
            CkRdC           => CkRdC,
            SYNC            => SYNC,
            MxFirst         => MxFirst,
            -- ADS5271
            ADCLK           => ADC_CLK,
            Ads5273Rst      => adc_reset_int,  --ADC_RESET_N,
--            LclkP           => ADC_LCLK_P,
--            LclkN           => ADC_LCLK_N,
            LclkP           => ADC_LCLK_P,
            LclkN           => ADC_LCLK_N,
            AdClkP          => ADC_ADCLK_P,
            AdClkN          => ADC_ADCLK_N,
            DatInCh0P       => ADC_D1_P,
            DatInCh0N       => ADC_D1_N,
            DatInCh1P       => ADC_D2_P,
            DatInCh1N       => ADC_D2_N,
            DatInCh2P       => ADC_D3_P,
            DatInCh2N       => ADC_D3_N,
            DatInCh3P       => ADC_D4_P,
            DatInCh3N       => ADC_D4_N,
            DatInCh4P       => ADC_D5_P,
            DatInCh4N       => ADC_D5_N,
            DatInCh5P       => ADC_D6_P,
            DatInCh5N       => ADC_D6_N,
            DatInCh6P       => ADC_D7_P,
            DatInCh6N       => ADC_D7_N,
            DatInCh7P       => ADC_D8_P,
            DatInCh7N       => ADC_D8_N,
            -- sram
            RAM_A           => RAM_A,
            RAM_DQ          => RAM_DQ,
            RAM_ADV         => RAM_ADV,
            RAM_CK_A        => RAM_CK_A,
            RAM_CK_B        => RAM_CK_B,
            RAM_E2          => RAM_E2,
            RAM_FT          => RAM_FT,
            RAM_G           => RAM_G,
            RAM_LBO         => RAM_LBO,
            RAM_RnW         => RAM_RnW,
            RAM_PE          => RAM_PE,
            RAM_ZQ          => RAM_ZQ,
            -- to cluster finder
        pixel_addr =>R_pixel_addr					,
        cluster_we =>R_cluster_we					,
		  		  align=>R_align,
        threshold_i=>R_threshold,
		  		threshold_o=>R_threshold_o,
        		adc_o=>R_adc_o,
            -- test connector
            TC              => TCdaq           --OPEN
            );




   ADC_RESET_N <= adc_reset_int;              --and jumpers(3);
    RSTB        <= '1';

    mimostar_clk_buff : OBUFDS_LVDS_33
        PORT MAP (
            O  => CkRdC_p,
            OB => CkRdC_n,
            I  => CkRdC
            );

    mimostar_sync_buff : OBUFDS_LVDS_33
        PORT MAP (
            O  => SYNC_p,
            OB => SYNC_n,
            I  => SYNC
            );



    inst_set_adc_spi : set_adc_spi
        PORT MAP (
            CLK50 => CLK50,
            RST   => RSTn,
            -- ADS527x
            conf  => jumpers (2 DOWNTO 0),
            nCS   => nCS,
            SCLK  => SCLK,
            SDATA => SDATA,
            -- test connector
            TC    => OPEN
            );
    IBUFG_instR : BUFG
   port map (
      O => five_rhic_clk, -- Clock buffer output
      I => TCD(5)  -- Clock buffer input (connect directly to top-level port)
   );
    IBUFG_instR5 : BUFG
   port map (
      O => rhic_clk, -- Clock buffer output
      I => TCD(4)  -- Clock buffer input (connect directly to top-level port)
   );

trigger_decoder_i: trigger_decoder 
    port map (
         RHIC		=>rhic_clk,  		-- ~ 10 MHz in
			RHICx5	=>five_rhic_clk,	  		-- 5 time RHIC clock in
         rst	   =>'0', 			-- System reset
			Data		=>four_word,
			Token		=>trig_word,
			comd		=>comd,	
			trg_come	 =>	trig_come	
    );



	process (five_rhic_clk)
	begin
		if five_rhic_clk'event and five_rhic_clk = '0'  then
			if rhic_clk='1' then 
				if rhic_here(0)=rhic_here(1) then
				rhic_here(0)<=not rhic_here(0);
				end if; 
			end if;
		end if;
	end process;

    PROCESS (RST_CLK)
    BEGIN  -- PROCESS
        IF RST_CLK'EVENT AND RST_CLK = '1' THEN  -- rising clock edge
		  if  CONV_INTEGER(rhic_check)<100 then
		  rhic_lost<='0';
				if rhic_here(0)=rhic_here(1)  then

					rhic_check<=rhic_check+1;
				elsif rhic_clk='0' then
					rhic_check<=(others=>'0');
					rhic_here(1)<=not rhic_here(1);
				end if;
			else
			rhic_lost<='1';
				if rhic_here(0)=(not rhic_here(1))  then
				rhic_check<=(others=>'0');
				end if;
			end if;
        END IF;
    END PROCESS;
-----------------------------------------------------------------------
-----------------SXM part
--------------------------------------------------------------------------


clock<=CLK50;
leo(38)<=CLK50;

--    LED(3) <= reset_n;          -- AND dcm_adc_locked;
    LED(9) <=R_get_nios_io;          -- AND dcm_adc_locked;
	 LED(10)<= R_do_daq ;
	 LED(4)<=ready_flag;
--	 LED(8 downto 6) <= TCdaq(3 downto 1);          -- AND dcm_adc_locked;
	 LED(5) <= TCdaq(0);
	 LED(6) <= idle_flag(2);
	 LED(7) <= fifo_addr(0);
	 LED(8) <= dcm_adc_locked;

--	 led(3)<=trigger_counter(11);
--LED(8 downto 4)<=trigger_counter(10 downto 6);
--LED(8 downto 4)<=readready_deli;
--LED(8 downto 4)<=fifo_addr(4 downto 0);
--LED(8 downto 6)<=R_threshold_o(2 downto 0);
--	 LED(8 downto 6) <= R_sort_select(2 downto 0);          -- AND dcm_adc_locked;
--	 LED(8 downto 3) <= ram_dqout(5 downto 0);          -- AND dcm_adc_locked;
--	  LED(5)<=TCdaq(0);
    --LED(2) <=rhic_lost;          -- AND dcm_adc_locked;
--	 LED(8 downto 3) <= ram_dqout(5 downto 0);          -- AND dcm_adc_locked;

--	 LED(7) <= R_RST_DCM;          -- AND dcm_adc_locked;
--	 LED(8) <= R_do_resync;
--
--LED(7 downto 4)<=dataout(3 downto 0);


    -- misc
    TC <= "ZZZZZZZZ";
--    nCS   <= 'Z';
--    SDATA <= 'Z';
--    SCLK  <= 'Z';


-- process (MxFirst, CLK50)
--	begin
--	if CLK50'event and CLK50 = '1'  then
--
--		  data_in<=R_pixel_addr;
--		  got_cluster<=R_cluster_we(0);
--
--		if MxFirst = '1' and (led_counter > (128*660*900+201)) then 
--			led_counter <= 0;
--		else
--			led_counter <= led_counter + 1;
--			if (led_counter > trig_pixel) and (led_counter < (trig_pixel+trig_length)) then
--				LED(2 downto 1)<="01";
--			else
--				LED(2 downto 1)<="00";
--			end if;
--		end if;
--	end if;
--end process;


 process (MxFirst, CLK50)
	begin
	if CLK50'event and CLK50 = '1'  then

fifo_addr<=leo(7 downto 1);

idle_deli(0)<= idle_flag(2)   ;
idle_deli(1)<= idle_flag(5) ;
idle_deli(2)<= idle_flag(8) ;
idle_deli(3)<= idle_flag(11)  ;
idle_deli(4)<= idle_flag(14)  ;

--idle_deli(0)<=idle_flag(0) and idle_flag(1) and idle_flag(2)   ;
--idle_deli(1)<=idle_flag(3) and idle_flag(4) and idle_flag(5) ;
--idle_deli(2)<=idle_flag(6) and idle_flag(7) and idle_flag(8) ;
--idle_deli(3)<=idle_flag(9) and idle_flag(10) and idle_flag(11)  ;
--idle_deli(4)<=idle_flag(12) and idle_flag(13) and idle_flag(14)  ;

--busy_deli(0)<=(not idle_flag(0)) and (not idle_flag(1)) and (not idle_flag(2))   ;
--busy_deli(1)<=(not idle_flag(3)) and (not idle_flag(4)) and (not idle_flag(5)) ;
--busy_deli(2)<=(not idle_flag(6)) and (not idle_flag(7)) and (not idle_flag(8)) ;
--busy_deli(3)<=(not idle_flag(9)) and (not idle_flag(10)) and (not idle_flag(11))  ;
--busy_deli(4)<=(not idle_flag(12)) and (not idle_flag(13)) and (not idle_flag(14))  ;

--readready_deli(0)<= sum_readready(2) ;
--readready_deli(1)<= sum_readready(5) ;
--readready_deli(2)<= sum_readready(8) ;
--readready_deli(3)<= sum_readready(11) ;
--readready_deli(4)<= sum_readready(14) ;

start(0)<=start_deli(0);
start(1)<=start_deli(0);
start(2)<=start_deli(0);
start(3)<=start_deli(1);
start(4)<=start_deli(1);
start(5)<=start_deli(1);
start(6)<=start_deli(2);
start(7)<=start_deli(2);
start(8)<=start_deli(2);
start(9)<=start_deli(3);
start(10)<=start_deli(3);
start(11)<=start_deli(3);
start(12)<=start_deli(4);
start(13)<=start_deli(4);
start(14)<=start_deli(4);

--		  start<=start_buf;
		  data_in<=R_pixel_addr;
		  trg_word<=x"000" &  x"00" & trigger_counter;
--		  got_cluster(1)<='0';
--		  got_cluster(2)<='1';
--				if flag_counter>50000000  then	--50   50+84480
--				 trigger_sum<='1';
--				end if;


--  				  trigger_delay<=leo(28);
 				  trigger_delay<=trigger_sum;
 				  trigger_delay1<=trigger_delay;

--			  if leo(28)='1' and trigger_delay='0' then
			  if trigger_delay='1' and trigger_delay1='0'  then
--			  ALS_trigger<='1';
--			  trigger_counter<=trigger_counter+1;

				  flag_counter<=(others=>'0');
--				  buffer_counter<=flag_counter;
				  buffer_counter<=length_counter;
--				  length_counter<=0;
--				  trigger_sum<='0';
				elsif  flag_counter<884479999 then
--			  ALS_trigger<='0';

				  flag_counter<=flag_counter+1;
				  length_counter<=flag_counter;
				  trg_word<=buffer_counter;
				end if;

				if CONV_INTEGER(flag_counter)>CONV_INTEGER(buffer_counter)-84480*4	  then
				als_flag1<='1';
				else
				als_flag1<='0';
				end if;
				if CONV_INTEGER(flag_counter)<CONV_INTEGER(buffer_counter)+84480*3	  then
				als_flag2<='1';
				else
				als_flag2<='0';
				end if;
				if  flag_counter < 84480*2	  then
				als_flag3<='1';
				else
				als_flag3<='0';
				end if;

				if (als_flag1='1' and als_flag2='1') or (als_flag3='1') then	--50000000-2*84480    3*84480
--				if (flag_counter>50000000-168960 and flag_counter<50000000+253440) or (flag_counter < 253440) then	--50000000-2*84480    3*84480
				 ready_flag<='1';
				else
				 if R_sort_select="0100" then
				 ready_flag<='0';
				 else
				 ready_flag<='1';
				 end if;
				end if;

				if CONV_INTEGER(flag_counter)>50 and CONV_INTEGER(flag_counter)<=84530  then	--50   50+84480
				 trigger_flag<='1';
				else
				 trigger_flag<='0';
				end if;

			got_cluster<= R_cluster_we;
--			got_cluster<= "00000011";
		if MxFirst = '1' and (led_counter > (128*660*90+201)) then 
			led_counter <= 0;
		else
			led_counter <= led_counter + 1;
			if (led_counter > trig_pixel) and (led_counter < (trig_pixel+trig_length)) then
				--LED(4)<='1';
			else
				--LED(4)<='0';
			end if;
		end if;
	end if;
end process;

-- process ( CLK50)
--	begin
--	if CLK50'event and CLK50 = '1'  then


--		if CONV_INTEGER(trig_counter)<84470 then
--		 LED(1)<='1';
--		 LED(2)<='1';
--		else
--		 LED(1)<='0';
--		 LED(2)<='0';
--		end if;


--      if CONV_INTEGER(trig_counter)<100000 then
--			trig_counter<=trig_counter+1;
--		end if;

--		if trg_come = '1'  then 
--			trig_counter <= (others=>'0');
--		end if;

--	end if;
--end process;

process(trigger_sum)
begin
if  trigger_sum='1'and trigger_sum'event then

--if trig_wordp/=trig_word then
-- ram_dqout<=ram_dqout+1;
-- end if;

--  if command="01" then
--		trg_word<=x"000" &  x"00" & trig_word;
----------------------		
--		trig_wordp<=trig_word+1;
--	end if;
end if;
end process;

END hft_daughtercard_top_arch;

