LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_unsigned.ALL;
USE IEEE.std_logic_arith.ALL;

LIBRARY UNISIM;
USE UNISIM.ALL;


ENTITY hft_daughtercard_top IS
    PORT (
        CLK_p       : IN    STD_LOGIC;
        CLK_n       : IN    STD_LOGIC;
        -- Mimostar 2
        CkRdC_p     : OUT   STD_LOGIC;
        CkRdC_n     : OUT   STD_LOGIC;
        SYNC_p      : OUT   STD_LOGIC;
        SYNC_n      : OUT   STD_LOGIC;
        MxFirst     : IN    STD_LOGIC;
        RSTB        : OUT   STD_LOGIC;
        -- ADS interface
        ADC_CLK     : OUT   STD_LOGIC;
        ADC_RESET_N : OUT   STD_LOGIC;
        ADC_LCLK_P  : IN    STD_LOGIC;  -- Six times ADCLK
        ADC_LCLK_N  : IN    STD_LOGIC;
        ADC_ADCLK_P : IN    STD_LOGIC;  -- Skewed and Data aligned version of the ADC sample clock. Used as strobe.
        ADC_ADCLK_N : IN    STD_LOGIC;
        ADC_D1_P    : IN    STD_LOGIC;
        ADC_D1_N    : IN    STD_LOGIC;
        ADC_D2_P    : IN    STD_LOGIC;
        ADC_D2_N    : IN    STD_LOGIC;
        ADC_D3_P    : IN    STD_LOGIC;
        ADC_D3_N    : IN    STD_LOGIC;
        ADC_D4_P    : IN    STD_LOGIC;
        ADC_D4_N    : IN    STD_LOGIC;
        ADC_D5_P    : IN    STD_LOGIC;
        ADC_D5_N    : IN    STD_LOGIC;
        ADC_D6_P    : IN    STD_LOGIC;
        ADC_D6_N    : IN    STD_LOGIC;
        ADC_D7_P    : IN    STD_LOGIC;
        ADC_D7_N    : IN    STD_LOGIC;
        ADC_D8_P    : IN    STD_LOGIC;
        ADC_D8_N    : IN    STD_LOGIC;
        -- ADS527x SPI
        nCS         : OUT   STD_LOGIC;
        SCLK        : OUT   STD_LOGIC;
        SDATA       : OUT   STD_LOGIC;
        -- SRAM
        RAM_A       : OUT   STD_LOGIC_VECTOR (17 DOWNTO 0);
        RAM_DQ      : INOUT STD_LOGIC_VECTOR (143 DOWNTO 0);
        RAM_ADV     : OUT   STD_LOGIC;
        RAM_CK_A    : OUT   STD_LOGIC;
        RAM_CK_B    : OUT   STD_LOGIC;
        RAM_E2      : OUT   STD_LOGIC;
        RAM_FT      : OUT   STD_LOGIC;
        RAM_G       : OUT   STD_LOGIC;
        RAM_LBO     : OUT   STD_LOGIC;
        RAM_RnW     : OUT   STD_LOGIC;
        RAM_PE      : OUT   STD_LOGIC;
        RAM_ZQ      : OUT   STD_LOGIC;
        -- LEDs
        jumpers     : IN    STD_LOGIC_VECTOR (3 DOWNTO 0);
        LED         : OUT   STD_LOGIC_VECTOR (10 DOWNTO 1);
        TC          : OUT   STD_LOGIC_VECTOR(7 DOWNTO 0)
        );
END hft_daughtercard_top;




ARCHITECTURE hft_daughtercard_top_arch OF hft_daughtercard_top IS

    COMPONENT ROC
        -- synthesis translate_off 
        GENERIC (WIDTH :     TIME       := 100 ns);
        -- synthesis translate_on 
        PORT (O        : OUT STD_ULOGIC := '1');
    END COMPONENT;

    COMPONENT IBUFGDS_LVDS_33
        PORT (
            I  : IN  STD_LOGIC;
            IB : IN  STD_LOGIC;
            O  : OUT STD_LOGIC
            );
    END COMPONENT;

    COMPONENT OBUFDS_LVDS_33
        PORT(
            O  : OUT STD_ULOGIC;
            OB : OUT STD_ULOGIC;
            I  : IN  STD_ULOGIC
            );
    END COMPONENT;

    COMPONENT dcm50to100
        PORT (
            U1_CLKIN_N_IN        : IN  STD_LOGIC;
            U1_CLKIN_P_IN        : IN  STD_LOGIC;
            --U1_CLKIN_IN        : IN  STD_LOGIC;
            U1_RST_IN            : IN  STD_LOGIC;
            --U1_CLKIN_IBUFG_OUT : OUT STD_LOGIC;
            U1_CLKIN_IBUFGDS_OUT : OUT STD_LOGIC;
            --U1_CLK0_OUT          : OUT STD_LOGIC;
            U1_CLK2X_OUT         : OUT STD_LOGIC;
            U1_STATUS_OUT        : OUT STD_LOGIC_VECTOR (2 DOWNTO 0);
            U2_CLKDV_OUT         : OUT STD_LOGIC;
            U2_CLK0_OUT          : OUT STD_LOGIC;
            U2_CLK270_OUT        : OUT STD_LOGIC;
            U2_LOCKED_OUT        : OUT STD_LOGIC;
            U2_STATUS_OUT        : OUT STD_LOGIC_VECTOR (2 DOWNTO 0)
            );
    END COMPONENT;

    COMPONENT daq
        PORT (
            CLK50           : IN    STD_LOGIC;
            CLK100          : IN    STD_LOGIC;
            CLK100_RAM_RD   : IN    STD_LOGIC;
            RST             : IN    STD_LOGIC;
            RST_DCM         : IN    STD_LOGIC;
            dcm_adc_locked  : OUT   STD_LOGIC;
            -- setup
            idle            : OUT   STD_LOGIC;
            fake_data_mode  : IN    STD_LOGIC;
            do_resync       : IN    STD_LOGIC;  -- when doing jtag
            resync_status   : OUT   STD_LOGIC;
            do_daq          : IN    STD_LOGIC;
            daq_busy        : OUT   STD_LOGIC;
            do_cds_daq      : IN    STD_LOGIC;
            cds_daq_busy    : OUT   STD_LOGIC;
            get_nios_io     : IN    STD_LOGIC;
            got_nios_io     : OUT   STD_LOGIC;
            do_clr_mem      : IN    STD_LOGIC;
            clr_mem_busy    : OUT   STD_LOGIC;
            daq_error       : OUT   STD_LOGIC;
            ack_daq_error   : IN    STD_LOGIC;
            do_mem_test     : IN    STD_LOGIC;
            mem_test_busy   : OUT   STD_LOGIC;
            mem_test_done   : OUT   STD_LOGIC;
            -- random read write
            nios_addr       : IN    STD_LOGIC_VECTOR (17 DOWNTO 0);
            nios_wr         : IN    STD_LOGIC;
            nios_data_wr    : IN    STD_LOGIC_VECTOR (143 DOWNTO 0);
            nios_data_rd    : OUT   STD_LOGIC_VECTOR (143 DOWNTO 0);
            -- full frame read
            do_full_frame   : IN    STD_LOGIC;
            full_frame_busy : OUT   STD_LOGIC;
            adc_select      : IN    STD_LOGIC_VECTOR (2 DOWNTO 0);
        sort_select      : IN    STD_LOGIC_VECTOR (1 DOWNTO 0);
            full_frame_addr : OUT   STD_LOGIC_VECTOR (17 DOWNTO 0);
            full_frame_data : OUT   STD_LOGIC_VECTOR (11 DOWNTO 0);
            full_frame_wr   : OUT   STD_LOGIC;
            full_frame_done : OUT   STD_LOGIC;
            -- mem test
            mem_test_fail   : OUT   STD_LOGIC;
            -- mimostar2
            CkRdC           : OUT   STD_LOGIC;
            SYNC            : OUT   STD_LOGIC;
            MxFirst         : IN    STD_LOGIC;
            -- ADS5271
            ADCLK           : OUT   STD_LOGIC;
            Ads5273Rst      : OUT   STD_LOGIC;
            LclkP           : IN    STD_LOGIC;  -- Six times ADCLK
            LclkN           : IN    STD_LOGIC;
            AdClkP          : IN    STD_LOGIC;  -- Skewed and Data aligned version of the ADC sample clock. Used as strobe.
            AdClkN          : IN    STD_LOGIC;
            DatInCh0P       : IN    STD_LOGIC;
            DatInCh0N       : IN    STD_LOGIC;
            DatInCh1P       : IN    STD_LOGIC;
            DatInCh1N       : IN    STD_LOGIC;
            DatInCh2P       : IN    STD_LOGIC;
            DatInCh2N       : IN    STD_LOGIC;
            DatInCh3P       : IN    STD_LOGIC;
            DatInCh3N       : IN    STD_LOGIC;
            DatInCh4P       : IN    STD_LOGIC;
            DatInCh4N       : IN    STD_LOGIC;
            DatInCh5P       : IN    STD_LOGIC;
            DatInCh5N       : IN    STD_LOGIC;
            DatInCh6P       : IN    STD_LOGIC;
            DatInCh6N       : IN    STD_LOGIC;
            DatInCh7P       : IN    STD_LOGIC;
            DatInCh7N       : IN    STD_LOGIC;
            -- sram
            RAM_A           : OUT   STD_LOGIC_VECTOR (17 DOWNTO 0);
            RAM_DQ          : INOUT STD_LOGIC_VECTOR (143 DOWNTO 0);
            RAM_ADV         : OUT   STD_LOGIC;
            RAM_CK_A        : OUT   STD_LOGIC;
            RAM_CK_B        : OUT   STD_LOGIC;
            RAM_E2          : OUT   STD_LOGIC;
            RAM_FT          : OUT   STD_LOGIC;
            RAM_G           : OUT   STD_LOGIC;
            RAM_LBO         : OUT   STD_LOGIC;
            RAM_RnW         : OUT   STD_LOGIC;
            RAM_PE          : OUT   STD_LOGIC;
            RAM_ZQ          : OUT   STD_LOGIC;
            -- random IO
            -- to cluster finder
            pixel_addr      : OUT   STD_LOGIC_VECTOR (17 DOWNTO 0);
            cluster_we      : OUT   STD_LOGIC_VECTOR (7 DOWNTO 0);
            -- test connector
            TC              : OUT   STD_LOGIC_VECTOR (7 DOWNTO 0)
            );
    END COMPONENT;

    COMPONENT set_adc_spi IS
        PORT (
            CLK50 : IN  STD_LOGIC;
            RST   : IN  STD_LOGIC;
            -- ADS527x
            conf  : IN  STD_LOGIC_VECTOR(2 DOWNTO 0);
            nCS   : OUT STD_LOGIC;
            SCLK  : OUT STD_LOGIC;
            SDATA : OUT STD_LOGIC;
            -- test connector
            TC    : OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
            );
    END COMPONENT;



    SIGNAL CLKin         : STD_LOGIC;
    SIGNAL CLK50         : STD_LOGIC;
    SIGNAL CLK100        : STD_LOGIC;
    SIGNAL CLK100_RAM_RD : STD_LOGIC;
    SIGNAL RST           : STD_LOGIC;
    SIGNAL RSTn          : STD_LOGIC;

    SIGNAL dcm_sys_locked : STD_LOGIC;
    SIGNAL dcm_adc_locked : STD_LOGIC;

    -- Mimostar2
    SIGNAL CkRdC : STD_LOGIC;
    SIGNAL SYNC  : STD_LOGIC;



    ---------------------------------------------------------------------------
    -- testing
    ---------------------------------------------------------------------------
    TYPE   state_type IS (IDLE, DO_MEM_TEST, WAIT_MEM_TEST_DONE, END_MEM_TEST, CLR_MEM, WAIT_CLR_MEM, RESYNC, WAIT_FOREVER, WAIT_DAQ_DONE, READ_FRAME, DONE);
    SIGNAL state : state_type;
    SIGNAL busy  : STD_LOGIC;

    SIGNAL do_resync  : STD_LOGIC;
    SIGNAL do_clr_mem : STD_LOGIC;
    SIGNAL do_daq     : STD_LOGIC;

    SIGNAL daq_busy      : STD_LOGIC;
    SIGNAL do_frame      : STD_LOGIC;
    SIGNAL frame_done    : STD_LOGIC;
    SIGNAL full_frame_wr : STD_LOGIC;

    SIGNAL full_frame_data : STD_LOGIC_VECTOR (11 DOWNTO 0);

    SIGNAL RST_CLK : STD_LOGIC;
    SIGNAL RST_DCM : STD_LOGIC;

    SIGNAL TCdaq         : STD_LOGIC_VECTOR (7 DOWNTO 0);
    SIGNAL adc_reset_int : STD_LOGIC;

    SIGNAL mem_test      : STD_LOGIC;
    SIGNAL mem_test_busy : STD_LOGIC;
    SIGNAL mem_test_done : STD_LOGIC;
    SIGNAL mem_test_fail : STD_LOGIC;

    
BEGIN  -- hft_daughtercard_top_arch

    -- POWER ON reste
--    ROC_inst : ROC
--        -- synthesis translate_off 
--        GENERIC MAP (WIDTH => 100 ns)
--        -- synthesis translate_on 
--        PORT MAP (O        => RST);

    PROCESS (RST_CLK)
        VARIABLE RSTcnt : INTEGER RANGE 0 TO 100000 := 0;
    BEGIN  -- PROCESS
        IF RST_CLK'EVENT AND RST_CLK = '1' THEN  -- rising clock edge
            IF RSTcnt < 1000 THEN
                RST_DCM <= '1';
            ELSE
                RST_DCM <= '0';
            END IF;
            IF RSTcnt < 9999 THEN
                RST <= '1';
                --RSTcnt := RSTcnt + 1;
            ELSE
                RST <= '0';  --jumpers(3); --'0' ;
            END IF;
            IF RSTcnt < 99000 THEN
                RSTcnt := RSTcnt + 1;
            ELSE
                IF dcm_sys_locked = '0' THEN
                    RSTcnt := 0;
                END IF;
            END IF;
        END IF;
    END PROCESS;
    RSTn <= NOT RST;


    -- LVDS clock input
--    LVDS_CLK_in : IBUFGDS_LVDS_33
--        PORT MAP (
--            I  => CLK_p,
--            IB => CLK_n,
--            O  => CLKin
--            );

    -- 50MHz IN
    -- 50MHz OUT
    -- 100MHz OUT
    -- 100MHz 270deg out
    dcm50to100_inst : dcm50to100
        PORT MAP (
            U1_CLKIN_N_IN        => CLK_n,
            U1_CLKIN_P_IN        => CLK_p,
            --U1_CLKIN_IN        => CLKin,
            U1_RST_IN            => RST_DCM,
            --U1_CLKIN_IBUFG_OUT => RST_CLK,
            U1_CLKIN_IBUFGDS_OUT => RST_CLK,
            --U1_CLK0_OUT          => CLK50,
            U1_CLK2X_OUT         => OPEN,
            U1_STATUS_OUT        => OPEN,
            U2_CLKDV_OUT         => CLK50,
            U2_CLK0_OUT          => CLK100,
            U2_CLK270_OUT        => CLK100_RAM_RD,
            U2_LOCKED_OUT        => dcm_sys_locked,
            U2_STATUS_OUT        => OPEN
            );

    daq_inst : daq
        PORT MAP (
            CLK50           => CLK50,
            CLK100          => CLK100,
            CLK100_RAM_RD   => CLK100_RAM_RD,
            RST             => RST,
            RST_DCM         => '0',
            dcm_adc_locked  => dcm_adc_locked,
            -- setup
            idle            => OPEN,
            fake_data_mode  => '1',
            do_resync       => do_resync,
            resync_status   => OPEN,
            do_daq          => do_daq,
            daq_busy        => daq_busy,
            do_cds_daq      => '0',
            cds_daq_busy    => OPEN,
            get_nios_io     => '0',
            got_nios_io     => OPEN,
            do_clr_mem      => do_clr_mem,
            clr_mem_busy    => busy,
            daq_error       => OPEN,
            ack_daq_error   => '0',
            do_mem_test     => mem_test,
            mem_test_busy   => mem_test_busy,
            mem_test_done   => mem_test_done,
            -- random read write
            nios_addr       => "000000000000000000",
            nios_wr         => '0',
            nios_data_wr    => x"00000000_00000000_00000000_00000000_0000",
            nios_data_rd    => OPEN,
            -- full frame read
            do_full_frame   => do_frame,
            full_frame_busy => OPEN,
            adc_select      => "000",
        sort_select      => "00",
            full_frame_addr => OPEN,
            full_frame_data => full_frame_data,
            full_frame_wr   => full_frame_wr,
            full_frame_done => frame_done,
            -- mem test
            mem_test_fail   => mem_test_fail,
            -- mimostar2
            CkRdC           => CkRdC,
            SYNC            => SYNC,
            MxFirst         => MxFirst,
            -- ADS5271
            ADCLK           => ADC_CLK,
            Ads5273Rst      => adc_reset_int,  --ADC_RESET_N,
--            LclkP           => ADC_LCLK_P,
--            LclkN           => ADC_LCLK_N,
            LclkP           => ADC_LCLK_P,
            LclkN           => ADC_LCLK_N,
            AdClkP          => ADC_ADCLK_P,
            AdClkN          => ADC_ADCLK_N,
            DatInCh0P       => ADC_D1_P,
            DatInCh0N       => ADC_D1_N,
            DatInCh1P       => ADC_D2_P,
            DatInCh1N       => ADC_D2_N,
            DatInCh2P       => ADC_D3_P,
            DatInCh2N       => ADC_D3_N,
            DatInCh3P       => ADC_D4_P,
            DatInCh3N       => ADC_D4_N,
            DatInCh4P       => ADC_D5_P,
            DatInCh4N       => ADC_D5_N,
            DatInCh5P       => ADC_D6_P,
            DatInCh5N       => ADC_D6_N,
            DatInCh6P       => ADC_D7_P,
            DatInCh6N       => ADC_D7_N,
            DatInCh7P       => ADC_D8_P,
            DatInCh7N       => ADC_D8_N,
            -- sram
            RAM_A           => RAM_A,
            RAM_DQ          => RAM_DQ,
            RAM_ADV         => RAM_ADV,
            RAM_CK_A        => RAM_CK_A,
            RAM_CK_B        => RAM_CK_B,
            RAM_E2          => RAM_E2,
            RAM_FT          => RAM_FT,
            RAM_G           => RAM_G,
            RAM_LBO         => RAM_LBO,
            RAM_RnW         => RAM_RnW,
            RAM_PE          => RAM_PE,
            RAM_ZQ          => RAM_ZQ,
            -- to cluster finder
            pixel_addr      => OPEN,
            cluster_we      => OPEN,
            -- test connector
            TC              => TCdaq           --OPEN
            );
    ADC_RESET_N <= adc_reset_int;              --and jumpers(3);
    RSTB        <= '1';

    mimostar_clk_buff : OBUFDS_LVDS_33
        PORT MAP (
            O  => CkRdC_p,
            OB => CkRdC_n,
            I  => CkRdC
            );

    mimostar_sync_buff : OBUFDS_LVDS_33
        PORT MAP (
            O  => SYNC_p,
            OB => SYNC_n,
            I  => SYNC
            );



    inst_set_adc_spi : set_adc_spi
        PORT MAP (
            CLK50 => CLK50,
            RST   => RSTn,
            -- ADS527x
            conf  => jumpers (2 DOWNTO 0),
            nCS   => nCS,
            SCLK  => SCLK,
            SDATA => SDATA,
            -- test connector
            TC    => OPEN
            );



    -- temporary test code to trigger DAQ
    
    PROCESS (CLK50, RST)
        VARIABLE DAQ_wait_cnt : INTEGER;
    BEGIN  -- PROCESS
        IF RST = '1' THEN               -- asynchronous reset (active high)
            state <= IDLE;
        ELSIF CLK50'EVENT AND CLK50 = '1' THEN  -- rising clock edge
            mem_test   <= '0';
            do_clr_mem <= '0';
            do_resync  <= '0';
            do_daq     <= '0';
            do_frame   <= '0';
            CASE state IS
                WHEN IDLE =>
                    IF jumpers(1) = '1' THEN
                        state <= CLR_MEM;
                    ELSE
                        state <= DO_MEM_TEST;  --CLR_MEM;                            
                    END IF;
                    DAQ_wait_cnt := 0;
                WHEN DO_MEM_TEST =>
                    mem_test <= '1';
                    IF mem_test_busy = '1' THEN
                        state <= WAIT_MEM_TEST_DONE;
                    END IF;
                WHEN WAIT_MEM_TEST_DONE =>
                    mem_test <= '1';
                    IF mem_test_done = '1' THEN
                        state <= END_MEM_TEST;
                    END IF;
                WHEN END_MEM_TEST =>
                    mem_test <= '1';
                    -- latch mem test error bits
                    LED(9)   <= mem_test_fail;
                    state    <= CLR_MEM;
                WHEN CLR_MEM =>
                    do_clr_mem <= '1';
                    IF busy = '1' THEN
                        state <= WAIT_CLR_MEM;
                    END IF;
                WHEN WAIT_CLR_MEM =>
                    IF busy = '0' THEN
                        state <= RESYNC;
                    END IF;
                WHEN RESYNC =>
                    do_resync <= '1';
                    do_daq    <= '1';
                    state     <= WAIT_FOREVER;
                WHEN WAIT_FOREVER =>
                    do_daq       <= '1';
                    DAQ_wait_cnt := DAQ_wait_cnt + 1;
                    IF daq_busy = '1' AND DAQ_wait_cnt >= 50000 + 2*(660*128) THEN
                        state <= WAIT_DAQ_DONE;
                    END IF;
                WHEN WAIT_DAQ_DONE =>
                    IF daq_busy = '0' THEN
                        state <= READ_FRAME;
                    END IF;
                WHEN READ_FRAME =>
                    do_frame <= '1';
                    IF frame_done = '1' THEN
                        state        <= DONE;
                        DAQ_wait_cnt := 0;
                    END IF;
                WHEN DONE =>
                    DAQ_wait_cnt := DAQ_wait_cnt + 1;
                    IF DAQ_wait_cnt > 5000000 THEN
                        state <= IDLE;
                    END IF;
                WHEN OTHERS =>
                    NULL;
            END CASE;
        END IF;
    END PROCESS;


    -- LEDs
    -- 1 doing clear
    LED(1) <= '1' WHEN state = CLR_MEM OR state = WAIT_CLR_MEM       ELSE '0';
    -- 2 resync mimostar
    LED(2) <= '1' WHEN state = RESYNC                                ELSE '0';
    -- 3 doing daq
    LED(3) <= '1' WHEN state = WAIT_FOREVER OR state = WAIT_DAQ_DONE ELSE '0';
    -- 4 doing frame read
    --LED(4) <= '1' WHEN state = READ_FRAME                            ELSE '0';
    -- 5 IDLE
    LED(5) <= '1' WHEN state = IDLE OR state = DONE                  ELSE '0';


    LED(4) <= RST;
    -- ????
    LED(6) <= TCdaq(0);                 --'Z';

    -- 7 fifo wr
    --LED(7) <= full_frame_wr;

    PROCESS (CLK50, RST)
    BEGIN  -- PROCESS
        IF RST = '1' THEN               -- asynchronous reset (active high)
            LED(8) <= '0';
        ELSIF CLK50'EVENT AND CLK50 = '1' THEN  -- rising clock edge
            IF full_frame_data = x"000" THEN
                LED(8) <= '1';
            ELSE
                LED(8) <= '0';
            END IF;
        END IF;
    END PROCESS;

    PROCESS (CLK50, RSTn)
        --PROCESS (RST_CLK, RSTn)
        VARIABLE div : STD_LOGIC;
    BEGIN  -- PROCESS
        IF RSTn = '0' THEN              -- asynchronous reset (active low)
            div := '0';
        ELSIF CLK50'EVENT AND CLK50 = '1' THEN  -- rising clock edge
            --ELSIF RST_CLK'EVENT AND RST_CLK = '1' THEN  -- rising clock edge
            LED(7) <= div;
            div    := NOT div;
        END IF;
    END PROCESS;
    -- LED(9)  <= dcm_adc_locked;
    LED(10) <= dcm_sys_locked;          -- AND dcm_adc_locked;


    -- misc
    TC <= "ZZZZZZZZ";
--    nCS   <= 'Z';
--    SDATA <= 'Z';
--    SCLK  <= 'Z';


END hft_daughtercard_top_arch;

