LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_unsigned.ALL;
USE IEEE.std_logic_arith.ALL;

ENTITY hft_daughtercard_top_tb IS

END hft_daughtercard_top_tb;


ARCHITECTURE hft_daughtercard_top_tb_arch OF hft_daughtercard_top_tb IS

    COMPONENT G8162Z72
        GENERIC (
            CONSTANT A_size      : INTEGER := 18;
            CONSTANT DQ_size     : INTEGER := 9;
            CONSTANT bank_size   : INTEGER := 1024 * 256;  -- *16M /4 bytes in parallel
            --133MHZ
            CONSTANT tKQpipe     : TIME    := 4.0 ns;
            CONSTANT tKQflow     : TIME    := 8.5 ns;
            CONSTANT tKQXpipe    : TIME    := 1.5 ns;
            CONSTANT tKQXflow    : TIME    := 3.0 ns;
            CONSTANT scan_length : INTEGER := 194);-- ZZ recovery-- Dq  -- Dq  
        PORT (
            SIGNAL A816 : IN    STD_LOGIC_VECTOR(A_size - 1 DOWNTO 0);  -- address
            SIGNAL DQa  : INOUT STD_LOGIC_VECTOR(DQ_size DOWNTO 1) BUS;  -- byte A data
            SIGNAL DQb  : INOUT STD_LOGIC_VECTOR(DQ_size DOWNTO 1) BUS;  -- byte B data
            SIGNAL DQc  : INOUT STD_LOGIC_VECTOR(DQ_size DOWNTO 1) BUS;  -- byte C data
            SIGNAL DQd  : INOUT STD_LOGIC_VECTOR(DQ_size DOWNTO 1) BUS;  -- byte D data
            SIGNAL DQe  : INOUT STD_LOGIC_VECTOR(DQ_size DOWNTO 1) BUS;  -- byte C data
            SIGNAL DQf  : INOUT STD_LOGIC_VECTOR(DQ_size DOWNTO 1) BUS;  -- byte D data
            SIGNAL DQg  : INOUT STD_LOGIC_VECTOR(DQ_size DOWNTO 1) BUS;  -- byte C data
            SIGNAL DQh  : INOUT STD_LOGIC_VECTOR(DQ_size DOWNTO 1) BUS;  -- byte D data
            SIGNAL nBa  : IN    STD_LOGIC;  -- bank A write enable
            SIGNAL nBb  : IN    STD_LOGIC;  -- bank B write enable
            SIGNAL nBc  : IN    STD_LOGIC;  -- bank C write enable
            SIGNAL nBd  : IN    STD_LOGIC;  -- bank D write enable
            SIGNAL nBe  : IN    STD_LOGIC;  -- bank A write enable
            SIGNAL nBf  : IN    STD_LOGIC;  -- bank B write enable
            SIGNAL nBg  : IN    STD_LOGIC;  -- bank C write enable
            SIGNAL nBh  : IN    STD_LOGIC;  -- bank D write enable
            SIGNAL CK   : IN    STD_LOGIC;  -- clock
                                            --    SIGNAL nCKE  : IN std_logic;-- only for NBT devices
            SIGNAL nW   : IN    STD_LOGIC;
            SIGNAL nE1  : IN    STD_LOGIC;  -- chip enable 1
            SIGNAL E2   : IN    STD_LOGIC;  -- chip enable 1
            SIGNAL nE3  : IN    STD_LOGIC;  -- chip enable 1
            SIGNAL nG   : IN    STD_LOGIC;  -- output enable
            SIGNAL pADV : IN    STD_LOGIC;  -- Advance not / load
            SIGNAL ZZ   : IN    STD_LOGIC;  -- power down
            SIGNAL nFT  : IN    STD_LOGIC;  -- Pipeline / Flow through
            SIGNAL nLBO : IN    STD_LOGIC;  -- Linear Burst Order not
            SIGNAL nPE  : IN    STD_LOGIC;
            SIGNAL ZQ   : IN    STD_LOGIC;
            SIGNAL TMS  : IN    STD_LOGIC;  -- Scan Test Mode Select
            SIGNAL TDI  : IN    STD_LOGIC;  -- Scan Test Data In
            SIGNAL TDO  : OUT   STD_LOGIC;  -- Scan Test Data Out
            SIGNAL TCK  : IN    STD_LOGIC);                -- Scan Test Clock
    END COMPONENT;

    -- sram
    SIGNAL RAM_A_FPGA   : STD_LOGIC_VECTOR (17 DOWNTO 0)  := (OTHERS => 'L');
    SIGNAL RAM_A        : STD_LOGIC_VECTOR (17 DOWNTO 0)  := (OTHERS => 'L');
    SIGNAL RAM_DQ       : STD_LOGIC_VECTOR (143 DOWNTO 0) := (OTHERS => 'L');
    SIGNAL RAM_DQ_FPGA  : STD_LOGIC_VECTOR (143 DOWNTO 0) := (OTHERS => 'L');
    SIGNAL RAM_ADV      : STD_LOGIC;
    SIGNAL RAM_CK_A     : STD_LOGIC;
    SIGNAL RAM_CK_B     : STD_LOGIC;
    SIGNAL RAM_E2       : STD_LOGIC;
    SIGNAL RAM_FT       : STD_LOGIC;
    SIGNAL RAM_G        : STD_LOGIC;
    SIGNAL RAM_LBO      : STD_LOGIC;
    SIGNAL RAM_RnW      : STD_LOGIC;
    SIGNAL RAM_RnW_FPGA : STD_LOGIC;
    SIGNAL RAM_PE       : STD_LOGIC;
    SIGNAL RAM_ZQ       : STD_LOGIC;

    SIGNAL RAM_CK_A_del : STD_LOGIC;
    SIGNAL RAM_CK_B_del : STD_LOGIC;

    -- mimostar2
    COMPONENT mimostar_emu_serialRO
        PORT (
            reset_sync, clk : IN  STD_LOGIC;
            chan_a, chan_b  : OUT STD_LOGIC_VECTOR(11 DOWNTO 0);
            clk10           : OUT STD_LOGIC;
            clk5            : OUT STD_LOGIC;
            mxfirst         : OUT STD_LOGIC;
            ssync           : OUT STD_LOGIC;
            rstmk           : OUT STD_LOGIC;
            lastcol         : OUT STD_LOGIC;
            lastrow         : OUT STD_LOGIC
            );
    END COMPONENT;



    -- dut
    COMPONENT hft_daughtercard_top
        PORT (
            CLK_p       : IN    STD_LOGIC;
            CLK_n       : IN    STD_LOGIC;
            -- Mimostar 2
            CkRdC_p       : OUT   STD_LOGIC;
            CkRdC_n       : OUT   STD_LOGIC;
            SYNC_p        : OUT   STD_LOGIC;
            SYNC_n        : OUT   STD_LOGIC;
            MxFirst     : IN    STD_LOGIC;
        RSTB : OUT STD_LOGIC;
            -- ADS interface
            ADC_CLK     : OUT   STD_LOGIC;
            ADC_RESET_N : OUT   STD_LOGIC;
            ADC_LCLK_P  : IN    STD_LOGIC;  -- Six times ADCLK
            ADC_LCLK_N  : IN    STD_LOGIC;
            ADC_ADCLK_P : IN    STD_LOGIC;  -- Skewed and Data aligned version of the ADC sample clock. Used as strobe.
            ADC_ADCLK_N : IN    STD_LOGIC;
            ADC_D1_P    : IN    STD_LOGIC;
            ADC_D1_N    : IN    STD_LOGIC;
            ADC_D2_P    : IN    STD_LOGIC;
            ADC_D2_N    : IN    STD_LOGIC;
            ADC_D3_P    : IN    STD_LOGIC;
            ADC_D3_N    : IN    STD_LOGIC;
            ADC_D4_P    : IN    STD_LOGIC;
            ADC_D4_N    : IN    STD_LOGIC;
            ADC_D5_P    : IN    STD_LOGIC;
            ADC_D5_N    : IN    STD_LOGIC;
            ADC_D6_P    : IN    STD_LOGIC;
            ADC_D6_N    : IN    STD_LOGIC;
            ADC_D7_P    : IN    STD_LOGIC;
            ADC_D7_N    : IN    STD_LOGIC;
            ADC_D8_P    : IN    STD_LOGIC;
            ADC_D8_N    : IN    STD_LOGIC;
            -- ADS527x SPI
            nCS         : OUT   STD_LOGIC;
            SCLK        : OUT   STD_LOGIC;
            SDATA       : OUT   STD_LOGIC;
            -- SRAM
            RAM_A       : OUT   STD_LOGIC_VECTOR (17 DOWNTO 0);
            RAM_DQ      : INOUT STD_LOGIC_VECTOR (143 DOWNTO 0);
            RAM_ADV     : OUT   STD_LOGIC;
            RAM_CK_A    : OUT   STD_LOGIC;
            RAM_CK_B    : OUT   STD_LOGIC;
            RAM_E2      : OUT   STD_LOGIC;
            RAM_FT      : OUT   STD_LOGIC;
            RAM_G       : OUT   STD_LOGIC;
            RAM_LBO     : OUT   STD_LOGIC;
            RAM_RnW     : OUT   STD_LOGIC;
            RAM_PE      : OUT   STD_LOGIC;
            RAM_ZQ      : OUT   STD_LOGIC;
            -- LEDs
		  jumpers	: in std_logic_vector (3 downto 0);
                  LED         : OUT   STD_LOGIC_VECTOR (10 DOWNTO 1);
            TC          : OUT   STD_LOGIC_VECTOR(7 DOWNTO 0)
            );
    END COMPONENT;



    SIGNAL mimo_clk     : STD_LOGIC;
    SIGNAL mimo_sync    : STD_LOGIC;
    SIGNAL mimo_mxfirst : STD_LOGIC;



    SIGNAL CLK   : STD_LOGIC := '0';
    SIGNAL CLK_n : STD_LOGIC := '1';

BEGIN  -- hft_daughtercard_top_tb_arch

    CLK   <= NOT CLK AFTER 10 ns;
    CLK_n <= NOT CLK;




    -- dut
    hft_daughtercard_top_inst : hft_daughtercard_top
        PORT MAP (
            CLK_p       => CLK,
            CLK_n       => CLK_n,
            -- Mimostar 2
            CkRdC_p       => mimo_clk,
            CkRdC_n       => open,
            SYNC_p        => mimo_sync,
            SYNC_n        => open,
            MxFirst     => mimo_mxfirst,
        RSTB => open,
            -- ADS interface
            ADC_CLK     => OPEN,
            ADC_RESET_N => OPEN,
            ADC_LCLK_P  => '0',
            ADC_LCLK_N  => '0',
            ADC_ADCLK_P => '0',
            ADC_ADCLK_N => '0',
            ADC_D1_P    => '0',
            ADC_D1_N    => '0',
            ADC_D2_P    => '0',
            ADC_D2_N    => '0',
            ADC_D3_P    => '0',
            ADC_D3_N    => '0',
            ADC_D4_P    => '0',
            ADC_D4_N    => '0',
            ADC_D5_P    => '0',
            ADC_D5_N    => '0',
            ADC_D6_P    => '0',
            ADC_D6_N    => '0',
            ADC_D7_P    => '0',
            ADC_D7_N    => '0',
            ADC_D8_P    => '0',
            ADC_D8_N    => '0',
            -- ADS527x SPI
            nCS         => OPEN,
            SCLK        => OPEN,
            SDATA       => OPEN,
            -- SRAM
            RAM_A       => RAM_A_FPGA,
            RAM_DQ      => RAM_DQ,
            RAM_ADV     => RAM_ADV,
            RAM_CK_A    => RAM_CK_A,
            RAM_CK_B    => RAM_CK_B,
            RAM_E2      => RAM_E2,
            RAM_FT      => RAM_FT,
            RAM_G       => RAM_G,
            RAM_LBO     => RAM_LBO,
            RAM_RnW     => RAM_RnW_FPGA,
            RAM_PE      => RAM_PE,
            RAM_ZQ      => RAM_ZQ,
            -- LEDs
	    jumpers => "0000",
            LED         => OPEN,
            TC          => OPEN
            );











    RAM_CK_A_del <= TRANSPORT RAM_CK_A     AFTER 4 ns;
    RAM_CK_B_del <= TRANSPORT RAM_CK_B     AFTER 4 ns;
    RAM_A        <= TRANSPORT RAM_A_FPGA   AFTER 5 ns;
    RAM_RnW      <= TRANSPORT RAM_RnW_FPGA AFTER 5 ns;

    SRAM_A : G8162Z72
        PORT MAP (
            A816 => RAM_A,
            DQa  => RAM_DQ (8 DOWNTO 0),
            DQb  => RAM_DQ (17 DOWNTO 9),
            DQc  => RAM_DQ (26 DOWNTO 18),
            DQd  => RAM_DQ (35 DOWNTO 27),
            DQe  => RAM_DQ (44 DOWNTO 36),
            DQf  => RAM_DQ (53 DOWNTO 45),
            DQg  => RAM_DQ (62 DOWNTO 54),
            DQh  => RAM_DQ (71 DOWNTO 63),
            nBa  => '0',
            nBb  => '0',
            nBc  => '0',
            nBd  => '0',
            nBe  => '0',
            nBf  => '0',
            nBg  => '0',
            nBh  => '0',
            CK   => RAM_CK_A_del,
                                        --    SIGNAL nCKE  : IN std_logic;-- only for NBT devices
            nW   => RAM_RnW,
            nE1  => '0',
            E2   => RAM_E2,
            nE3  => '0',
            nG   => RAM_G,
            pADV => RAM_ADV,
            ZZ   => '0',
            nFT  => RAM_FT,
            nLBO => RAM_LBO,
            nPE  => RAM_PE,
            ZQ   => RAM_ZQ,
            TMS  => '1',
            TDI  => '1',
            TDO  => OPEN,
            TCK  => '1'
            );

    SRAM_B : G8162Z72
        PORT MAP (
            A816 => RAM_A,
            DQa  => RAM_DQ (80 DOWNTO 72),
            DQb  => RAM_DQ (89 DOWNTO 81),
            DQc  => RAM_DQ (98 DOWNTO 90),
            DQd  => RAM_DQ (107 DOWNTO 99),
            DQe  => RAM_DQ (116 DOWNTO 108),
            DQf  => RAM_DQ (125 DOWNTO 117),
            DQg  => RAM_DQ (134 DOWNTO 126),
            DQh  => RAM_DQ (143 DOWNTO 135),
            nBa  => '0',
            nBb  => '0',
            nBc  => '0',
            nBd  => '0',
            nBe  => '0',
            nBf  => '0',
            nBg  => '0',
            nBh  => '0',
            CK   => RAM_CK_B_del ,
                                        --    SIGNAL nCKE  : IN std_logic;-- only for NBT devices
            nW   => RAM_RnW,
            nE1  => '0',
            E2   => RAM_E2,
            nE3  => '0',
            nG   => RAM_G,
            pADV => RAM_ADV,
            ZZ   => '0',
            nFT  => RAM_FT,
            nLBO => RAM_LBO,
            nPE  => RAM_PE,
            ZQ   => RAM_ZQ,
            TMS  => '1',
            TDI  => '1',
            TDO  => OPEN,
            TCK  => '1'
            );

    -- mimostar2
    mimostar2 : mimostar_emu_serialRO
        PORT MAP (
            reset_sync => mimo_sync,
            clk        => mimo_clk,
            chan_a     => OPEN,
            chan_b     => OPEN,
            clk10      => OPEN,
            clk5       => OPEN,
            mxfirst    => mimo_mxfirst,
            ssync      => OPEN,
            rstmk      => OPEN,
            lastcol    => OPEN,
            lastrow    => OPEN
            );


END hft_daughtercard_top_tb_arch;













