LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_unsigned.ALL;
USE IEEE.std_logic_arith.ALL;

ENTITY hft_mem_test IS
    PORT (
        CLK           : IN  STD_LOGIC;
        RST           : IN  STD_LOGIC;
        --
        do_mem_test   : IN  STD_LOGIC;
        mem_test_busy : OUT STD_LOGIC;
        mem_test_done : OUT STD_LOGIC;
        status        : OUT STD_LOGIC;
        -- failed tests
        fail_cnt      : OUT STD_LOGIC;
        fail_ff00     : OUT STD_LOGIC;
        fail_wr_rd    : OUT STD_LOGIC;
        -- clear mem
        clrmem        : OUT STD_LOGIC;
        clrmem_busy   : IN  STD_LOGIC;
        -- sram
        sram_idle     : IN  STD_LOGIC;
        get_io        : OUT STD_LOGIC;
        got_io        : IN  STD_LOGIC;
        addr          : OUT STD_LOGIC_VECTOR (17 DOWNTO 0);
        data_wr       : OUT STD_LOGIC_VECTOR (143 DOWNTO 0);
        data_rd       : IN  STD_LOGIC_VECTOR (143 DOWNTO 0);
        io_wr         : OUT STD_LOGIC;
        --io_mode       : OUT STD_LOGIC;
        -- led
        --LED           : OUT STD_LOGIC_VECTOR (10 DOWNTO 1);
        --
        tc            : OUT STD_LOGIC
        );
END hft_mem_test;


ARCHITECTURE mem_test_arch OF hft_mem_test IS

    TYPE   state_type IS (IDLE, CLR_MEM, CHK_CLR, CNT_WR, CNT_RD, FF00_WR, FF00_RD, WR_RD_CHK, FINISH);
    SIGNAL state : state_type;

    SIGNAL addr_cnt  : STD_LOGIC_VECTOR (20 DOWNTO 0);
    SIGNAL read_addr : STD_LOGIC_VECTOR (20 DOWNTO 0);
    SIGNAL read_data : STD_LOGIC_VECTOR (143 DOWNTO 0);

    SIGNAL doing_cnt_test  : STD_LOGIC;
    SIGNAL status_cnt      : STD_LOGIC;
    SIGNAL doing_ff00_test : STD_LOGIC;
    SIGNAL status_ff00     : STD_LOGIC;
    SIGNAL status_wr_rd    : STD_LOGIC;
    SIGNAL doing_wr_rd     : STD_LOGIC;

BEGIN  -- mem_test_arch

    main_test_loop : PROCESS (CLK, RST)
        VARIABLE alive : STD_LOGIC;
    BEGIN  -- PROCESS main_test_loop
        IF RST = '0' THEN                   -- asynchronous reset (active low)
            state    <= IDLE;
            addr_cnt <= (OTHERS => '0');
            alive    := '0';
        ELSIF CLK'EVENT AND CLK = '1' THEN  -- rising clock edge
            --LED(9)  <= alive;
            clrmem <= '0';
            --io_mode <= '0';
            io_wr  <= '0';
            CASE state IS
                WHEN IDLE =>
                    -- wait a bit
                    addr_cnt <= addr_cnt + 1;
                    IF do_mem_test = '1' AND sram_idle = '1'THEN
                        clrmem   <= '1';
                        addr_cnt <= (OTHERS => '0');
                        IF clrmem_busy = '1' THEN
                            state <= CLR_MEM;
                        END IF;
                        alive := NOT alive;
                    END IF;
                WHEN CLR_MEM =>
                    --initiate mem clear. wait clear done.
                    addr_cnt <= addr_cnt + 1;
                    IF got_io = '1' THEN
                        addr_cnt <= (OTHERS => '0');
                        state    <= CHK_CLR;
                    END IF;
                WHEN CHK_CLR =>
                    --io_mode  <= '1';
                    -- check all locations clear
                    addr     <= addr_cnt (17 DOWNTO 0);
                    addr_cnt <= addr_cnt + 1;
                    IF addr_cnt = 262143 THEN
                        addr_cnt <= (OTHERS => '0');
                        state    <= CNT_WR;
                    END IF;
                WHEN CNT_WR =>
                    --io_mode              <= '1';
                    io_wr                <= '1';
                    -- write counter to mem. XOR some of the data bits
                    addr                 <= addr_cnt (17 DOWNTO 0);
                    data_wr              <= (OTHERS => '0');
                    data_wr(20 DOWNTO 0) <= addr_cnt;
                    addr_cnt             <= addr_cnt + 1;
                    IF addr_cnt = 262143 THEN
                        addr_cnt <= (OTHERS => '0');
                        state    <= CNT_RD;
                    END IF;
                WHEN CNT_RD =>
                    --io_mode  <= '1';
                    -- verify counter
                    addr     <= addr_cnt (17 DOWNTO 0);
                    addr_cnt <= addr_cnt + 1;
                    IF addr_cnt = 262143 THEN
                        addr_cnt <= (OTHERS => '0');
                        state    <= FF00_WR;
                    END IF;
                    NULL;
                WHEN FF00_WR =>
                    --io_mode <= '1';
                    io_wr <= '1';
                    -- write altrnating 0x00000000 and 0xFFFFFFFF
                    addr  <= addr_cnt (17 DOWNTO 0);
                    IF addr_cnt(0) = '0' THEN
                        data_wr <= (OTHERS => '0');
                        data_wr <= x"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa";
                        --data_wr(107 DOWNTO 72) <= (OTHERS=>'0');
                    ELSE
                        data_wr <= (OTHERS => '1');
                        data_wr <= x"555555555555555555555555555555555555";
                        --data_wr(107 DOWNTO 72) <= (OTHERS=>'1');
                    END IF;
                    addr_cnt <= addr_cnt + 1;
                    IF addr_cnt = 262143 THEN
                        addr_cnt <= (OTHERS => '0');
                        state    <= FF00_RD;
                    END IF;
                WHEN FF00_RD =>
                    --io_mode  <= '1';
                    -- verify alternating 0x00000000 and 0xFFFFFFFF
                    addr     <= addr_cnt (17 DOWNTO 0);
                    addr_cnt <= addr_cnt + 1;
                    IF addr_cnt = 262143 THEN
                        addr_cnt <= (OTHERS => '0');
                        state    <= WR_RD_CHK;
                    END IF;
                    NULL;
                WHEN WR_RD_CHK =>
                    --io_mode <= '1';
                    IF addr_cnt(0) = '0' THEN
                        io_wr <= '0';
                    ELSE
                        io_wr <= '1';
                    END IF;
                    -- alternate write and read. read a few mem locations behind
                    IF addr_cnt(0) = '0' THEN
                        addr <= addr_cnt (18 DOWNTO 1) - 10;
                    ELSE
                        addr <= addr_cnt (18 DOWNTO 1);
                    END IF;
                    --addr    <= addr_cnt (18 DOWNTO 1);
                    data_wr                 <= (OTHERS => '0');
                    data_wr(20 -1 DOWNTO 0) <= addr_cnt (20 DOWNTO 1);

                    addr_cnt <= addr_cnt + 1;
                    IF addr_cnt = 524287 THEN
                        addr_cnt <= (OTHERS => '0');
                        state    <= FINISH;
                    END IF;
                WHEN FINISH =>
                    addr_cnt <= addr_cnt + 1;
                    -- do after test loop cleanup
--                    IF addr_cnt = 1000000 THEN
                    addr_cnt <= (OTHERS => '0');
                    IF do_mem_test = '0' THEN
                        state <= IDLE;
                    END IF;
--                    END IF;
                    NULL;
                WHEN OTHERS => NULL;
            END CASE;
        END IF;
    END PROCESS main_test_loop;

    get_io        <= '0' WHEN state = IDLE OR state = FINISH ELSE '1';
    mem_test_busy <= '0' WHEN state = IDLE                   ELSE '1';
    mem_test_done <= '1' WHEN state = FINISH                 ELSE '0';

    read_addr_delay_for_CHK : PROCESS (CLK, RST)
        VARIABLE temp_addr  : STD_LOGIC_VECTOR (20 DOWNTO 0);
        VARIABLE temp_addr1 : STD_LOGIC_VECTOR (20 DOWNTO 0);
        VARIABLE temp_addr2 : STD_LOGIC_VECTOR (20 DOWNTO 0);
        VARIABLE temp_addr3 : STD_LOGIC_VECTOR (20 DOWNTO 0);
        VARIABLE temp_addr4 : STD_LOGIC_VECTOR (20 DOWNTO 0);
        VARIABLE temp_addr5 : STD_LOGIC_VECTOR (20 DOWNTO 0);
    BEGIN  -- PROCESS read_addr_delay_for CHK
        IF RST = '0' THEN               -- asynchronous reset (active low)

        ELSIF CLK'EVENT AND CLK = '1' THEN  -- rising clock edge
            read_addr  <= temp_addr;
            temp_addr  := temp_addr1;
            temp_addr1 := temp_addr2;
            temp_addr2 := temp_addr3;
            temp_addr3 := temp_addr4;
            temp_addr4 := temp_addr5;
            temp_addr5 := addr_cnt;
            --read_addr <= temp_addr;
            --temp_addr := addr_cnt;
            read_data  <= data_rd;
        END IF;
    END PROCESS read_addr_delay_for_CHK;

    check_count : PROCESS (CLK, RST)
        VARIABLE test : STD_LOGIC_VECTOR (6 DOWNTO 0);
    BEGIN  -- PROCESS check_count
        IF RST = '0' THEN                   -- asynchronous reset (active low)
            test       := (OTHERS => '0');
            status_cnt <= '0';
        ELSIF CLK'EVENT AND CLK = '1' THEN  -- rising clock edge
            test (5 DOWNTO 0) := test (6 DOWNTO 1);
            IF state = CNT_RD THEN
                test(6) := '1';
            ELSE
                test(6) := '0';
            END IF;
            doing_cnt_test <= test(0);
            IF test(0) = '1' THEN
                IF read_addr >= 0 THEN
                    
                    IF read_data (20 DOWNTO 0) = read_addr AND read_data (143 DOWNTO 21) = 0 THEN
                        -- pass
                        NULL;
                    ELSE
                        status_cnt <= '1';
                    END IF;
                END IF;
            END IF;
            IF state = IDLE THEN
                status_cnt <= '0';
            END IF;
        END IF;
    END PROCESS check_count;

    check_FF00 : PROCESS (CLK, RST)
        VARIABLE test : STD_LOGIC_VECTOR (6 DOWNTO 0);
    BEGIN  -- PROCESS check_count
        IF RST = '0' THEN                   -- asynchronous reset (active low)
            test        := (OTHERS => '0');
            status_ff00 <= '0';
        ELSIF CLK'EVENT AND CLK = '1' THEN  -- rising clock edge
            test (5 DOWNTO 0) := test (6 DOWNTO 1);
            IF state = FF00_RD THEN
                test(6) := '1';
            ELSE
                test(6) := '0';
            END IF;
            doing_ff00_test <= test(0);
            IF test(0) = '1' THEN
            --    IF (read_addr(0) = '0' AND read_data = x"000000000000000000000000000000000000") OR
            --        (read_addr(0) = '1' AND read_data = x"ffffffffffffffffffffffffffffffffffff") THEN
                IF (read_addr(0) = '0' AND read_data = x"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa") OR
                    (read_addr(0) = '1' AND read_data = x"555555555555555555555555555555555555") THEN
                    -- pass
                    NULL;
                ELSE
                    status_ff00 <= '1';
                END IF;
            END IF;
            IF state = IDLE THEN
                status_ff00 <= '0';
            END IF;
        END IF;
    END PROCESS check_FF00;

    check_rd_wr : PROCESS (CLK, RST)
        VARIABLE test : STD_LOGIC_VECTOR (6 DOWNTO 0);
    BEGIN  -- PROCESS check_rd_wr
        IF RST = '0' THEN                   -- asynchronous reset (active low)
            test         := (OTHERS => '0');
            status_wr_rd <= '0';
        ELSIF CLK'EVENT AND CLK = '1' THEN  -- rising clock edge
            test (5 DOWNTO 0) := test (6 DOWNTO 1);
            IF state = WR_RD_CHK THEN
                test(6) := '1';
            ELSE
                test(6) := '0';
            END IF;
            doing_wr_rd <= test(0);
            IF test(0) = '1' THEN
                IF read_addr >= 22 THEN     -- skip while we fill the lookback
                    IF read_addr(0) = '0' THEN
                        IF (read_addr(20 DOWNTO 1) - 10) = read_data (19 DOWNTO 0) AND read_data (143 DOWNTO 20) = 0 THEN
                            NULL;
                            -- match
                        ELSE
                            status_wr_rd <= '1';
                        END IF;
                    END IF;
                END IF;
            END IF;
            IF state = IDLE THEN
                status_wr_rd <= '0';
            END IF;
        END IF;
    END PROCESS check_rd_wr;

    status <= status_cnt XOR status_ff00 XOR status_wr_rd;

    --LED(1) <= status_cnt;
    --LED(2) <= status_ff00;
    --LED(3) <= status_wr_rd;
    --LED(4) <= status_cnt XOR status_ff00 XOR status_wr_rd;

    fail_cnt   <= status_cnt;
    fail_ff00  <= status_ff00;
    fail_wr_rd <= status_wr_rd;

    PROCESS (CLK, RST)
    BEGIN  -- PROCESS
        IF RST = '0' THEN                   -- asynchronous reset (active low)
            --        LED(5) <= '0';
        ELSIF CLK'EVENT AND CLK = '1' THEN  -- rising clock edge
            IF status_cnt = '1' OR status_ff00 = '1' OR status_wr_rd = '1' THEN
                --            LED(5) <= '1';
            END IF;
        END IF;
    END PROCESS;

    PROCESS (CLK, RST)
        VARIABLE cnt    : INTEGER RANGE 0 TO 40000000;
        VARIABLE ledout : STD_LOGIC;
    BEGIN  -- PROCESS
        IF RST = '0' THEN                   -- asynchronous reset (active low)
            cnt    := 0;
            ledout := '0';
        ELSIF CLK'EVENT AND CLK = '1' THEN  -- rising clock edge
            IF cnt = 40000000 THEN
                cnt    := 0;
                ledout := NOT ledout;
            END IF;
            cnt := cnt + 1;
            --        LED(10) <= ledout;
        END IF;
    END PROCESS;

    --LED(8) <= '1';

END mem_test_arch;



