LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;

ENTITY hft_testbench IS

END hft_testbench;

ARCHITECTURE hft_testbench_arch OF hft_testbench IS

    COMPONENT hft_top
        PORT (
            CLK_p    : IN    STD_LOGIC;
            CLK_n    : IN    STD_LOGIC;
            -- SRAM
            RAM_A    : OUT   STD_LOGIC_VECTOR (17 DOWNTO 0);
            RAM_DQ   : INOUT STD_LOGIC_VECTOR (143 DOWNTO 0);
            RAM_ADV  : OUT   STD_LOGIC;
            RAM_CK_A : OUT   STD_LOGIC;
            RAM_CK_B : OUT   STD_LOGIC;
            RAM_E2   : OUT   STD_LOGIC;
            RAM_FT   : OUT   STD_LOGIC;
            RAM_G    : OUT   STD_LOGIC;
            RAM_LBO  : OUT   STD_LOGIC;
            RAM_RnW  : OUT   STD_LOGIC;
            RAM_PE   : OUT   STD_LOGIC;
            RAM_ZQ   : OUT   STD_LOGIC;
            -- LEDs
            LED      : OUT   STD_LOGIC_VECTOR (10 DOWNTO 1);
            -- TC
            TC       : OUT   STD_LOGIC_VECTOR (7 DOWNTO 0)
            );
    END COMPONENT;

    COMPONENT G8162Z72
        GENERIC (
            CONSTANT A_size      : INTEGER := 18;
            CONSTANT DQ_size     : INTEGER := 9;
            CONSTANT bank_size   : INTEGER := 1024 * 256;  -- *16M /4 bytes in parallel
            --133MHZ
            CONSTANT tKQpipe     : TIME    := 4.0 ns;
            CONSTANT tKQflow     : TIME    := 8.5 ns;
            CONSTANT tKQXpipe    : TIME    := 1.5 ns;
            CONSTANT tKQXflow    : TIME    := 3.0 ns;
            CONSTANT scan_length : INTEGER := 194);-- ZZ recovery-- Dq  -- Dq  
        PORT (
            SIGNAL A816 : IN    STD_LOGIC_VECTOR(A_size - 1 DOWNTO 0);  -- address
            SIGNAL DQa  : INOUT STD_LOGIC_VECTOR(DQ_size DOWNTO 1) BUS;  -- byte A data
            SIGNAL DQb  : INOUT STD_LOGIC_VECTOR(DQ_size DOWNTO 1) BUS;  -- byte B data
            SIGNAL DQc  : INOUT STD_LOGIC_VECTOR(DQ_size DOWNTO 1) BUS;  -- byte C data
            SIGNAL DQd  : INOUT STD_LOGIC_VECTOR(DQ_size DOWNTO 1) BUS;  -- byte D data
            SIGNAL DQe  : INOUT STD_LOGIC_VECTOR(DQ_size DOWNTO 1) BUS;  -- byte C data
            SIGNAL DQf  : INOUT STD_LOGIC_VECTOR(DQ_size DOWNTO 1) BUS;  -- byte D data
            SIGNAL DQg  : INOUT STD_LOGIC_VECTOR(DQ_size DOWNTO 1) BUS;  -- byte C data
            SIGNAL DQh  : INOUT STD_LOGIC_VECTOR(DQ_size DOWNTO 1) BUS;  -- byte D data
            SIGNAL nBa  : IN    STD_LOGIC;  -- bank A write enable
            SIGNAL nBb  : IN    STD_LOGIC;  -- bank B write enable
            SIGNAL nBc  : IN    STD_LOGIC;  -- bank C write enable
            SIGNAL nBd  : IN    STD_LOGIC;  -- bank D write enable
            SIGNAL nBe  : IN    STD_LOGIC;  -- bank A write enable
            SIGNAL nBf  : IN    STD_LOGIC;  -- bank B write enable
            SIGNAL nBg  : IN    STD_LOGIC;  -- bank C write enable
            SIGNAL nBh  : IN    STD_LOGIC;  -- bank D write enable
            SIGNAL CK   : IN    STD_LOGIC;  -- clock
                                            --    SIGNAL nCKE  : IN std_logic;-- only for NBT devices
            SIGNAL nW   : IN    STD_LOGIC;
            SIGNAL nE1  : IN    STD_LOGIC;  -- chip enable 1
            SIGNAL E2   : IN    STD_LOGIC;  -- chip enable 1
            SIGNAL nE3  : IN    STD_LOGIC;  -- chip enable 1
            SIGNAL nG   : IN    STD_LOGIC;  -- output enable
            SIGNAL pADV : IN    STD_LOGIC;  -- Advance not / load
            SIGNAL ZZ   : IN    STD_LOGIC;  -- power down
            SIGNAL nFT  : IN    STD_LOGIC;  -- Pipeline / Flow through
            SIGNAL nLBO : IN    STD_LOGIC;  -- Linear Burst Order not
            SIGNAL nPE  : IN    STD_LOGIC;
            SIGNAL ZQ   : IN    STD_LOGIC;
            SIGNAL TMS  : IN    STD_LOGIC;  -- Scan Test Mode Select
            SIGNAL TDI  : IN    STD_LOGIC;  -- Scan Test Data In
            SIGNAL TDO  : OUT   STD_LOGIC;  -- Scan Test Data Out
            SIGNAL TCK  : IN    STD_LOGIC);                -- Scan Test Clock
    END COMPONENT;



    -- sram
    SIGNAL RAM_A_FPGA  : STD_LOGIC_VECTOR (17 DOWNTO 0) := (OTHERS=> 'L');
    SIGNAL RAM_A       : STD_LOGIC_VECTOR (17 DOWNTO 0) := (OTHERS=> 'L');
    SIGNAL RAM_DQ      : STD_LOGIC_VECTOR (143 DOWNTO 0) := (OTHERS=> 'L');
    SIGNAL RAM_DQ_FPGA : STD_LOGIC_VECTOR (143 DOWNTO 0) := (OTHERS=> 'L');
    SIGNAL RAM_ADV     : STD_LOGIC;
    SIGNAL RAM_CK_A    : STD_LOGIC;
    SIGNAL RAM_CK_B    : STD_LOGIC;
    SIGNAL RAM_E2      : STD_LOGIC;
    SIGNAL RAM_FT      : STD_LOGIC;
    SIGNAL RAM_G       : STD_LOGIC;
    SIGNAL RAM_LBO     : STD_LOGIC;
    SIGNAL RAM_RnW     : STD_LOGIC;
    SIGNAL RAM_RnW_FPGA     : STD_LOGIC;
    SIGNAL RAM_PE      : STD_LOGIC;
    SIGNAL RAM_ZQ      : STD_LOGIC;

    SIGNAL RAM_CK_A_del : STD_LOGIC;
    SIGNAL RAM_CK_B_del : STD_LOGIC;



    SIGNAL CLK   : STD_LOGIC := '0';
    SIGNAL CLK_n : STD_LOGIC := '1';

BEGIN  -- hft_testbench_arch

    CLK   <= NOT CLK AFTER 5 ns;
    CLK_n <= NOT CLK;



    dut : hft_top
        PORT MAP (
            CLK_p    => CLK,
            CLK_n    => CLK_n,
            -- SRAM
            RAM_A    => RAM_A_FPGA,
            RAM_DQ   => RAM_DQ,
            RAM_ADV  => RAM_ADV,
            RAM_CK_A => RAM_CK_A,
            RAM_CK_B => RAM_CK_B,
            RAM_E2   => RAM_E2,
            RAM_FT   => RAM_FT,
            RAM_G    => RAM_G,
            RAM_LBO  => RAM_LBO,
            RAM_RnW  => RAM_RnW_FPGA,
            RAM_PE   => RAM_PE,
            RAM_ZQ   => RAM_ZQ,
            -- LEDs
            LED      => OPEN,
            -- TC
            TC       => OPEN
            );







--    RAM_CK_A_del <= TRANSPORT RAM_CK_A AFTER 5 ns;
--    RAM_CK_B_del <= TRANSPORT RAM_CK_B AFTER 5 ns;
    RAM_CK_A_del <= TRANSPORT RAM_CK_A AFTER 4 ns;
    RAM_CK_B_del <= TRANSPORT RAM_CK_B AFTER 4 ns;

----    RAM_DQ     <= TRANSPORT RAM_DQ_FPGA AFTER 9 ns;
    RAM_A <= TRANSPORT RAM_A_FPGA       AFTER 5 ns;
--
    RAM_RnW <= TRANSPORT RAM_RnW_FPGA AFTER 5 ns;
--    
--    PROCESS (RAM_DQ, RAM_DQ_FPGA)
--    BEGIN  -- PROCESS
----        IF RAM_DQ_FPGA'EVENT OR RAM_DQ'EVENT THEN
--            FOR i IN 0 TO 143 LOOP
--                IF RAM_DQ_FPGA(i) = '1' THEN
--                    RAM_DQ(i) <= TRANSPORT '1' AFTER 5 ns;
--                ELSIF RAM_DQ_FPGA(i) = '0' THEN
--                    RAM_DQ(i) <= TRANSPORT '0' AFTER 5 ns;
--                ELSE
--                    RAM_DQ(i) <= TRANSPORT 'Z' AFTER 5 ns;
--                END IF;
--                IF RAM_DQ(i) = 'H' THEN
--                    RAM_DQ_FPGA(i) <= 'H';
--                ELSIF RAM_DQ(i) = 'L' THEN
--                    RAM_DQ_FPGA(i) <= 'L';
--                    ELSE
--                        RAM_DQ_FPGA(i) <= 'Z';
--                END IF;
--            END LOOP;  -- i
----        END IF;
--    END PROCESS;

    SRAM_A : G8162Z72
        PORT MAP (
            A816 => RAM_A,
            DQa  => RAM_DQ (8 DOWNTO 0),
            DQb  => RAM_DQ (17 DOWNTO 9),
            DQc  => RAM_DQ (26 DOWNTO 18),
            DQd  => RAM_DQ (35 DOWNTO 27),
            DQe  => RAM_DQ (44 DOWNTO 36),
            DQf  => RAM_DQ (53 DOWNTO 45),
            DQg  => RAM_DQ (62 DOWNTO 54),
            DQh  => RAM_DQ (71 DOWNTO 63),
            nBa  => '0',
            nBb  => '0',
            nBc  => '0',
            nBd  => '0',
            nBe  => '0',
            nBf  => '0',
            nBg  => '0',
            nBh  => '0',
            CK   => RAM_CK_A_del,
                                        --    SIGNAL nCKE  : IN std_logic;-- only for NBT devices
            nW   => RAM_RnW,
            nE1  => '0',
            E2   => RAM_E2,
            nE3  => '0',
            nG   => RAM_G,
            pADV => RAM_ADV,
            ZZ   => '0',
            nFT  => RAM_FT,
            nLBO => RAM_LBO,
            nPE  => RAM_PE,
            ZQ   => RAM_ZQ,
            TMS  => '1',
            TDI  => '1',
            TDO  => OPEN,
            TCK  => '1'
            );

    SRAM_B : G8162Z72
        PORT MAP (
            A816 => RAM_A,
            DQa  => RAM_DQ (80 DOWNTO 72),
            DQb  => RAM_DQ (89 DOWNTO 81),
            DQc  => RAM_DQ (98 DOWNTO 90),
            DQd  => RAM_DQ (107 DOWNTO 99),
            DQe  => RAM_DQ (116 DOWNTO 108),
            DQf  => RAM_DQ (125 DOWNTO 117),
            DQg  => RAM_DQ (134 DOWNTO 126),
            DQh  => RAM_DQ (143 DOWNTO 135),
            nBa  => '0',
            nBb  => '0',
            nBc  => '0',
            nBd  => '0',
            nBe  => '0',
            nBf  => '0',
            nBg  => '0',
            nBh  => '0',
            CK   => RAM_CK_B_del ,
                                        --    SIGNAL nCKE  : IN std_logic;-- only for NBT devices
            nW   => RAM_RnW,
            nE1  => '0',
            E2   => RAM_E2,
            nE3  => '0',
            nG   => RAM_G,
            pADV => RAM_ADV,
            ZZ   => '0',
            nFT  => RAM_FT,
            nLBO => RAM_LBO,
            nPE  => RAM_PE,
            ZQ   => RAM_ZQ,
            TMS  => '1',
            TDI  => '1',
            TDO  => OPEN,
            TCK  => '1'
            );




END hft_testbench_arch;
