LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;

ENTITY io_arbitriation IS
    PORT (
        CLK                   : IN  STD_LOGIC;
        RST                   : IN  STD_LOGIC;
        -- SRAM io PORT
        get_io                : OUT STD_LOGIC;
        got_io                : IN  STD_LOGIC;
        io_wr                 : OUT STD_LOGIC;
        io_addr               : OUT STD_LOGIC_VECTOR (17 DOWNTO 0);
        io_data_wr            : OUT STD_LOGIC_VECTOR (143 DOWNTO 0);
        io_data_rd            : IN  STD_LOGIC_VECTOR (143 DOWNTO 0);
        -- full frame
        get_io_full_frame     : IN  STD_LOGIC;
        got_io_full_frame     : OUT STD_LOGIC;
        io_addr_full_frame    : IN  STD_LOGIC_VECTOR (17 DOWNTO 0);
        io_data_rd_full_frame : OUT STD_LOGIC_VECTOR (143 DOWNTO 0);
        -- nios IO
        get_io_nios           : IN  STD_LOGIC;
        got_io_nios           : OUT STD_LOGIC;
        io_wr_nios            : IN  STD_LOGIC;
        io_addr_nios          : IN  STD_LOGIC_VECTOR (17 DOWNTO 0);
        io_data_wr_nios       : IN  STD_LOGIC_VECTOR (143 DOWNTO 0);
        io_data_rd_nios       : OUT STD_LOGIC_VECTOR (143 DOWNTO 0);
        -- mem test
        get_io_mem_test       : IN  STD_LOGIC;
        got_io_mem_test       : OUT STD_LOGIC;
        io_wr_mem_test        : IN  STD_LOGIC;
        io_addr_mem_test      : IN  STD_LOGIC_VECTOR (17 DOWNTO 0);
        io_data_wr_mem_test   : IN  STD_LOGIC_VECTOR (143 DOWNTO 0);
        io_data_rd_mem_test   : OUT STD_LOGIC_VECTOR (143 DOWNTO 0);
        -- test connector
        TC                    : OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
        );
END io_arbitriation;

ARCHITECTURE io_arbitriation_arch OF io_arbitriation IS

BEGIN  -- io_arbitriation_arch

    PROCESS (CLK, RST)
    BEGIN  -- PROCESS
        IF rst = '1' THEN
            get_io            <= '0';
            got_io_full_frame <= '0';
            got_io_nios       <= '0';
            io_wr             <= '0';
            io_addr           <= (OTHERS => '0');
            io_data_wr        <= (OTHERS => '0');
        ELSIF CLK'EVENT AND CLK = '1' THEN
            got_io_full_frame <= '0';
            got_io_nios       <= '0';
            io_wr             <= '0';
            IF get_io_full_frame = '1' THEN
                get_io            <= '1';
                got_io_full_frame <= got_io;
                io_wr             <= '0';
                io_addr           <= io_addr_full_frame;
                io_data_wr        <= (OTHERS => '0');  --io_data_wr_full_frame;
            ELSIF get_io_nios = '1' THEN
                get_io      <= '1';
                got_io_nios <= got_io;
                io_wr       <= io_wr_nios;
                io_addr     <= io_addr_nios;
                io_data_wr  <= io_data_wr_nios;
            ELSIF get_io_mem_test = '1' THEN
                get_io          <= '1';
                got_io_mem_test <= got_io;
                io_wr           <= io_wr_mem_test;
                io_addr         <= io_addr_mem_test;
                io_data_wr      <= io_data_wr_mem_test;
            ELSE
                get_io            <= '0';
                got_io_full_frame <= '0';
                got_io_nios       <= '0';
                io_wr             <= '0';
                io_addr           <= (OTHERS => '0');
                io_data_wr        <= (OTHERS => '0');
            END IF;
        END IF;
    END PROCESS;

    io_data_rd_full_frame <= io_data_rd;
    io_data_rd_nios       <= io_data_rd;
    io_data_rd_mem_test   <= io_data_rd;

END io_arbitriation_arch;
