LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_signed.ALL;
USE IEEE.std_logic_arith.ALL;

ENTITY tmp IS

END tmp;

ARCHITECTURE tmp_arch OF tmp IS

    SIGNAL clk   : STD_LOGIC := '0';
    TYPE   statetype IS (IDLE, FULL_FRAME_RD, FULL_FRAME_ACK);
    SIGNAL state : statetype := IDLE;

    SIGNAL lines      : INTEGER RANGE 0 TO 127;
    SIGNAL pixels     : INTEGER RANGE 0 TO 65;
    SIGNAL subframes  : INTEGER RANGE 0 TO 9;
    SIGNAL marker_cnt : INTEGER RANGE -1 TO 19;

    SIGNAL frame_rd : STD_LOGIC := '0';

    SIGNAL fifo_almost_full : STD_LOGIC := '0';
    SIGNAL sorted           : STD_LOGIC := '0';

    SIGNAL fifo_wr_sig : STD_LOGIC;
    SIGNAL ram_we      : STD_LOGIC;
    SIGNAL write_sig   : STD_LOGIC;
    SIGNAL RAM_ADDR    : STD_LOGIC_VECTOR (17 DOWNTO 0);
    
BEGIN  -- tmp_arch

    clk      <= NOT clk AFTER 5 ns;
    frame_rd <= '1'     AFTER 250 ns, '0' AFTER 900 us, '1' AFTER 1 ms;
    sorted   <= '1'     AFTER 900 us;

    PROCESS(CLK)
    BEGIN
        IF CLK'EVENT AND CLK = '1' THEN
            
            CASE state IS
                WHEN IDLE =>
                    lines      <= 0;
                    pixels     <= 2;
                    subframes  <= 0;
                    marker_cnt <= -1;
                    RAM_ADDR   <= (OTHERS => '0');
                    IF frame_rd = '1' THEN
                        state <= FULL_FRAME_RD;
                    END IF;
                WHEN FULL_FRAME_RD =>
                    IF frame_rd = '0' THEN
                        fifo_wr_sig <= '0';
                        ram_we      <= '1';
                        write_sig   <= '0';
                        RAM_ADDR    <= (OTHERS => '0');
                        state       <= IDLE;
                    ELSE
                        IF fifo_almost_full = '1' THEN
                            fifo_wr_sig <= '0';
                        ELSE
                            fifo_wr_sig <= '1';
                            IF sorted = '0' THEN
                                ram_we    <= '1';
                                write_sig <= '0';
                                -- do not sort pixels
                                RAM_ADDR  <= RAM_ADDR + 1;
                                IF RAM_ADDR = 660*128-2 THEN
                                    state <= FULL_FRAME_ACK;
                                ELSE
                                    state <= FULL_FRAME_RD;
                                END IF;
                            ELSE
                                -- sort pixels
                                IF lines = 128-1 AND pixels = 66-1 AND subframes = 10-1 AND marker_cnt=20-1 THEN
                                    state <= FULL_FRAME_ACK;
                                ELSE
                                    IF pixels = 66-1 AND subframes = 10-1 THEN
                                        IF marker_cnt = 20 -1 THEN
                                            lines     <= lines + 1;
                                            pixels    <= 2;
                                            subframes <= 0;
                                        ELSE
                                            marker_cnt <= marker_cnt + 1;
                                        END IF;
                                    ELSE
                                        marker_cnt <= -1;
                                        IF subframes = 10-1 THEN
                                            pixels    <= pixels + 1;
                                            subframes <= 0;
                                        ELSE
                                            subframes <= subframes + 1;
                                        END IF;
                                    END IF;
                                    IF pixels = 66-1 AND subframes = 10-1 AND marker_cnt/=-1 THEN
                                        RAM_ADDR <= conv_std_logic_vector(lines*10*66 + marker_cnt, 18);
                                    ELSE
                                        RAM_ADDR <= conv_std_logic_vector(lines*(10*66) + subframes*66 + pixels, 18);
                                    END IF;

                                    state <= FULL_FRAME_RD;
                                END IF;
                            END IF;
                        END IF;
                    END IF;
                WHEN FULL_FRAME_ACK =>
                    fifo_wr_sig <= '0';
                    ram_we      <= '1';
                    write_sig   <= '0';
                    RAM_ADDR    <= (OTHERS => '0');
                    RAM_ADDR    <= conv_std_logic_vector(lines*(10*66) + subframes*66 + pixels, 18);
                    IF frame_rd = '0' THEN
                        state <= IDLE;
                    END IF;
                WHEN OTHERS =>
                    NULL;
            END CASE;
        END IF;
    END PROCESS;

END tmp_arch;

