LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_unsigned.ALL;
USE IEEE.std_logic_arith.ALL;

ENTITY set_adc_spi IS
    PORT (
        CLK50 : IN  STD_LOGIC;
        RST   : IN  STD_LOGIC;
        -- ADS527x
        conf  : IN  STD_LOGIC_VECTOR (2 DOWNTO 0);
        nCS   : OUT STD_LOGIC;
        SCLK  : OUT STD_LOGIC;
        SDATA : OUT STD_LOGIC;
        -- test connector
        TC    : OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
        );
END set_adc_spi;


ARCHITECTURE set_adc_spi_arch OF set_adc_spi IS

    COMPONENT adc_spi
        PORT (
            CLK   : IN  STD_LOGIC;
            RST   : IN  STD_LOGIC;
            -- internal interface
            data  : IN  STD_LOGIC_VECTOR (7 DOWNTO 0);
            send  : IN  STD_LOGIC;
            busy  : OUT STD_LOGIC;
            -- ADS527x
            nCS   : OUT STD_LOGIC;
            SCLK  : OUT STD_LOGIC;
            SDATA : OUT STD_LOGIC;
            -- test connector
            TC    : OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
            );
    END COMPONENT;

    SIGNAL send : STD_LOGIC;
    SIGNAL busy : STD_LOGIC;
    SIGNAL data : STD_LOGIC_VECTOR (7 DOWNTO 0);

    TYPE   state_type IS (IDLE, WT1, WT2, WT3, WT4);
    SIGNAL state : state_type;
    
BEGIN  -- set_adc_spi_arch


    PROCESS (CLK50, RST)
        VARIABLE cnt : INTEGER := 0;
    BEGIN  -- PROCESS
        IF RST = '0' THEN               -- asynchronous reset (active low)
            cnt  := 0;
--            send <= '0';
        ELSIF CLK50'EVENT AND CLK50 = '1' THEN  -- rising clock edge
            IF cnt /= 1000005 THEN
                cnt := cnt + 1;
            END IF;
            IF cnt = 1000000 THEN
--                send <= '1';
                --ELSE
                -- send <= '0';
            END IF;
        END IF;
    END PROCESS;

--    data <= "0000" & conf (2 DOWNTO 1) & "00";
    adc_spi_int : adc_spi
        PORT MAP (
            CLK => CLK50,
            RST => RST,

              data => data , --"0000" & conf (2 downto 1) & "00",                          -- internal interface
            --data  => "00000100",
            --data  => "00000000",
            --data  => "00000011",
            --data  => "00010001",
            --data  => "00001000",
            send  => send, --conf(0),           --send,
            busy  => busy,
                                        -- ADS527x
            nCS   => nCS,
            SCLK  => SCLK,
            SDATA => SDATA,
                                        -- test connector
            TC    => TC
            );

    PROCESS (CLK50, RST)
        VARIABLE cnt : INTEGER;
    BEGIN  -- PROCESS CLK50
        IF RST = '0' THEN               -- asynchronous reset (active low)
            state <= IDLE;
            cnt   := 0;
        ELSIF CLK50'EVENT AND CLK50 = '1' THEN  -- rising clock edge
            IF cnt > 0 THEN
                cnt := cnt - 1;
            END IF;
            CASE state IS
                WHEN IDLE =>
                    cnt   := 10;
                    send  <= '1';
                    data  <= "01000001";
                    state <= WT1;
                WHEN WT1 =>
                    send <= '0';
                    IF cnt = 0 AND busy = '0' THEN
                        cnt   := 10;
                        send  <= '1';
                        data  <= "01011001";
                        state <= WT2;
                    END IF;
                WHEN WT2 =>
                    send <= '0';
                    IF cnt = 0 AND busy = '0' THEN
                        cnt   := 10;
                        send  <= '1';
                        data  <= "01100010";
                        state <= WT3;
                    END IF;
                WHEN WT3 =>
                    send <= '0';
                    IF cnt = 0 AND busy = '0' THEN
                        cnt   := 10000000;
                        send  <= '1';
								if conf(0)='1' then
                           data  <= "00001100";
							   else 
                           data  <= "00000000";
							   end if;
                        state <= WT4;
                    END IF;
                WHEN WT4 =>
                    send <= '0';
                    IF cnt = 0 AND busy = '0' THEN
                        cnt   := 10000000;
                     --   send  <= '1';
                     --   data  <= "01010101";
                        state <= IDLE;
                    END IF;
                WHEN OTHERS => NULL;
            END CASE;
        END IF;
    END PROCESS;


END set_adc_spi_arch;
