--------------------------------------------------------------------------------
--	 module name :ClusterFifos
--function: waiting for a trigger, after trigger comes, prepares a data head when waiting the 
--real data come, then accept the real data until it ends and add a end token at the end of the data. 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
-- in ARCHITECTURE 
-- between component          ccccccccccccccccccccccccccccccccccccccccccccccccccc
-- between signal declaration sssssssssssssssssssssssssssssssssssssssssssssssssss
-- between wire connent       wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww
-- between process            ppppppppppppppppppppppppppppppppppppppppppppppppppp
-- between instantiation	   iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii

entity ClusterFifos is
generic (
	ADDRSIZE 	: integer  :=9 ;
	DATASIZE 	: integer  :=32;
	WAIT_WINDOW_LENGTH 	: integer  :=60;
	CLUSTER_WINDOW_LENGTH	:	integer :=2000;
	FIFO_LENGTH	:	integer :=200;
	CHIP_ID	:	integer :=10
	 );
port (
 	reset_n : in std_logic ;	  

-- clocks
	pix_clk 	: in std_logic ; -- same clock that moves pixels through the cluster finder
--	dbdb_clk	: in std_lobic ; -- read end clk, Daughter Board Data Bus

-- trigger data
	trg_word: in std_logic_vector(31 downto 0) ;	
	thresholdr_i: in std_logic_vector(8 downto 0) ;	
	thresholdc_i: in std_logic_vector(8 downto 0) ;	
	adc_i: in std_logic_vector(11 downto 0) ;--   idle_state : in std_logic_vector(4 downto 0);
														
															-- this gets loaded into the beginning of each
															-- event block 
--	chip_id : in std_logic_vector(11 downto 0) ; -- ladder(11 downto 6) & chip(5 downto 0)
												-- this also gets loaded at the beginning of each block
															
-- input data and interface control
	data_in: in std_logic_vector(17 downto 0) ;															
	got_cluster	: in std_logic;  --from cluster finder, found one
	start			: in std_logic; -- contains TRG command, DAQ command, token
--	got_cluster : in std_logic; -- from cluster finder, found one
	idle_flag	:out std_logic; --true when module is in idle waiting for trigger
--	writting_flag	:out std_logic; --true when module is in idle waiting for trigger
--	do_aborts	:	in std_logic; -- a control bit to enable module to respond to trigger aborts
	
-- output data and interface control
--	address		: in  std_logic_vector(5 downto 0);-- each CFM for the ladder has unique address
		fifo_data1			: INout	STD_LOGIC ;-- the data I.E. the address of cluster center
		fifo_data2			: INout	STD_LOGIC_VECTOR(19 downto 0) ;-- the data I.E. the address of cluster center
		fifo_data3			: INout	STD_LOGIC ;-- the data I.E. the address of cluster center
		fifo_data4			: INout	STD_LOGIC ;-- the data I.E. the address of cluster center
		fifo_data5			: INout	STD_LOGIC_VECTOR(8 downto 0) ;-- the data I.E. the address of cluster center
	read_ready	: out std_logic; --true when module is ready to be read out, tri stated.
	sum_ready	: out std_logic; --true when module is ready to be read out, tri stated.
	rdreq	:	in std_logic; -- true causes fifo to clock data, q, out until empty (when this CFM addressed)

----new 
	read_clk 		: in std_logic;
	chip_address	: in std_logic_vector(6 downto 0)
	
		);
end entity ClusterFifos;

ARCHITECTURE a of ClusterFifos is


--cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
component myfifo
	PORT
	(
		data		: IN STD_LOGIC_VECTOR (31 DOWNTO 0);
		wrreq		: IN STD_LOGIC ;
		rdreq		: IN STD_LOGIC ;
		rdclk		: IN STD_LOGIC ;
		wrclk		: IN STD_LOGIC ;
		aclr		: IN STD_LOGIC  := '0';
		q			: OUT STD_LOGIC_VECTOR (31 DOWNTO 0);
		rdempty		: OUT STD_LOGIC ;
		wrfull		: OUT STD_LOGIC 
	);
end component;
--ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
--sssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss
type stype is (WAIT_TRG, WAIT_WINDOW,WRITTING,WAIT_READ);
type rtype is (IDLE,READING);
signal state: stype:=WAIT_TRG;




--signal write_distribute : STD_LOGIC_vector(4 downto 0):="00000";
signal write_distribute : STD_LOGIC:='0';
--signal get_distribute 	: STD_LOGIC_vector(4 downto 0):="00000";
signal get_distribute 	: STD_LOGIC:='0';
-- we use write_distribute and get_distribute as a trigger token before and after trigger comeing


signal trg_word_buf 			: STD_LOGIC_vector(31 downto 0);
signal write_data 			: STD_LOGIC_vector(DATASIZE-1 downto 0);
signal modify_data 			: STD_LOGIC_vector(DATASIZE-1 downto 0);
signal read_data 			: STD_LOGIC_vector(DATASIZE-1 downto 0);
signal write_pre : STD_LOGIC:='0';	 -- the pre request of write, 
signal read_req : STD_LOGIC:='0';	 -- read request signal
signal write_req : STD_LOGIC:='0';	 -- write request signal

signal w_write_over : STD_LOGIC:='0';	--writing being over signal 
signal r_write_over : STD_LOGIC:='0';	--reading being over signal
signal write_over : STD_LOGIC:='0';		--exchange data between write and read processes


signal window_counter : integer range 0 to 102399:=0;		--counts the length of waitting window
signal write_counter : integer range 0 to 86015 :=0;		--counts the number of data from cluster-finding  despite	there is a cluster
signal read_counter : integer range 0 to 4096 :=0;		--counts the number of data from cluster-finding  despite	there is a cluster
signal written_counter : integer range 0 to 4096:=0;	--counts the number of data written in fifo
--signal read_counter : integer:=0;		--counts the number of data read from fifo
--signal sum_counter : integer:=0;			--counts the number of data in fifo.

signal rdempty : STD_LOGIC:='0';				--no use
signal wrfull : STD_LOGIC:='0';
signal full : STD_LOGIC:='0';
signal deley : STD_LOGIC:='0';
signal read_deley : STD_LOGIC:='0';
signal empty_deley : STD_LOGIC:='0';
signal chip_selected : STD_LOGIC:='0';
--signal	trig_word:  std_logic_vector(31 downto 0) ;	
--signal mycounter 			: STD_LOGIC_vector(11 downto 0):=(others=>'0');
signal select_counter 			: STD_LOGIC_vector(4 downto 0):=(others=>'0');
--ssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss




begin
--wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww
--read_ready<= write_over  when CONV_INTEGER(chip_address)=CHIP_ID else	
--			'Z';		 -- if data writing is over, set write_over.
--
--read_req<=rdreq and write_over when CONV_INTEGER(chip_address)=CHIP_ID else	
--			'0'; --if data writing is over, reading is permitted
--

write_over<= w_write_over xor r_write_over;
--f1<=f2 xor f3;
-- sum_counter<=	written_counter-read_counter;	 --sun_counter counts data number in fifo

--  get_distribute<= write_distribute+1 when write_distribute(0)='0' else
--						 write_distribute+2 when write_distribute(1)='0' else
--						 write_distribute+4 when write_distribute(2)='0' else
--						 write_distribute+8 when write_distribute(3)='0' else
--						 write_distribute+16 when write_distribute(4)='0' else
--						 write_distribute;

--get_distribute<= '1' when write_distribute='0' else --we use write_distribute to indicate tirgger's coming.
																							-- if a trigger comes write_distribute will be set	  
--					 write_distribute;

--write_data<=x"0000007"  when	window_counter=1 else  --when we waiting in window, we can put data head in fifo
--				trg_word					   when	window_counter=2 else  --when we waiting in window, we can put data head in fifo
-- CONV_STD_LOGIC_VECTOR(CHIP_ID,32)  when window_counter=3	 else  --when we waiting in window, we can put data head in fifo
---- "000000000000000"& idle_state  		when window_counter=WAIT_WINDOW_LENGTH-1	 else  --when we waiting in window, we can put data head in fifo
--				"00000000000011000000000001011111"  when	write_counter=CLUSTER_WINDOW_LENGTH else  --when we waiting in window, we can put data head in fifo
--				"00000000000000" & data_in			when write_distribute(0)='1';
--
--
--fifo_data1<=read_data(31)   				when write_over='1'  and CONV_INTEGER(chip_address)=CHIP_ID else 'Z' ;  
--fifo_data2<=read_data(30 downto 11)   when write_over='1'  and CONV_INTEGER(chip_address)=CHIP_ID else (others=>'Z'); 
--fifo_data3<=read_data(10)  	     		when write_over='1'  and CONV_INTEGER(chip_address)=CHIP_ID else 'Z'	;
--fifo_data4<=read_data(9)  		 		when write_over='1'  and CONV_INTEGER(chip_address)=CHIP_ID else 'Z'  ;
--fifo_data5<=read_data(8 downto 0)     when write_over='1'  and CONV_INTEGER(chip_address)=CHIP_ID else (others=>'Z'); 
--



--data_out<="01010101010101010101010101010101"  when 	write_over='1'  and CONV_INTEGER(chip_address)=CHIP_ID else	--when data writing is over, reading is requested,	data_out is connected.
--				(others=>'Z');
--write_req<='1' when  (window_counter>0 and window_counter<=3) or  (write_counter=CLUSTER_WINDOW_LENGTH and state=WRITTING)  else --when we are setting data head or writing data into fifo, writing is permitted
--             write_pre and got_cluster when write_distribute(0)='1' else 
--			'0';										--write_req is the write request signal in fifo
--									--write_req is the write request signal in fifo

idle_flag<= '1' when state=WAIT_TRG   else					--  system idle when it waiting for a trigger
			'0';
--wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww
----------------------------------------------------------------------------------------------
process(pix_clk,reset_n)						  --write process
begin
if  reset_n='0' then
	  write_counter<=0;
--	  written_counter<=0;
	  window_counter<=0;
  	 write_pre<='0';									--forbid data writing.
  	 w_write_over<='0';									--forbid data writing.
	 state <= WAIT_TRG;
	write_distribute<='0';
--	writting_flag<='0';
 else


if pix_clk='1' and pix_clk'event  then

		 if write_distribute='1' then
			write_data<= "00000000000000" & data_in;
       end if;

		 if write_counter=CLUSTER_WINDOW_LENGTH then
		 if FIFO_LENGTH=1 then
			write_data<= x"55555555";
		  else
			write_data<= x"eeeeeeee";
		  end if;
       end if;

--		 if window_counter=3 then
--			write_data<= CONV_STD_LOGIC_VECTOR(CHIP_ID,32);
--       end if;
--
--		 if window_counter=2 then
--			write_data<= trg_word;
--       end if;
--
--		 if window_counter=1 then
--			write_data<= x"0000007";
--       end if;





		 if (window_counter>0 and window_counter<=17) or  (write_counter=CLUSTER_WINDOW_LENGTH and state=WRITTING) then
			write_req<='1';
       elsif write_distribute='1' then
			write_req<=write_pre and got_cluster;
		 else
			write_req<='0';
       end if;

			if written_counter=1015 then
			  	 full<='1';
			 end if;




if  start='1' then--and start'last_value='0' then		 --a trigger comes, set a bit of write_distribute
  write_distribute<='1';

end if;

if state=WAIT_READ and rdempty='1' and start='0' then	--when reading is over, zero the bit of write_distribute
write_distribute<='0';
end if;
case state is
    when WAIT_TRG	 =>								  -- waitting for a trigger, then switch to WAIT_WINDOW
--		reset <='1';
		if write_distribute ='1' then			  --if a trigger arrives , switch to WAIT_WINDOW
--		reset <='0';
--		 writting_flag<='0';
		 trg_word_buf<=trg_word;
		 window_counter<=0;							 
		 write_pre<='0';									--forbid data writing.
		 state<= WAIT_WINDOW;
--		 trig_word<=trg_word;
		 written_counter<=0;
		 end if;
		 
    when WAIT_WINDOW	 =>
		  full<='0';

 		if window_counter=WAIT_WINDOW_LENGTH-1 then		  -- WAIT_WINDOW is over , turn to WRITTING
--			writting_flag<='1';	
			write_counter<=0;	
			window_counter<=0;								  -- in WAIT_WINDOW, we set data head, so we 
		 write_pre<='1' ;									 --should add corresponding value in written_counter
	--	written_counter<=written_counter+1;
		 state<= WRITTING;
		 else
		 window_counter<=window_counter+1; 
		 end if;	
	 when WRITTING	 =>

		 	if write_counter=CLUSTER_WINDOW_LENGTH or full='1' then			--	WRITTING is over , turn to WAIT_READ
--		writting_flag<='0';
				write_pre<='0';
				 w_write_over<=not w_write_over;				-- tell reading process, you can read
				 state<= WAIT_READ;
			 else 
				 if got_cluster='1' and full='0' then
				 	 written_counter<=written_counter+1;
	-- 	 write_counter<=write_counter+1;				-- for test
				 end if;
			 	 write_counter<=write_counter+1;				-- real counter
		
			end if;
    when WAIT_READ =>
		write_counter<=0;
	 if rdempty='1' and write_distribute='0'   then	-- if reading is over and write_distribute is zeroed
		 state<= WAIT_TRG;											--got_cluster a new process.
		 end if;
end case;

end if;

end if;
end process;
-----------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------



process(read_clk,reset_n)										  --read process
begin 

if  reset_n='0' then
--	  read_counter<=0;
  	 r_write_over<='0';								
--	 	mycounter<=(others=>'0');

 else
	  
if  read_clk='1'and read_clk'event then
empty_deley<=write_over;
read_deley<=read_req;

		if  CONV_INTEGER(chip_address)=CHIP_ID and CONV_INTEGER(select_counter)>6  then
		 chip_selected<='1';
		 elsif  CONV_INTEGER(chip_address)=CHIP_ID then
		 select_counter<=select_counter+1;
		 else
		 select_counter<=(others=>'0');
		 chip_selected<='0';
		end if;

		if  chip_selected='1'  then
				if rdempty='0' then
				sum_ready<=read_deley ;
				else
				sum_ready<='0';
				end if;
		else
				sum_ready<='Z';
		end if;


    if  chip_selected='1' then
			read_ready<= empty_deley  ;
			read_req<=rdreq and empty_deley;
	 else	
			read_ready<='Z';		 -- if data writing is over, set write_over.
			read_req<='0';
    end if;

		 if write_over='1'  and chip_selected='1' and read_counter>17 and read_counter<1023 then
			fifo_data1<=read_data(31);
			fifo_data2<=read_data(30 downto 11);
			fifo_data3<=read_data(10);
			fifo_data4<=read_data(9);
			fifo_data5<=read_data(8 downto 0);
		 elsif write_over='1'  and chip_selected='1'  then
			fifo_data1<=modify_data(31);
			fifo_data2<=modify_data(30 downto 11);
			fifo_data3<=modify_data(10);
			fifo_data4<=modify_data(9);
			fifo_data5<=modify_data(8 downto 0);		 
		 else
		 	fifo_data1<='Z';
			fifo_data2<=(others=>'Z');
			fifo_data3<='Z';
			fifo_data4<='Z';
			fifo_data5<=(others=>'Z');
       end if;

--		 if write_over='1'  and CONV_INTEGER(chip_address)=CHIP_ID  then
--			fifo_data1<=read_data(31);
--			fifo_data2<=read_data(30 downto 11);
--			fifo_data3<=read_data(10);
--			fifo_data4<=read_data(9);
--			fifo_data5<=read_data(8 downto 0);
--		 else
--		 	fifo_data1<='Z';
--			fifo_data2<=(others=>'Z');
--			fifo_data3<='Z';
--			fifo_data4<='Z';
--			fifo_data5<=(others=>'Z');
--		 end if;

if read_counter=1 then
  modify_data<=x"aaaaaaaa";
end if;
if read_counter=2 then
--  modify_data<=x"000" & trg_word_buf(19 downto 0);
  modify_data<=trg_word_buf;
end if;
if read_counter=3 then
  modify_data<= CONV_STD_LOGIC_VECTOR(CHIP_ID,32);
--  modify_data<= trg_word(31 downto 20) & "0" & full & CONV_STD_LOGIC_VECTOR(CHIP_ID,10);
end if;
if read_counter=4 then
   modify_data<= CONV_STD_LOGIC_VECTOR(written_counter,32);
--   modify_data<= x"00000" & mycounter;

end if;
if read_counter=5 then
  modify_data<="00000000000000" & thresholdr_i & thresholdc_i;
end if;
if read_counter=6 then
  modify_data<=x"000" & "0000000" & full & adc_i;
end if;
if read_counter=7 then
  modify_data<=(others=>'0');
end if;
if read_counter>1021 then
		 if FIFO_LENGTH=1 then
			modify_data<= x"55555555";
		  else
			modify_data<= x"eeeeeeee";
		  end if;
end if;


		 if rdempty='1' then
			read_counter<=0;
		  elsif read_req='1'  then
		    read_counter<=read_counter+1;
		 end if;


		 	if rdempty='1' and write_over='1' then	 -- is readong is over ,tell 
--			if deley='1' then
--mycounter<=mycounter+1;
		r_write_over<=not r_write_over;					 --trigger process to zero  write_distribute
--			 deley<='0';
--			 else
--			 deley<='1';
--			 end if;
--		 	elsif  write_over='1'and rdreq='1' then	 -- reading data
--	 	 read_counter<=read_counter+1;
		end if;



end if;


end if;
end process;
---------------------------------------------------------------------------------------------

--iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii
myfifo_i: myfifo											  -- instantiate my fifo
	PORT map
	(
		data		=>write_data,
		wrreq		=>write_req,
		rdreq		=>read_req,
		rdclk		=>read_clk,
		wrclk		=>pix_clk,
		aclr		=>not reset_n,
		q			=>read_data,
		rdempty		=>rdempty,
		wrfull		=>wrfull
	);
--iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii


--wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww

--wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww
end architecture;
