--------------------------------------------------------------------------------
-- Company: 
-- Engineer:
--
-- Create Date:    11:16:14 05/15/06
-- Design Name:    
-- Module Name:    myconfig - Behavioral
-- Project Name:   
-- Target Device:  
-- Tool versions:  
-- Description:
--
-- Dependencies:
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity myconfig is
generic (
	SELECTION_D 	: integer  :=9 ; -- chip select command
	CLEANUP_D  		: integer  :=20; -- clean up command
	READPIXCLE_D 	: integer  :=6;  --read command
	READSRAM_D 	: integer  :=7;  --read command
	CONFIGSRAM_D	:	integer :=10; --write circular buffer command
	MAPSIZE_D	:	integer :=262143;
	BLOCKSIZE_D	:	integer :=10;
	OFFSET_D		:	integer :=2	;
	INVALID_D		:	integer :=21
				
	 );
    Port ( clk : in std_logic;
           reset_n : in std_logic;
 			  from_addr   :in  std_logic_vector(17 downto 0);

			  command: out std_logic_vector(9 downto 0);
			  fifo_r : out std_logic_vector(19 downto 0);
           fifo_r_req : in std_logic;
           fifo_w : in std_logic_vector(19 downto 0);
           fifo_w_req : in std_logic;
           fifo_r_empty : out std_logic;
           fifo_w_full : out std_logic;
			  avalon_clk :in std_logic
			  );
end myconfig;

architecture Behavioral of myconfig is
--cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
component myfifo
	PORT
	(
		data		: IN STD_LOGIC_VECTOR (19 DOWNTO 0);
		wrreq		: IN STD_LOGIC ;
		rdreq		: IN STD_LOGIC ;
		rdclk		: IN STD_LOGIC ;
		wrclk		: IN STD_LOGIC ;
		aclr		: IN STD_LOGIC  := '0';
		q			: OUT STD_LOGIC_VECTOR (19 DOWNTO 0);
		rdempty		: OUT STD_LOGIC ;
		wrfull		: OUT STD_LOGIC 
	);
end component;

component dfifo 
	port (
	din: IN std_logic_VECTOR(19 downto 0);
	rd_clk: IN std_logic;
	rd_en: IN std_logic;
	rst: IN std_logic;
	wr_clk: IN std_logic;
	wr_en: IN std_logic;
	dout: OUT std_logic_VECTOR(19 downto 0);
	empty: OUT std_logic;
	full: OUT std_logic);
END component;
--ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
--sssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss
type stype is (WAIT_COMMAND, CLEANUP,CHIP_SELECT, READSRAM,READING);
type com_type is (IDLE,READ_IN,SEND_OUT);
type select_type is (SELECT_IDLE,SELECT_READ_IN,SELECT_SEND_OUT);
type read_type is (WAIT_START,READ_START);
type read_sram_type is (WAIT_START_SRAM,READ_START_SRAM,READ_OVER_SRAM);
--type write_type is (WRITE_IDLE,WRITE_READ_IN,WRITE_SEND_OUT);


signal state,next_state: stype:=WAIT_COMMAND;
signal com_state,next_com_state: com_type:=IDLE;
signal sec_state,next_sec_state: select_type:=SELECT_IDLE;
--signal red_state,next_red_state: read_type:=WAIT_START;
signal red_state : std_logic_vector(1 downto 0):="00";
signal rs_state,next_rs_state: read_sram_type:=WAIT_START_SRAM;
--signal w_state,next_w_state: write_type:=WRITE_IDLE;

signal  data_config_in :  std_logic_vector(19 downto 0);
signal  data_config_out :  std_logic_vector(19 downto 0);
signal  chip_selecting:  std_logic_vector(1 downto 0):="00";
signal  sub_chip_selecting:  std_logic_vector(7 downto 0):="00000110";
signal  sub_data_selecting:  integer range 0 to 35:=0;
signal  bit_data_selecting:  std_logic_vector(3 downto 0):="0000";
signal  map_length:  std_logic_vector(17 downto 0):=(others=>'0');
signal  write_map_address:  std_logic_vector(5 downto 0):="000000";
signal  write_map_data   :  std_logic_vector(5 downto 0):="000000";
signal  sram_data   :  std_logic_vector(143 downto 0):=(others=>'0');
signal  sram_data_r   :  std_logic_vector(143 downto 0):=(others=>'0');
signal  from_addrl   :  std_logic_vector(17 downto 0):=(others=>'0');
signal  from_adcl   :  std_logic_vector(11 downto 0):=(others=>'0');

signal  block_length:  std_logic_vector(11 downto 0):=(others=>'0');
signal  add_chip_selecting:  std_logic_vector(7 downto 0);
signal  sram_writen_enable:  std_logic:='1';
signal  to_config_out_wreq_r:  std_logic;




signal           from_config_in :  std_logic_vector(19 downto 0);
signal          to_config_in_rreq :  std_logic;
signal           from_config_in_empty :  std_logic;
signal           to_config_out :  std_logic_vector(19 downto 0);
signal           to_config_out_wreq :  std_logic;
signal           from_config_out_full :  std_logic;		  


signal           wrfull :  std_logic;		  
signal           rdempty :  std_logic;	
signal          dataclock :  std_logic;	
signal          myclock :  std_logic;	
	  
--sssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss

begin
--wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww
dataclock<='1' when	 from_addr(7 downto 0)=sub_chip_selecting else
			'0';
myclock<=dataclock when	red_state="01" else
			clk;
to_config_out_wreq<='1' when	red_state="01"  else
to_config_out_wreq_r;

--to_config_out<="00000000"&	from_addr(11 downto 0)			when state=READING					   else
--					data_config_in 			   when com_state=SEND_OUT       				   else
--             "00000000000000000011"     when  sec_state=SELECT_SEND_OUT					 else
--					"00000000000000000111"     when state=CLEANUP 				           else
--					"00000000" & sram_data_r(11 downto 0)   when state=READSRAM 		else
--					(others=>'Z') ;
--to_config_out<=(others=>'Z');
--add_chip_selecting<=sub_chip_selecting-1;--+CONV_STD_LOGIC_VECTOR(OFFSET_D,8); 

--sram_data_r<=to_sram_data when	 sram_writen_enable='1' ;
		

--wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww
----------------------------------------------------------------------------------------------
--process(clk,reset_n)						  --write process
--begin
--if clk='0' and clk'event  then
--	from_addrl<=from_addr; 
-- end if;
--end process;

process(clk,reset_n)						  --write process
begin
if  reset_n='0' then
sram_writen_enable<='1';
 state<=WAIT_COMMAND;
 next_state<= WAIT_COMMAND;
com_state<=IDLE;
next_com_state<=IDLE;
sec_state<=SELECT_IDLE;
next_sec_state<=SELECT_IDLE;
--red_state<=WAIT_START;
--next_red_state<=WAIT_START;
rs_state<=WAIT_START_SRAM;
next_rs_state<=WAIT_START_SRAM;
command<=(others=>'0');
--w_state<= WRITE_IDLE;
--next_w_state<=WRITE_IDLE;
 elsif clk='1' and clk'event  then


state<=next_state;
com_state<=next_com_state;
sec_state<=next_sec_state;
--red_state<=next_red_state;
rs_state<=next_rs_state;

--w_state<=next_w_state;

 case state is
    when WAIT_COMMAND =>
		case 	com_state is
		when  IDLE =>
				 sram_writen_enable<='1';
		 		 to_config_out_wreq_r<='0';
		 		 to_config_in_rreq<='0';
		 if from_config_in_empty='0' and next_state=WAIT_COMMAND  then
		 		 to_config_in_rreq<='1';
		 next_com_state<= READ_IN;
		 end if;
		when  READ_IN =>
		 to_config_in_rreq<='1';
		data_config_in<=from_config_in;
		 next_com_state<= SEND_OUT;
		when  SEND_OUT=>
--		to_config_out<=data_config_in;
 		 to_config_in_rreq<='0';
if data_config_in=conv_std_logic_vector(SELECTION_D,20)  then
  		  if	next_com_state/= com_state then
			   		 to_config_out_wreq_r<='0'; 
			else
					 to_config_out_wreq_r<='1';
		  next_com_state<=IDLE;
		  next_state<=  CHIP_SELECT;

		  end if;		  
		 elsif data_config_in=conv_std_logic_vector(CLEANUP_D,20)  then
		  if	next_com_state/= com_state then
			   		 to_config_out_wreq_r<='0';
			else

   		 to_config_out_wreq_r<='1';
		  next_com_state<=IDLE;
		  next_state<=  CLEANUP;

		  end if;
		 elsif data_config_in=conv_std_logic_vector(READSRAM_D,20)  then
		  if	next_com_state/= com_state then
			   		 to_config_out_wreq_r<='0';
			else
   		 to_config_out_wreq_r<='1';
		  next_com_state<=IDLE;
		  next_state<=  READSRAM;

		  end if;
	    elsif data_config_in=conv_std_logic_vector(READPIXCLE_D,20)  then
 		  if	next_com_state/= com_state then
			   		 to_config_out_wreq_r<='0'; 
			else
		  to_config_out_wreq_r<='1';
		  next_com_state<=IDLE;
		  next_state<=  READING;
		  end if;


		  else
		  next_com_state<=IDLE;
		  end if;		
		  end case;

   when CLEANUP =>

			 next_state<=  WAIT_COMMAND;




   when CHIP_SELECT =>
		case sec_state is
		when SELECT_IDLE=>

		 		if from_config_in_empty='0' and next_state=CHIP_SELECT  then
				to_config_in_rreq<='1';
		 		next_sec_state<= SELECT_READ_IN;
		 		end if;
		 when SELECT_READ_IN=>
				 to_config_in_rreq<='1';
				data_config_in<=from_config_in;
		 		next_sec_state<= SELECT_SEND_OUT;
		 when SELECT_SEND_OUT=>
		 --to_config_out<="00000000000000000011";
				  if	next_sec_state/= sec_state then
			   		 to_config_out_wreq_r<='0';	
			else
				 to_config_in_rreq<='0';
				 if    data_config_in(19 downto 15)="00001" then 
				 chip_selecting<=data_config_in(1 downto 0);
				 				 to_config_out_wreq_r<='1';
				 elsif data_config_in(19 downto 15)="00010" then
				 sub_chip_selecting<=data_config_in(7 downto 0);
								 to_config_out_wreq_r<='1';
				 elsif data_config_in(19 downto 15)="00011" then
				 sub_data_selecting<=CONV_INTEGER(data_config_in(4 downto 0));
								 to_config_out_wreq_r<='1';
				 elsif data_config_in(19 downto 15)="00100" then
				 bit_data_selecting<=data_config_in(3 downto 0);
								 to_config_out_wreq_r<='1';
				 elsif data_config_in(19 downto 15)="00101" then
				 command<=data_config_in(9 downto 0);
								 to_config_out_wreq_r<='1';
				end if;
				  next_sec_state<=SELECT_IDLE;
				  next_state<=  WAIT_COMMAND;
				  	  
				  end if;

		end case;
   when READSRAM =>

				  next_state<=  WAIT_COMMAND;
				  	  

   when READING =>

	if red_state="00" and from_addr="111111111111111101" then
			 red_state<="01";
	elsif red_state="01" and block_length="010000000000" then
				red_state<="00";
				next_state<=  WAIT_COMMAND;
				to_config_out_wreq_r<='0';
	end if;
--	if red_state="01" and to_config_out_wreq_r='1' then
--				to_config_out_wreq_r<='0';
--	end if;
end case;


end if;
end process;
process(dataclock)
begin
if	red_state="00" then
 				 block_length<=(others=>'0');
elsif  dataclock='1' and dataclock'event then

			block_length<=block_length+1;
end if;
end process;
----------------------------------------------------------------------------------------------
config_in: dfifo 
	port map (
	din =>fifo_w,
	rd_clk	=>clk,
	rd_en=>to_config_in_rreq,
	rst  =>not reset_n,
	wr_clk	=>avalon_clk,
	wr_en	=>fifo_w_req,
	dout=>from_config_in,
	empty=>from_config_in_empty,
	full =>wrfull
);	

config_out:myfifo
port map(
		data		=>to_config_out,
		wrreq		=>to_config_out_wreq,
		rdreq		=>fifo_r_req,
		rdclk		=>avalon_clk,
		wrclk		=>myclock,
		aclr		=>not reset_n,
		q			=>fifo_r,
		rdempty		=>fifo_r_empty,
		wrfull		=>from_config_out_full
);
end Behavioral;
